/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<iostream>
#include<iomanip>
#include"../board/cposition.h"
#include"../movegen/movegenerator.h"
#include"../common/magicmoves.h"
#include"../eval/evaluation.h"
#include"../hashtable/transposition.h"
#include"../search/analyze.h"
#include"../search/timer.h"
using namespace std;

//-------------------------------------------------------
//
//           unittest Analysis:
//

void test_analyse()
{
  bool r=true;
  cout << "Test Analysis::analyse() .............................." << endl;
  
  ChessPosition cp;
  TranspositionTable tt;
  ThreadPool tp(0,tt);
  Analysis ana(cp,tt,tp);
  long long nps,nodes,time,eval,depth{12};
  float ebf;

  
  
  //*
  //-------------------------- Special positions -------------------------------
  {
    Timer tim;
    vector<pair<string,string>> s;

    s.push_back({"rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1","e2e4 d2d4 g1f3 b1c3"});// startpos
  
    auto do_test_position=[&](pair<string,string> s){
      cp.fen(s.first);
      ana.init(cp);    
      ana.set_hash_size(128);
      ana.clear_hash();
      ana.clear_history();
      ana.set_wtime(60000);
      ana.set_btime(60000);
      ana.set_movestogo(1);
      ana.set_depth(depth);
      tim.reset();
      tim.start();
      ana.iterative_search();
      tim.stop();
      time=tim.total_time().centiseconds();
      nodes=ana.get_node_count();
      if(time > 0){nps=nodes*100/time;}else{nps=0;}
      ebf=ana.get_ebf();
      eval=ana.eval();
      string solution=ana.best_move().algebraic();
      cout << "position: " << s.first << "\t"
      << "  depth=" << depth
      << "  nps=" << nps 
      << "  nodes=" << nodes 
      << "  time=" << setprecision(4) << time/100.0
      << "  ebf=" << setprecision(2) << ebf
      << "  eval=" << setprecision(2) << eval
      << "  bestmove " << solution
      << (s.second.find(solution)==string::npos?"  FAIL ":"  OK ")
      << endl;
      if(s.second.find(solution)==string::npos){r=false;}
    };

    for(auto si : s){do_test_position(si);}
  }
  //*/


  if(r){cout << "OK" << endl;}
  else{cout << "FAIL" << endl;}
}






