/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<iostream>
#include<iomanip>
#include<sstream>
#include<fstream>
#include<string>
#include<thread>
#include<stdexcept>
#include"../search/analyze.h"
#include"../communicator.h"
#include"../hashtable/transposition.h"
#include"../engine.h"
#include"../xbengine.h"
using namespace std;

//-------------------------------------------------------
//
//           unittest CECP 
//

void test_cecp()
{
  cout << "Test CECP ............................................." << flush;
  bool r=true;


  ChessPosition cp;
  Communicator com;
  ofstream ofs;
  Engine eng(com,ofs);
  XboardEngine xbe(com,ofs);
  Timer tim;
  int t1,t2;

  //--------------------------------- test 1 -----------------------------------------------
  //
  //cout << endl << "test1: " << endl;
  xbe.from_console("new");
  xbe.from_console("post");
  xbe.from_console("level 100 100 0");
  xbe.from_console("setboard rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1");
  xbe.from_console("time 10000");
  xbe.from_console("go");
  tim.start();

  while(true){
    eng.from_console();
    eng.to_console();
    string answer=com.from_engine();
    if(answer.size()>0){
      //xbe.to_console(answer);
      if(answer.find("move")!=string::npos){break;}
    }
    std::this_thread::sleep_for(std::chrono::milliseconds(50));
  }
  tim.stop();
  t1=tim.total_time().centiseconds();
  //----------------------------------------------------------------------------------------

  
  //--------------------------------- test 2 -----------------------------------------------
  //
  //cout << endl << "test2: " << endl;
  tim.reset();
  xbe.from_console("new");
  xbe.from_console("post");
  xbe.from_console("level 100 100 0");
  xbe.from_console("setboard rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 99");
  xbe.from_console("time 10000");
  xbe.from_console("go");
  tim.start();

  while(true){
    eng.from_console();
    eng.to_console();
    string answer=com.from_engine();
    if(answer.size()>0){
      //xbe.to_console(answer);
      if(answer.find("move")!=string::npos){break;}
    }
    std::this_thread::sleep_for(std::chrono::milliseconds(50));
  }
  tim.stop();
  t2=tim.total_time().centiseconds();
  //----------------------------------------------------------------------------------------

  //cout << "t1=" << t1 << "  t2=" << t2 << endl;
  if(t2 < 10*t1){r=false;}


  
  //--------------------------------- test 3 -----------------------------------------------
  //
  //cout << endl << "test3: " << endl;
  tim.reset();
  xbe.from_console("new");
  xbe.from_console("post");
  xbe.from_console("level 15 60 0");
  xbe.from_console("force");
  xbe.from_console("usermove e2e4");
  xbe.from_console("usermove c7c5");
  xbe.from_console("usermove g1f3");
  xbe.from_console("usermove d7d6");
  xbe.from_console("usermove d2d4");
  xbe.from_console("usermove c5d4");
  xbe.from_console("usermove f3d4");
  xbe.from_console("usermove g7g6");
  xbe.from_console("usermove c1e3");
  xbe.from_console("usermove f8g7");
  xbe.from_console("usermove b1c3");
  xbe.from_console("usermove g8f6");
  xbe.from_console("usermove f1e2");
  xbe.from_console("usermove e8g8");
  xbe.from_console("usermove e1g1");
  xbe.from_console("usermove b8c6");
  xbe.from_console("usermove d4b3");
  xbe.from_console("usermove b7b6");
  xbe.from_console("usermove f2f4");
  xbe.from_console("usermove c8b7");
  xbe.from_console("usermove e2f3");  
  xbe.from_console("usermove d8c7");
  xbe.from_console("usermove d1e2");  
  xbe.from_console("usermove a8c8");
  xbe.from_console("time 6000");
  xbe.from_console("go");
  tim.start();

  while(true){
    eng.from_console();
    eng.to_console();
    string answer=com.from_engine();
    if(answer.size()>0){
      //xbe.to_console(answer);
      if(answer.find("move")!=string::npos){break;}
    }
    std::this_thread::sleep_for(std::chrono::milliseconds(50));
  }
  tim.stop();
  t1=tim.total_time().seconds();
  if(t1<7){r=false;cout << "force mode bug   t1=" << t1 << endl;}
  //----------------------------------------------------------------------------------------




  xbe.from_console("quit");
  eng.from_console();

  if(r){cout << "OK" << endl;}else{cout << "FAIL" << endl;}
  
}






