/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<iostream>
#include<iomanip>
#include<sstream>
#include<thread>
#include"../board/cposition.h"
#include"../movegen/mlist.h"
#include"../hashtable/transposition.h"
#include"../egt/egt4pieces.h"
using namespace std;

//-------------------------------------------------------
//
//           unittest kbnk endgame table:
//

string longest_line(Egt4Pieces&);


void test_egt4pieces()
{
  cout << "Test Egt4Pieces ......................................." << flush;
  bool r=true;

  ChessPosition cp;
  vector<Move> ml;
  Timer tim;

  
  //-----------------------------------------------------------------------
  //
  /*
  Egt4Pieces egt4,egt5;
  egt4.load("../egt/kqkr_v2.egt");
  if(egt4.is_consistent()){std::cerr << "EGT4 is consistent" << endl;}else{std::cerr << "EGT4 is NOT consistent" << endl;}
  egt5.load("../egt/kqkr_v3.egt");
  if(egt5.is_consistent()){std::cerr << "EGT5 is consistent" << endl;}else{std::cerr << "EGT5 is NOT consistent" << endl;}

  unsigned diff_key=egt4.diff(egt5);
  std::cerr << "egt4.diff(egt5)=" << diff_key << endl;
  std::cerr << "egt4[" << diff_key << "]=" << egt4[diff_key] << endl;
  std::cerr << "egt5[" << diff_key << "]=" << egt5[diff_key] << endl;
  return;
  */
  //
  //-----------------------------------------------------------------------


  
  {
    tim.start();
    Egt4Pieces egt;
    for(int i=0;i<500000;i++){egt.create();}
    tim.stop();
  
    //------ fen: -------------------------------------------------------------------------
    string s="7k/8/8/8/8/8/8/5NBK w - - 0 1";
    egt.fen(s);
    string s1=egt.fen();
    if(s1!=s){r=false;}  


    //------ reverse movegenerator: -------------------------------------------------------
    //
    s="8/4k3/8/B7/3N4/8/4K3/8 w - - 0 1";
    egt.fen(s);
    egt.generate_previous_moves(ml);
    if(ml.size()!=8){r=false;}

    //
    s="8/4k3/8/B7/3N4/8/4K3/8 b - - 0 1";
    egt.fen(s);
    egt.generate_previous_moves(ml);
    if(ml.size()!=18){r=false;}

    //
    s="8/4k1K1/8/B7/3N4/8/8/8 b - - 0 1";
    egt.fen(s);
    egt.generate_previous_moves(ml);
    if(ml.size()!=16){r=false;}

    //
    s="8/4k1K1/8/B7/3N4/8/8/8 w - - 0 1";
    egt.fen(s);
    egt.generate_previous_moves(ml);
    if(ml.size()!=5){r=false;}

    if(!r){std::cerr << "test reverse movegenerator failed" << endl;}
    //
    //-------------------------------------------------------------------------------------


    //------ make move: -------------------------------------------------------------------
    //
    {
      s="8/8/8/3NkB2/8/8/4K3/8 b - - 0 1";
      egt.fen(s);
      string s1=egt.get_key_str();
      egt.generate_moves(ml);
      for(auto m : ml){
	egt.make_move(m);
	string s2=egt.get_key_str();
	egt.unmake_move(m);
	string s3=egt.get_key_str();
	if(s1!=s3){r=false;}
      }
    }
    if(!r){std::cerr << "test make move failed" << endl;}

    
    //------ check legality of the positions: ---------------------------------------------
    //
    s="8/4k3/8/B7/3N4/8/4K3/8 w - - 0 1";
    egt.fen(s);
    if(!egt.is_legal()){
      r=false;
    }

    s="8/4kK3/8/B7/3N4/8/8/8 w - - 0 1";
    egt.fen(s);
    if(egt.is_legal()){
      r=false;
    }

    s="8/4kK3/8/B7/3N4/8/8/8 b - - 0 1";
    egt.fen(s);
    if(egt.is_legal()){
      r=false;
    }

    s="8/2k5/8/B7/3N4/8/4K3/8 w - - 0 1";
    egt.fen(s);
    if(egt.is_legal()){
      r=false;
    }

    s="8/2k5/8/B7/3N4/8/4K3/8 b - - 0 1";
    egt.fen(s);
    if(!egt.is_legal()){
      r=false;
    }

    s="8/8/4k3/B7/3N4/8/4K3/8 w - - 0 1";
    egt.fen(s);
    if(egt.is_legal()){
      r=false;
    }

    s="8/8/4k3/B7/3N4/8/4K3/8 b - - 0 1";
    egt.fen(s);
    if(!egt.is_legal()){
      r=false;
    }
    if(!r){std::cerr << "test check legality failed" << endl;}


    //------ initial eval: ----------------------------------------------------------------
    //
    s="8/8/3Kk3/8/8/8/8/6BN w - - 0 1";
    int eval=egt.eval(s);
    if(eval!=0xfe){r=false;}
    if(!r){std::cerr << "test1 initial eval failed" << endl;}

    s="8/8/2K1k3/8/8/8/8/6B1 w - - 0 1";
    eval=egt.eval(s);
    if(eval!=0x00){r=false;}
    if(!r){std::cerr << "test2 initial eval failed" << endl;}
  
    s="8/8/2K1k3/8/8/8/8/6N1 w - - 0 1";
    eval=egt.eval(s);
    if(eval!=0x00){r=false;}
    if(!r){std::cerr << "test3 initial eval failed" << endl;}

    s="8/8/2K1k3/8/8/8/8/6n1 w - - 0 1";
    eval=egt.eval(s);
    if(eval!=0x00){r=false;}
    if(!r){std::cerr << "test4 initial eval failed" << endl;}

    s="7k/4N3/6K1/8/3B4/8/8/8 b - - 0 1";
    eval=egt.eval(s);
    if(eval!=30000){r=false;}
    if(!r){std::cerr << "test5 initial eval failed: eval=" << eval << endl;}

    s="7K/4n3/6k1/8/3b4/8/8/8 w - - 0 1";
    eval=egt.eval(s);
    if(eval!=-30000){r=false;} 
    if(!r){std::cerr << "test6 initial eval failed: eval=" << eval << endl;}


    //--------------------------------- Mate in ... ---------------------------------------
    //  
    s="6k1/6B1/6K1/5N2/8/8/8/8 b - - 0 1";
    egt.fen(s);
    int key=egt.get_key();
    if(egt[key]!=0xFD){r=false;}
    if(!r){std::cerr << "test1 mate in... failed" << endl;}

    s="7k/4N1B1/6K1/8/8/8/8/8 b - - 0 1";
    egt.fen(s);
    key=egt.get_key();
    if(egt[key]!=0){r=false;}
    if(!r){std::cerr << "test2 mate in... failed" << endl;}

    s="7k/4N3/6KB/8/8/8/8/8 w - - 0 1";
    egt.fen(s);
    key=egt.get_key();
    if(egt[key]!=1){r=false;}
    if(!r){std::cerr << "test3 mate in... failed" << endl;}
  
    s="6k1/4N3/6KB/8/8/8/8/8 b - - 0 1";
    egt.fen(s);
    key=egt.get_key();
    if(egt[key]!=2){r=false;}
  
    s="6k1/8/6KB/5N2/8/8/8/8 w - - 0 1";
    egt.fen(s);
    key=egt.get_key();
    if(egt[key]!=3){r=false;}
    if(!r){std::cerr << "test mate in... failed" << endl;}

  
    //----------------------  Consistency: ----------------------------------
    //
    r=egt.is_consistent();
    if(!r){std::cerr << "test consistency failed" << endl;}
    //
    //-----------------------------------------------------------------------

    //cout << "Longest line:" << endl;
    //cout << longest_line(egt);

    //---------------------- stats: -----------------------------------------
    //
    /*
    cout << egt.stats() << endl;
    cout << "  creation time: " << tim.total_time().milliseconds() << "ms " << endl;
    */
    //
    //-----------------------------------------------------------------------
  }



  
  //---------------------- KQKR: ------------------------------------------
  Egt4Pieces egt2(white_queen_id,black_rook_id);
  tim.reset();
  tim.start();
  for(int i=0;i<500000;i++){egt2.create();}
  tim.stop();
  
    
  //------ Symmetry problem: ------------------------------------------------------------
  {
    string s1="8/8/8/1Q6/8/k7/r7/K7 w - - 0 1";
    egt2.fen(s1);
    int ev1=egt2.eval();

    s1="8/8/8/8/8/8/4Q3/Krk5 w - - 0 1";
    egt2.fen(s1);
    int ev2=egt2.eval();

    if(ev1!=ev2){r=false;}
  }
  
  //------ fen: -------------------------------------------------------------------------
  {
    string s1="7k/8/8/8/8/8/8/5QrK w - - 0 1";
    egt2.fen(s1);
    string s2=egt2.fen();
    //std::cerr << s2 << endl;
    if(s1!=s2){r=false;}  
    string s3="7K/8/8/8/8/8/8/5qRk b - - 0 1";
    egt2.fen(s3);
    string s4=egt2.fen();
    //std::cerr << s4 << endl;
    if(s1!=s4){r=false;}  
  }

  //------ key: -------------------------------------------------------------------------
  {
    string s1="8/8/6Q1/8/8/2K5/8/k7 b - - 0 1";
    egt2.fen(s1);
    if(egt2.get_key_str()!="K: c3   k: a1   Q: g6   r: a1   (black)"){r=false;}
  }

  //------ Movegenerator: ---------------------------------------------------------------
  {
    string s1="4k3/3r4/8/8/8/8/3QK3/8 w - - 0 1";
    egt2.fen(s1);
    egt2.generate_moves(ml);
    if(ml.size()!=25){r=false;}

    s1="4k3/3r4/8/8/8/8/3QK3/8 b - - 0 1";
    egt2.fen(s1);
    egt2.generate_moves(ml);
    if(ml.size()!=17){r=false;}

    s1="8/6Q1/6r1/8/8/2K5/8/k7 w - - 0 1";
    egt2.fen(s1);
    egt2.generate_moves(ml);
    if(ml.size()!=22){r=false;}
  }

  //------ Reverse movegenerator: -------------------------------------------------------
  {
    string s1="8/5k2/3r4/K7/8/3Q4/8/8 b - - 0 1";
    egt2.fen(s1);
    egt2.generate_previous_moves(ml);
    if(ml.size()!=19){r=false;}

    s1="8/5k2/3r4/K7/8/3Q4/8/8 w - - 0 1";
    egt2.fen(s1);
    egt2.generate_previous_moves(ml);
    if(ml.size()!=17){r=false;}

    s1="8/5k2/8/K7/8/3Q4/8/8 b - - 0 1";
    egt2.fen(s1);
    egt2.generate_previous_moves(ml);
    if(ml.size()!=42){r=false;}

    s1="8/5k2/3r4/K7/8/8/8/8 w - - 0 1";
    egt2.fen(s1);
    egt2.generate_previous_moves(ml);
    if(ml.size()!=40){r=false;}

    s1="8/8/8/8/8/8/1Q6/k1K5 b - - 0 1";
    egt2.fen(s1);
    egt2.generate_previous_moves(ml);
    if(ml.size()!=42){r=false;}

    s1="8/8/8/8/8/k1K5/2Q5/8 w - - 0 1";
    egt2.fen(s1);
    egt2.generate_previous_moves(ml);
    if(ml.size()!=2){r=false;}
  }
  

  
  //------ Mate in ...: -------------------------------------------------------------------
  {
    string s1="7r/8/5k1K/8/8/8/8/8 w - - 0 1";
    egt2.fen(s1);
    if(egt2.eval()!=-30000){r=false;}

    s1="7Q/8/5K1k/8/8/8/8/8 b - - 0 1";
    egt2.fen(s1);
    if(egt2.eval()!=30000){r=false;}

    //---
    s1="8/6Q1/6r1/8/8/2K5/8/k7 w - - 0 1";
    egt2.fen(s1);
    if(egt2.eval()!=29995){r=false;}

    s1="8/8/6Q1/8/8/2K5/8/k7 b - - 0 1";
    egt2.fen(s1);
    if(egt2.eval()!=29996){r=false;}

    s1="8/8/6Q1/8/8/2K5/k7/8 w - - 0 1";
    egt2.fen(s1);
    if(egt2.eval()!=29997){r=false;}

    s1="8/8/8/8/8/2K5/k1Q5/8 b - - 0 1";
    egt2.fen(s1);
    if(egt2.eval()!=29998){r=false;}

    s1="8/8/8/8/8/k1K5/2Q5/8 w - - 0 1";
    egt2.fen(s1);
    if(egt2.eval()!=29999){r=false;}

    s1="8/8/8/8/8/kQK5/8/8 b - - 0 1";
    egt2.fen(s1);
    if(egt2.eval()!=30000){r=false;}
    //---
    
  }

  {
    //----------------------  Memory management: ----------------------------
    //
    Egt4Pieces egt;
    unsigned char* pm=new unsigned char[8000000];
    egt.fen("k7/2K5/8/1N1B4/8/8/8/8 b - - 0 1");
    if(egt.eval()!=0xFF){cout << "1)  k7/2K5/8/1N1B4/8/8/8/8 b - - 0 1    eval=" << egt.eval() << endl;r=false;}
    egt.create();
    if(egt.eval()!=0x00){cout << "2)  k7/2K5/8/1N1B4/8/8/8/8 b - - 0 1    eval=" << egt.eval() << endl;r=false;}
    egt.clear();
    if(egt.eval()!=0xFF){cout << "3)  k7/2K5/8/1N1B4/8/8/8/8 b - - 0 1    eval=" << egt.eval() << endl;r=false;}
    egt.set_memory(pm);
    if(egt.eval()!=0xFF){cout << "4)  k7/2K5/8/1N1B4/8/8/8/8 b - - 0 1    eval=" << egt.eval() << endl;r=false;}
    egt.create();
    if(egt.eval()!=0x00){cout << "5)  k7/2K5/8/1N1B4/8/8/8/8 b - - 0 1    eval=" << egt.eval() << endl;r=false;}
    delete[] pm;

    TranspositionTable tt;
    Egt4Pieces egtkqkr{white_queen_id,black_rook_id};

    tt.set_threshold(6000000);
    egtkqkr.set_memory(tt.get_uchar_adress());
    while(!egtkqkr.ready()){
      egtkqkr.create();
    }
    tt.clear();
    if(egt2!=egtkqkr){std::cerr << "TT corrupts EGT" << endl;r=false;}
  }
  
  //----------------------  Consistency: ----------------------------------
  //
  r=egt2.is_consistent();
  if(!r){std::cerr << "test consistency failed" << endl;}
  //
  //-----------------------------------------------------------------------

  
  
  
  //cout << egt2.stats() << endl;
  //
  //-----------------------------------------------------------------------

  //--------------------- save egt: ---------------------------------------
  //
  /*
  egt2.save("test_egt_kqkr.txt");
  Egt4Pieces egt3;
  egt3.load("test_egt_kqkr.txt");
  if(egt3.is_consistent()){std::cerr << "EGT3 is consistent" << endl;}else{std::cerr << "EGT3 is NOT consistent" << endl;}
  if(egt2==egt3){
    std::cerr << "EGT's equal" << endl;
  }
  */
  //
  //-----------------------------------------------------------------------


  
  
  
  if(r){cout << "OK";}else{cout << "FAIL";}
  cout << "  creation time: " << tim.total_time().milliseconds() << "ms " << endl;
}









    
//----------------------  Longest line: ---------------------------------
//
string longest_line(Egt4Pieces& egt)
{
  ostringstream oss;
  vector<Move> ml;
  int key_longest{0};
  int eval_longest{30000};
  for(unsigned i=0;i<egt.size();i++){
    if(egt.is_legal_key(i)){
      egt.set_key(i);
      if(egt.eval()>0 && egt.eval()!=0xFE && egt.eval()<eval_longest){
	eval_longest = egt.eval();
	key_longest=i;
      }
    }
  }
    
  egt.set_key(key_longest);
  std::cerr << egt.fen() << "  eval=" << egt.eval() << endl;

  int cnt{0};
  while(true){
    int ev=egt.eval();
    egt.generate_moves(ml);
    for(auto m : ml){
      egt.make_move(m);
      if(egt.eval()==ev+1){
	oss << m.algebraic() << "(" << egt.eval() << ") " << flush;
	break;
      }
      egt.unmake_move(m);
    }

    if(egt.eval()==30000){break;}
    if(++cnt>140){
      std::cerr << endl << endl;
      std::cerr << "ERROR:  current position: " << egt.fen() << "  current eval=" << egt.eval() << "  current key=" << egt.get_key()
		<< (egt.is_legal_key(egt.get_key())?" legal ":" illegal ") << endl;
      std::cerr << egt.fen() << "  eval=" << egt.eval() << "  key=" << egt.get_key() << endl;
      egt.generate_moves(ml);
      for(auto m : ml){
	egt.make_move(m);
	std::cerr << m.algebraic() << "(" << egt.eval() << ") ";
	egt.unmake_move(m);
      }
      std::cerr << endl;
      break;
    }
  }
  oss << endl;
  return oss.str();
}
  


