/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<iostream>
#include<iomanip>
#include"../board/cposition.h"
#include"../movegen/movegenerator.h"
#include"../movegen/mlist.h"
#include"../search/analyze.h"
using namespace std;

//-------------------------------------------------------
//
//           unittest movelist:
//

void test_movelist()
{
  cout << "Test MList ............................................" << flush;
  bool r=true;

  ChessPosition cp;
  TranspositionTable tt;
  ThreadPool tp(4,tt);
  Analysis ana(cp,tt,tp);
  HistoryTable ht;

  {
    cp.newgame();
    cp.make_move(e2,e4);
    cp.make_move(e7,e5);
    cp.make_move(g1,f3);
    cp.make_move(b8,c6);
    cp.make_move(f1,b5);
    cp.make_move(a7,a6);

    Move hm(e1,h1,castling,WhiteKing(),NoPiece());
    Move ka(a2,a4,normal_move,WhitePawn(),NoPiece());
    Move kb(h2,h4,normal_move,WhitePawn(),NoPiece());
    Move ka_2(a2,a3,normal_move,WhitePawn(),NoPiece());
    Move kb_2(h2,h3,normal_move,WhitePawn(),NoPiece());

    MList ml(true);
    cp.generate_moves(ml);
    ml.set_hashtable_move(hm);
    ml.set_killers(ka,kb);
    ml.set_killers_2(ka_2,kb_2);
    ana.init(cp);
    ml.set_analysis(&ana);
    ml.sort();

    if(ml[0]!=hm){cout << "error 1" << endl;r=false;}
  }


  {
    cp.newgame();
    cp.make_move(e2,e4);
    cp.make_move(e7,e5);
    cp.make_move(g1,f3);
    cp.make_move(b8,c6);
    cp.make_move(f1,b5);
    cp.make_move(a7,a6);

    Move hm(f3,e5,normal_move,WhiteKnight(),BlackPawn());
    Move ka(b5,c6,normal_move,WhiteBishop(),BlackKnight());
    Move kb(b5,a6,normal_move,WhiteBishop(),BlackPawn());
    
    MList ml(true);
    cp.generate_moves(ml);
    ml.set_hashtable_move(hm);
    ml.set_killers(ka,kb);
    ana.init(cp);
    ml.set_analysis(&ana);
    ml.sort();

    if(ml[0]!=hm){cout << "error 2" << endl;r=false;}
  }

  {
    cp.newgame();
    cp.make_move(e2,e4);
    cp.make_move(e7,e5);
    cp.make_move(g1,f3);
    cp.make_move(b8,c6);
    cp.make_move(f1,b5);
    cp.make_move(a7,a6);
    cp.make_move(e1,g1);

    Move hm(g8,f6,normal_move,BlackKnight(),NoPiece());
    Move ka(f8,e7,normal_move,BlackBishop(),NoPiece());
    Move kb(f8,c5,normal_move,BlackBishop(),NoPiece());
    Move bm(e8,e7,normal_move,BlackKing(),NoPiece());
    
    MList ml(true);
    cp.generate_moves(ml);
    ml.set_hashtable_move(hm);
    ml.set_killers(ka,kb);
    ana.init(cp);
    ml.set_analysis(&ana);
    ml.sort();

    if(ml[0]!=hm){cout << "error 3" << endl;r=false;}
  }


  {
    cp.newgame();
    MList ml(true);
    cp.generate_moves(ml);
    vector<Move> sml;
    Move m1(a2,a3,normal_move,WhitePawn(),NoPiece());
    Move m2(f2,f4,normal_move,WhitePawn(),NoPiece());
    Move m3(g1,f3,normal_move,WhiteKnight(),NoPiece());
    sml.push_back(m1);
    sml.push_back(m2);
    sml.push_back(m3);
    ml.set_searchmoves(sml);
    if(ml.size()!=3){cout << "error 4" << endl;r=false;}
  }
  

  if(r){cout << "OK" << endl;}else{cout << "FAIL" << endl;}

}






