/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<iostream>
#include<iomanip>
#include<sstream>
#include<fstream>
#include<string>
#include<thread>
#include<stdexcept>
#include"../search/analyze.h"
#include"../communicator.h"
#include"../hashtable/transposition.h"
#include"../engine.h"
using namespace std;

//-------------------------------------------------------
//
//           unittest UCI: 
//

void test_uci()
{
  cout << "Test UCI .............................................." << flush;
  bool r=true;


  {
    ofstream ofs;
    Communicator com;
    Engine eng(com,ofs);
    std::this_thread::sleep_for(seconds(5));
    string commando,answer;  
    int pcnt{0};

    //--------------------- test1: ---------------------------------------------------------------------------------------------    
    //
    //cerr << "start test1" << endl;
    commando="position fen rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1 moves ";
    commando+="d2d4 e7e6 c2c4 f7f5 b1c3 g8f6 d1c2 f8b4 e2e3 b7b6 f1d3 e8g8 g1e2 c8b7 e1g1 b8c6 f2f3 b4d6 a2a3 a7a5 ";
    commando+="c3b5 d6e7 c1d2 b7c8 e3e4 f5e4 f3e4 e6e5 d4d5 e7c5 g1h1 c6e7 a1c1 f6g4 f1f8 d8f8 h2h3 g4f2 h1h2 f2d3 ";
    commando+="c2d3 c5d6 b5d6 c7d6 h2g1 a8b8 b2b4 a5b4 c1f1 f8e8 a3b4 e8g6 b4b5 h7h5 d3f3 c8b7 d2e3 g6e8 e2g3 b8c8 ";
    commando+="g3h5 c8c4 e3b6 c4b4 f3g4 e7g6 f1f6 b4b1 g1h2 g8h7 g4f5 b1f1 f5f1 g6f4 f6d6 e8h5 g2g3 h5f7 b6c7 g7g5 b5b6 b7c8";
    com.to_engine(commando);
    com.to_engine("isready");
    while(true){
      eng.from_console();
      eng.to_console();
      answer=com.from_engine();
      std::this_thread::sleep_for(std::chrono::milliseconds(50));
      if(answer.find("readyok")!=string::npos){break;}
    }
    commando="go wtime 62070 btime 64920 winc 0 binc 0 movestogo 39";
    com.to_engine(commando);

    pcnt=0;
    while(true){
      eng.from_console();
      eng.to_console();
      answer=com.from_engine();
      std::this_thread::sleep_for(std::chrono::milliseconds(50));
      if(answer.find("info")!=string::npos){continue;}
      if(answer.find("bestmove")!=string::npos){break;}
      if(++pcnt==10){com.to_engine("ponderhit");}
    }
    if(answer.find("bestmove")==string::npos){
      cerr << "error1   answer: " << answer << endl;r=false;
    }
    if(!r)cout << "error1" << endl;
    //cerr << "end test1   r=" << boolalpha << r << endl;

    //--------------------- test2: ---------------------------------------------------------------------------------------------    
    //
    //cerr << "start test2" << endl;
    commando="position fen k7/1q6/8/8/8/8/1Q6/K7 w - - 0 1 moves b2g7";
    com.to_engine(commando);
    com.to_engine("isready");
    commando="go wtime 62070 btime 64920 winc 0 binc 0 movestogo 39 ponder";
    com.to_engine(commando);

    answer="";
    pcnt=0;
    while(true){
      eng.from_console();
      eng.to_console();
      answer=com.from_engine();
      std::this_thread::sleep_for(std::chrono::milliseconds(50));
      if(answer.find("info")!=string::npos){continue;}
      if(answer.find("bestmove")!=string::npos){break;}
      if(++pcnt==10){com.to_engine("ponderhit");}
    }
    {
      istringstream iss(answer);
      string s;
      iss >> s;
      if(s!="bestmove"){cerr << "error2   answer=" << answer << endl;r=false;}
      iss >> s;
      if(s!="b7g7"){cerr << "error2b   answer=" << answer << endl;r=false;}
    }
    if(!r)cout << "error2" << endl;
    //cerr << "end test2" << endl;

    
    //--------------------- test3: ---------------------------------------------------------------------------------------------    
    //
    //cerr << "start test3" << endl;
    commando="position fen 8/8/8/8/kq6/8/1Q6/K7 w - - 0 1 moves b2d4";
    com.to_engine(commando);
    commando="go wtime 62070 btime 64920 winc 0 binc 0 movestogo 39 ponder";
    com.to_engine(commando);

    answer="";
    pcnt=0;
    while(true){
      eng.from_console();
      eng.to_console();
      answer=com.from_engine();
      std::this_thread::sleep_for(std::chrono::milliseconds(50));
      if(answer.find("info")!=string::npos){continue;}
      if(answer.find("bestmove")!=string::npos){break;}
      if(++pcnt==10){com.to_engine("ponderhit");}
    }
    if(answer.find("bestmove")!=string::npos){
      istringstream iss(answer);
      string s1,s2;
      iss >> s1 >> s2;
      if(s2.find("b4d4")==string::npos){cerr << "error3 " << s2 << endl;r=false;}
    }
    if(!r)cout << "error3" << endl;
    //cerr << "end test3" << endl;

    //--------------------- test4: ---------------------------------------------------------------------------------------------    
    //
    //cerr << "start test4" << endl;
    com.to_engine("ucinewgame");
    com.to_engine("isready");
    while(true){
      eng.from_console();
      eng.to_console();
      answer=com.from_engine();
      std::this_thread::sleep_for(std::chrono::milliseconds(50));
      if(answer.find("readyok")!=string::npos){break;}
    }
    commando="position fen 8/8/8/8/kq6/8/1Q6/K7 w - - 0 1 moves b2d4";
    com.to_engine(commando);
    commando="go infinite";
    com.to_engine(commando);

    answer="";
    pcnt=0;
    while(true){
      eng.from_console();
      eng.to_console();
      answer=com.from_engine();
      std::this_thread::sleep_for(std::chrono::milliseconds(50));
      if(answer.find("info")!=string::npos){continue;}
      if(answer.find("bestmove")!=string::npos){break;}
      if(++pcnt==10){com.to_engine("stop");}
    }
    if(answer.find("bestmove")!=string::npos){
      istringstream iss(answer);
      string s1,s2;
      iss >> s1 >> s2;
      if(s2!="b4d4"){cerr << "error4  " << s2 << endl;r=false;}
    }
    if(!r)cout << "error4" << endl;
    //cerr << "end test4" << endl;

    
    //--------------------- test5: ---------------------------------------------------------------------------------------------    
    //
    //cerr << "start test5" << endl;
    commando="position fen k7/pp5P/8/8/8/8/8/K7 w - - 0 1";
    com.to_engine(commando);
    commando="go depth 2";
    com.to_engine(commando);

    answer="";
    pcnt=0;
    while(true){
      eng.from_console();
      eng.to_console();
      answer=com.from_engine();
      std::this_thread::sleep_for(std::chrono::milliseconds(50));
      if(answer.find("info")!=string::npos){continue;}
      if(answer.find("bestmove")!=string::npos){break;}
      if(++pcnt==10){com.to_engine("stop");}
    }
    if(answer.find("bestmove")!=string::npos){
      istringstream iss(answer);
      string s1,s2;
      iss >> s1 >> s2;
      if(s2!="h7h8q" && s2!="h7h8r"){
	cerr << "error with promotion   s2=" << s2 << endl;r=false;
      }
    }
    if(!r)cout << "error5" << endl;
    //cerr << "end test5" << endl;


    //--------------------- test6: ---------------------------------------------------------------------------------------------    
    //
    //cerr << "start test6" << endl;
    commando="isready";
    com.to_engine(commando);
    while(com.from_engine()!="readyok"){
      eng.from_console();
      eng.to_console();
    }
    commando="position startpos";
    com.to_engine(commando);
    commando="go infinite searchmoves a2a4 h2h4";
    com.to_engine(commando);

    answer="";
    pcnt=0;
    while(true){
      eng.from_console();
      eng.to_console();
      answer=com.from_engine();
      std::this_thread::sleep_for(std::chrono::milliseconds(500));
      if(answer.find("info")!=string::npos){continue;}
      if(answer.find("bestmove")!=string::npos){break;}
      if(++pcnt==10){com.to_engine("stop");}
    }
    if(answer.find("bestmove a2a4")==string::npos && answer.find("bestmove h2h4")==string::npos){
      cerr << "error6   answer: " << answer << endl;r=false;
    }
    else{}
    if(!r)cout << "error6" << endl;
    //cerr << "end   test6" << endl;

    //--------------------- test7  test ponder mode: ---------------------------------------------------------------------------    
    // 
    //cerr << "start test7" << endl;
    commando="position fen R6k/6pp/4q3/4N3/8/8/8/7K b - - 0 1 moves e6g8";
    com.to_engine(commando);
    commando="isready";
    com.to_engine(commando);
    while(com.from_engine()!="readyok"){
      eng.from_console();
      eng.to_console();
    }
    commando="go ponder";
    com.to_engine(commando);

    std::this_thread::sleep_for(std::chrono::milliseconds(500));    
    commando="ponderhit";
    com.to_engine(commando);
    
    pcnt=0;
    while(true){
      eng.from_console();
      eng.to_console();
      answer=com.from_engine();
      std::this_thread::sleep_for(std::chrono::milliseconds(50));
      if(answer.find("info")!=string::npos){continue;}
      if(answer.find("bestmove")!=string::npos){break;}
      if(++pcnt==10){com.to_engine("stop");}
    }
    if(answer.find("e5f7")==string::npos){cerr << "error7b   answer: " << answer << endl;r=false;}
    //
    //--------------------------------------------------------------------------------------------------------------------------    
    if(!r)cout << "error7" << endl;
    //cerr << "end   test7" << endl;

    //--------------------- test8  test ponder mode: ---------------------------------------------------------------------------    
    //
    {
      //cerr << "start test8" << endl;
      commando="newgame";
      com.to_engine(commando);
      commando="position startpos moves";
      com.to_engine(commando);
      commando="isready";
      com.to_engine(commando);
      while(com.from_engine()!="readyok"){
	eng.from_console();
	eng.to_console();
      }
      commando="go movestime 1000";
      com.to_engine(commando);

      pcnt=0;
      while(true){
	eng.from_console();
	eng.to_console();
	answer=com.from_engine();
	//if(answer!=""){std::cerr << answer << endl;}
	std::this_thread::sleep_for(std::chrono::milliseconds(50));
	if(answer.find("info")!=string::npos){continue;}
	if(answer.find("bestmove")!=string::npos){break;}
	if(++pcnt==200){com.to_engine("stop");}
      }
      if(answer.find("ponder")==string::npos){r=false;cerr << "error 8 no pondermove" << endl;}
      if(answer.find("0000")!=string::npos){r=false;cerr << "error 8 pondermove==nullmove" << endl;}
    }    
    //
    //--------------------------------------------------------------------------------------------------------------------------    
    if(!r)cout << "error8" << endl;
    //cerr << "end   test8" << endl;
    
    
    com.to_engine("quit");
    eng.from_console();
  }
  


  if(r){cout << "OK" << endl;}else{cout << "FAIL" << endl;}
}






