/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#ifndef BPOSITION_H
#define BPOSITION_H

#include"../movegen/move.h"
#include"../common/bitboard.h"
#include<vector>
#include<string>
#include<fstream>
using std::ofstream;
using std::string;
using std::vector;

class Square;
class Piece;

class BoardPosition
{
public:
  //----------------constructor/destructor/assignment--------------------
  //
  BoardPosition();
  BoardPosition(const BoardPosition&);
  BoardPosition& operator=(const BoardPosition&);
  ~BoardPosition();
  //---------------------------------------------------------------------


  //----------------changing the board-----------------------------------
  //
  void clear();
  void fen(const string&);
  void set_piece_on(const Square&,int);
  void remove_piece_from(const Square&,int);
  void compute_pieces();
  void make_move(const Move&);
  void unmake_move(const Move&);
  void set_white_to_move();
  void set_black_to_move();
  void flip();
  void flop();
  void switch_color_to_move();
  //---------------------------------------------------------------------


  //--------------------------queries------------------------------------
  //
  int stm()const;
  bool white_to_move()const;
  bool black_to_move()const;
  bool corrupt(unsigned long long=0)const;

  bool operator==(const BoardPosition&)const;
  bool operator!=(const BoardPosition&)const;

  const BitBoard& white_pawn()const;
  const BitBoard& white_knight()const;
  const BitBoard& white_bishop()const;
  const BitBoard& white_rook()const;
  const BitBoard& white_queen()const;
  const BitBoard& white_king()const;
  const BitBoard& black_pawn()const;
  const BitBoard& black_knight()const;
  const BitBoard& black_bishop()const;
  const BitBoard& black_rook()const;
  const BitBoard& black_queen()const;
  const BitBoard& black_king()const;
  const BitBoard& piece(int)const;
  BitBoard white_pieces()const;
  BitBoard black_pieces()const;
  BitBoard pieces()const;
  const int& operator[](const int& )const;
  
  //

  //-------------------------- logging: ---------------------------------
  //
  string dump()const;
  bool consistent()const;

private:
  BitBoard m_white_pieces,m_black_pieces,m_pieces;
  vector<BitBoard> m_piece;
  array<int,64> m_board;
  unsigned int m_stm;
};



#endif
