/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"bkrecord.h"


Book_record::Book_record()
{
  for(int i=0;i<8;i++){key[i]=0;}
  for(int i=0;i<2;i++){move[i]=weight[i]=0;}
  for(int i=0;i<4;i++){learn[i]=0;}
}

Book_record::Book_record(char* buffer)
{
  for(int i=0;i<8;i++){key[i]=buffer[i];}
  for(int i=0;i<2;i++){move[i]=buffer[i+8];}
  for(int i=0;i<2;i++){weight[i]=buffer[i+10];}
  for(int i=0;i<4;i++){learn[i]=buffer[i+12];}
}


bool Book_record::operator<(const Book_record& x)const
{
  if(key[0]>x.key[0]){return false;}
  if(key[0]<x.key[0]){return true;}
  if(key[1]>x.key[1]){return false;}
  if(key[1]<x.key[1]){return true;}
  if(key[2]>x.key[2]){return false;}
  if(key[2]<x.key[2]){return true;}
  if(key[3]>x.key[3]){return false;}
  if(key[3]<x.key[3]){return true;}
  if(key[4]>x.key[4]){return false;}
  if(key[4]<x.key[4]){return true;}
  if(key[5]>x.key[5]){return false;}
  if(key[5]<x.key[5]){return true;}
  if(key[6]>x.key[6]){return false;}
  if(key[6]<x.key[6]){return true;}
  if(key[7]>x.key[7]){return false;}
  if(key[7]<x.key[7]){return true;}
  return false;
}

bool Book_record::operator==(const Book_record& x)const
{
  if(key[0]!=x.key[0]){return false;}
  if(key[1]!=x.key[1]){return false;}
  if(key[2]!=x.key[2]){return false;}
  if(key[3]!=x.key[3]){return false;}
  if(key[4]!=x.key[4]){return false;}
  if(key[5]!=x.key[5]){return false;}
  if(key[6]!=x.key[6]){return false;}
  if(key[7]!=x.key[7]){return false;}
  return true;
}

bool Book_record::operator<=(const Book_record& x)const
{
  if(*this==x){return true;}
  if(*this<x){return true;}
  return false;
}

bool Book_record::operator>=(const Book_record& x)const
{
  if(*this<x){return false;}else{return true;}
}

bool Book_record::operator>(const Book_record& x)const
{
  if(*this<=x){return false;}else{return true;}
}

void Book_record::set_key(uint64_t x)
{
  key[0]=x>>56 & 0xFF;
  key[1]=x>>48 & 0xFF;
  key[2]=x>>40 & 0xFF;
  key[3]=x>>32 & 0xFF;
  key[4]=x>>24 & 0xFF;
  key[5]=x>>16 & 0xFF;
  key[6]=x>>8  & 0xFF;
  key[7]=x     & 0xFF;
}

void Book_record::set_move(const BKmove& x)
{
  Square from,to;
  short int code;
  short int m{0};
  from=x.from();
  to=x.to();
  code=x.code();
  m=code<<12;
  m+=from.rank_number()<<9;
  m+=from.file_number()<<6;
  m+=to.rank_number()<<3;
  m+=to.file_number();
  move[0]=m>>8&0xFF;
  move[1]=m&0xFF;
}

void Book_record::set_weight(uint16_t x)
{
  weight[0]=x>>8 & 0xFF;
  weight[1]=x & 0xFF;
}

void Book_record::set_learn(uint32_t x)
{
  learn[0]=x>>24 & 0xFF;
  learn[1]=x>>16 & 0xFF;
  learn[2]=x>>8 & 0xFF;
  learn[3]=x & 0xFF;
}

uint64_t Book_record::get_key()const
{
  uint64_t r{0};
  r+=key[0];r<<=8;
  r+=key[1];r<<=8;
  r+=key[2];r<<=8;
  r+=key[3];r<<=8;
  r+=key[4];r<<=8;
  r+=key[5];r<<=8;
  r+=key[6];r<<=8;
  r+=key[7];
  return r;
}

BKmove Book_record::get_move()const
{
  BKmove ttmove;
  Square from,to;
  char code;
  uint16_t m{0};
  m=(move[0]<<8)+move[1];

  from=Square(m>>6 & 0x3F);
  to=Square(m & 0x3F);
  code=m>>12 & 0xF;
  ttmove.set_from(from);
  ttmove.set_to(to);
  switch(code)
    {
    case 4: ttmove.set_code(4);break;
    case 3: ttmove.set_code(3);break;
    case 2: ttmove.set_code(2);break;
    case 1: ttmove.set_code(1);break;
    }

  return ttmove;
}


uint16_t Book_record::get_weight()const
{
  uint16_t r;
  r=weight[0];r<<=8;
  r+=weight[1];
  return r;
}

uint32_t Book_record::get_learn()const
{
  uint32_t r{0};
  r=learn[0];r<<=8;
  r+=learn[1];r<<=8;
  r+=learn[2];r<<=8;
  r+=learn[3];
  return r;
}

