/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#ifndef BITBOARD_H
#define BITBOARD_H

#include"../common/square.h"
#include<vector>
#include<initializer_list>
#include<cstdint>
using std::vector;
using std::initializer_list;
using std::uint16_t;
using std::uint32_t;
using std::uint64_t;


class BitBoard
{
 private:
  uint64_t bb{0};
  const static array<char,128> m_first_square_id;
  const static array<char,1<<16> m_popcnt16;
 
 public:
  BitBoard();
  BitBoard(const BitBoard&);
  explicit BitBoard(const uint64_t&);
  BitBoard(initializer_list<Square>);
  static const uint64_t& to_ull(const BitBoard&);
  void clear();
  void set_square(const Square&);
  void reset_square(const Square&);
  void flip();
  void flop();
  Square get_first_square()const;
  Square get_unique_square()const;
  BitBoard get_reset_square();
  vector<Square> get_squares()const;
  int popcount()const;
  
  bool operator<(BitBoard)const;
  bool operator>(BitBoard)const;
  bool operator==(BitBoard)const;
  bool operator!=(BitBoard)const;
  operator bool()const;
  
  BitBoard& operator&=(const BitBoard&);
  BitBoard& operator|=(const BitBoard&);
  BitBoard& operator^=(const BitBoard&);
  BitBoard& operator&=(const unsigned long long);
  BitBoard& operator|=(const unsigned long long);
  BitBoard& operator^=(const unsigned long long);
  BitBoard& operator>>=(int);
  BitBoard& operator<<=(int);
  BitBoard operator~()const;
  void write()const;
  string dump()const;

  friend BitBoard operator&(const BitBoard&,const BitBoard&);
  friend BitBoard operator|(const BitBoard&,const BitBoard&);
  friend BitBoard operator^(const BitBoard&,const BitBoard&);
  friend BitBoard operator<<(const BitBoard&,int);
  friend BitBoard operator>>(const BitBoard&,int);
};

BitBoard operator&(const BitBoard&,const BitBoard&);
BitBoard operator|(const BitBoard&,const BitBoard&);
BitBoard operator^(const BitBoard&,const BitBoard&);
BitBoard operator<<(const BitBoard&,int);
BitBoard operator>>(const BitBoard&,int);

#endif
