/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"connections.h"
#include<iostream>
using std::cout;
using std::endl;


void Connections::init_squares()
{
  m_square.clear();
  BitBoard square;
  for(int i=0;i<64;i++)
    {
      square.clear();
      square.set_square(i);
      m_square.push_back(square);
    }
}

void Connections::init_w_pawn_attacks()
{
  int square2;
  for(int square1=a1.id();square1<=h8.id();square1++)
    {
      square2=square1+7;
      if(square2<=h8.id() && (square2&0x07)<(square1&0x07))
	{
	  mw_pawn_attacks.at(square1).set_square(square2);
	}
      square2=square1+9;
      if(square2<=h8.id() && (square2&0x07)>(square1&0x07))
	{
	  mw_pawn_attacks.at(square1).set_square(square2);
	}
    }
}

void Connections::init_b_pawn_attacks()
{
  int square2;
  for(int square1=a1.id();square1<=h8.id();square1++)
    {
      square2=square1-7;
      if(square2>=a1.id() && (square2&0x07)>(square1&0x07))
	{
	  mb_pawn_attacks[square1].set_square(square2);
	}
      square2=square1-9;
      if(square2>=a1.id() && (square2&0x07)<(square1&0x07))
	{
	  mb_pawn_attacks[square1].set_square(square2);
	}
    }
}

void Connections::init_knight_attacks()
{
  int square2;
  for(int square1=a1.id();square1<=h8.id();square1++)
    {
      square2=square1+10;
      if(square2<64 && (square2&0x07)>(square1&0x07))
	{
	  mknight_attacks[square1].set_square(square2);
	}
      square2=square1+17;
      if(square2<64 && (square2&0x07)>(square1&0x07))
	{
	  mknight_attacks[square1].set_square(square2);
	}
      square2=square1+15;
      if(square2<64 && (square2&0x07)<(square1&0x07))
	{
	  mknight_attacks[square1].set_square(square2);
	}
      square2=square1+6;
      if(square2<64 && (square2&0x07)<(square1&0x07))
	{
	  mknight_attacks[square1].set_square(square2);
	}
      square2=square1-10;
      if(square2>=0 && (square2&0x07)<(square1&0x07))
	{
	  mknight_attacks[square1].set_square(square2);
	}
      square2=square1-17;
      if(square2>=0 && (square2&0x07)<(square1&0x07))
	{
	  mknight_attacks[square1].set_square(square2);
	}
      square2=square1-15;
      if(square2>=0 && (square2&0x07)>(square1&0x07))
	{
	  mknight_attacks[square1].set_square(square2);
	}
      square2=square1-6;
      if(square2>=0 && (square2&0x07)>(square1&0x07))
	{
	  mknight_attacks[square1].set_square(square2);
	}
    }
}


void Connections::init_bishop_attacks()
{
  int square2;
  for(int square1=a1.id();square1<=h8.id();square1++)
    {
      square2=square1+9;
      while(square2<64 && (square2&0x07)>(square1&0x07))
	{
	  mbishop_attacks[square1].set_square(square2);
	  square2+=9;
	}
      square2=square1+7;
      while(square2<64 && (square2&0x07)<(square1&0x07))
	{
	  mbishop_attacks[square1].set_square(square2);
	  square2+=7;
	}
      square2=square1-7;
      while(square2>=0 && (square2&0x07)>(square1&0x07))
	{
	  mbishop_attacks[square1].set_square(square2);
	  square2-=7;
	}
      square2=square1-9;
      while(square2>=0 && (square2&0x07)<(square1&0x07))
	{
	  mbishop_attacks[square1].set_square(square2);
	  square2-=9;
	}
    }
}


void Connections::init_rook_attacks()
{
  int square2;
  for(int square1=a1.id();square1<=h8.id();square1++)
    {
      square2=square1+1;
      while(square2<64 && (square2&0x07)>(square1&0x07))
	{
	  mrook_attacks[square1].set_square(square2);
	  square2+=1;
	}
      square2=square1+8;
      while(square2<64)
	{
	  mrook_attacks[square1].set_square(square2);
	  square2+=8;
	}
      square2=square1-1;
      while(square2>=0 && (square2&0x07)<(square1&0x07))
	{
	  mrook_attacks[square1].set_square(square2);
	  square2-=1;
	}
      square2=square1-8;
      while(square2>=0)
	{
	  mrook_attacks[square1].set_square(square2);
	  square2-=8;
	}
    }
}


void Connections::init_queen_attacks()
{
  mqueen_attacks=mbishop_attacks;
  for(int i=a1.id();i<=h8.id();i++)
    {
      mqueen_attacks[i]|=mrook_attacks[i];
    }
}

void Connections::init_king_attacks()
{
  int square2;
  for(int square1=0;square1<64;square1++)
    {
      square2=square1+1;
      if(square2<64 && (square2&0x07)>(square1&0x07))
	{
	  mking_attacks[square1].set_square(square2);
	}
      square2=square1+9;
      if(square2<64 && (square2&0x07)>(square1&0x07))
	{
	  mking_attacks[square1].set_square(square2);
	}
      square2=square1+8;
      if(square2<64)
	{
	  mking_attacks[square1].set_square(square2);
	}
      square2=square1+7;
      if(square2<64 && (square2&0x07)<(square1&0x07))
	{
	  mking_attacks[square1].set_square(square2);
	}
      square2=square1-1;
      if(square2>=0 && (square2&0x07)<(square1&0x07))
	{
	  mking_attacks[square1].set_square(square2);
	}
      square2=square1-9;
      if(square2>=0 && (square2&0x07)<(square1&0x07))
	{
	  mking_attacks[square1].set_square(square2);
	}
      square2=square1-8;
      if(square2>=0)
	{
	  mking_attacks[square1].set_square(square2);
	}
      square2=square1-7;
      if(square2>=0 && (square2&0x07)>(square1&0x07))
	{
	  mking_attacks[square1].set_square(square2);
	}
    }
}


void Connections::init_attacks_00()
{
  int square2;
  for(int square1=0;square1<64;square1++)
    {
      square2=square1+1;
      while((square2&0x07)>(square1&0x07))
	{
	  m_attacks_00.at(square1).set_square(square2);
	  square2+=1;
	}
    }  
}

void Connections::init_attacks_45()
{
  int square2;
  for(int square1=0;square1<64;square1++)
    {
      square2=square1+9;
      while(square2<64 && (square2&0x07)>(square1&0x07))
	{
	  m_attacks_45[square1].set_square(square2);
	  square2+=9;
	}
    }
}

void Connections::init_attacks_90()
{
  int square2;
  for(int square1=0;square1<64;square1++)
    {
      square2=square1+8;
      while(square2<64)
	{
	  m_attacks_90[square1].set_square(square2);
	  square2+=8;
	}
    }
}

void Connections::init_attacks_135()
{
  int square2;
  for(int square1=0;square1<64;square1++)
    {
      square2=square1+7;
      while(square2<64 && (square2&0x07)<(square1&0x07))
	{
	  m_attacks_135[square1].set_square(square2);
	  square2+=7;
	}
    }
}

void Connections::init_attacks_180()
{
  int square2;
  for(int square1=0;square1<64;square1++)
    {
      square2=square1-1;
      while(square2>=0 && (square2&0x07)<(square1&0x07))
	{
	  m_attacks_180[square1].set_square(square2);
	  square2-=1;
	}
    }
}

void Connections::init_attacks_225()
{
  int square2;
  for(int square1=0;square1<64;square1++)
    {
      square2=square1-9;
      while(square2>=0 && (square2&0x07)<(square1&0x07))
	{
	  m_attacks_225[square1].set_square(square2);
	  square2-=9;
	}
    }
}

void Connections::init_attacks_270()
{
  int square2;
  for(int square1=0;square1<64;square1++)
    {
      square2=square1-8;
      while(square2>=0)
	{
	  m_attacks_270[square1].set_square(square2);
	  square2-=8;
	}
    }
}

void Connections::init_attacks_315()
{
  int square2;
  for(int square1=0;square1<64;square1++)
    {
      square2=square1-7;
      while(square2>=0 && (square2&0x07)>(square1&0x07))
	{
	  m_attacks_315[square1].set_square(square2);
	  square2-=7;
	}
    }
}

void Connections::init_file()
{
  BitBoard obb,bb;
  for(int s=0;s<64;s++)
    {
      bb.clear();
      bb.set_square(s);
      obb.clear();
      while(obb!=bb)
	{
	  obb=bb;
	  bb|=(bb>>8);
	}
      obb.clear();
      while(obb!=bb)
	{
	  obb=bb;
	  bb|=(bb<<8);
	}
      m_file[s]=bb;
    }
}

void Connections::init_rank()
{
  BitBoard bb;
  for(int s=0;s<64;s++)
    {
      bb.clear();
      switch(s/8)
	{
	case 0:bb.set_square(a1);bb.set_square(b1);bb.set_square(c1);bb.set_square(d1);
	  bb.set_square(e1);bb.set_square(f1);bb.set_square(g1);bb.set_square(h1);break;
	case 1:bb.set_square(a2);bb.set_square(b2);bb.set_square(c2);bb.set_square(d2);
	  bb.set_square(e2);bb.set_square(f2);bb.set_square(g2);bb.set_square(h2);break;
	case 2:bb.set_square(a3);bb.set_square(b3);bb.set_square(c3);bb.set_square(d3);
	  bb.set_square(e3);bb.set_square(f3);bb.set_square(g3);bb.set_square(h3);break;
	case 3:bb.set_square(a4);bb.set_square(b4);bb.set_square(c4);bb.set_square(d4);
	  bb.set_square(e4);bb.set_square(f4);bb.set_square(g4);bb.set_square(h4);break;
	case 4:bb.set_square(a5);bb.set_square(b5);bb.set_square(c5);bb.set_square(d5);
	  bb.set_square(e5);bb.set_square(f5);bb.set_square(g5);bb.set_square(h5);break;
	case 5:bb.set_square(a6);bb.set_square(b6);bb.set_square(c6);bb.set_square(d6);
	  bb.set_square(e6);bb.set_square(f6);bb.set_square(g6);bb.set_square(h6);break;
	case 6:bb.set_square(a7);bb.set_square(b7);bb.set_square(c7);bb.set_square(d7);
	  bb.set_square(e7);bb.set_square(f7);bb.set_square(g7);bb.set_square(h7);break;
	case 7:bb.set_square(a8);bb.set_square(b8);bb.set_square(c8);bb.set_square(d8);
	  bb.set_square(e8);bb.set_square(f8);bb.set_square(g8);bb.set_square(h8);break;
	}
      m_rank[s]=bb;
    }
}

void Connections::init_ray()
{
  for(int s1=0;s1<64;s1++){
    for(int s2=0;s2<64;s2++){
      if(closed_line_segment(s1,s2)&m_attacks_00[s1]){m_ray[s1][s2]=m_attacks_00[s1];}
      else if(closed_line_segment(s1,s2)&m_attacks_45[s1]){m_ray[s1][s2]=m_attacks_45[s1];}
      else if(closed_line_segment(s1,s2)&m_attacks_90[s1]){m_ray[s1][s2]=m_attacks_90[s1];}
      else if(closed_line_segment(s1,s2)&m_attacks_135[s1]){m_ray[s1][s2]=m_attacks_135[s1];}
      else if(closed_line_segment(s1,s2)&m_attacks_180[s1]){m_ray[s1][s2]=m_attacks_180[s1];}
      else if(closed_line_segment(s1,s2)&m_attacks_225[s1]){m_ray[s1][s2]=m_attacks_225[s1];}
      else if(closed_line_segment(s1,s2)&m_attacks_270[s1]){m_ray[s1][s2]=m_attacks_270[s1];}
      else if(closed_line_segment(s1,s2)&m_attacks_315[s1]){m_ray[s1][s2]=m_attacks_315[s1];}
    }
  }
}


void Connections::init_linesegments()
{
  for(int s1=0;s1<64;s1++){
    for(int s2=0;s2<64;s2++){
      if(m_attacks_00[s1]&m_square[s2]){
	for(char s3=s1+1;s3<s2;s3+=1){
	  m_line_segment[s1][s2]|=m_square[s3];
	}
      }
      if(m_attacks_45[s1]&m_square[s2]){
	for(char s3=s1+9;s3<s2;s3+=9){
	  m_line_segment[s1][s2]|=m_square[s3];
	}
      }
      if(m_attacks_90[s1]&m_square[s2]){
	for(char s3=s1+8;s3<s2;s3+=8){
	  m_line_segment[s1][s2]|=m_square[s3];
	}
      }
      if(m_attacks_135[s1]&m_square[s2]){
	for(char s3=s1+7;s3<s2;s3+=7){
	  m_line_segment[s1][s2]|=m_square[s3];
	}
      }
      if(m_attacks_180[s1]&m_square[s2]){
	for(char s3=s1-1;s3>s2;s3-=1){
	  m_line_segment[s1][s2]|=m_square[s3];
	}
      }
      if(m_attacks_225[s1]&m_square[s2]){
	for(char s3=s1-9;s3>s2;s3-=9){
	  m_line_segment[s1][s2]|=m_square[s3];
	}
      }
      if(m_attacks_270[s1]&m_square[s2]){
	for(char s3=s1-8;s3>s2;s3-=8){
	  m_line_segment[s1][s2]|=m_square[s3];
	}
      }
      if(m_attacks_315[s1]&m_square[s2]){
	for(char s3=s1-7;s3>s2;s3-=7){
	  m_line_segment[s1][s2]|=m_square[s3];
	}
      }
    }
  }
  
  for(Square s1=a1;s1<=h8;s1++){
    for(Square s2=a1;s2<=h8;s2++){
      m_closed_line_segment[s1.id()][s2.id()]=m_line_segment[s1.id()][s2.id()];
      m_closed_line_segment[s1.id()][s2.id()]|=s1.bitboard();
      m_closed_line_segment[s1.id()][s2.id()]|=s2.bitboard();
    }
  }
}

Connections::Connections()
{
  BitBoard bb;
  vector<BitBoard> connbb_64(64,bb);
  vector<vector<BitBoard>> connbb_64x64(64,connbb_64);

  init_squares();
  mw_pawn_attacks=connbb_64;init_w_pawn_attacks();
  mb_pawn_attacks=connbb_64;init_b_pawn_attacks();
  mknight_attacks=connbb_64;init_knight_attacks();
  mbishop_attacks=connbb_64;init_bishop_attacks();
  mrook_attacks=connbb_64;init_rook_attacks();
  mqueen_attacks=connbb_64;init_queen_attacks();
  mking_attacks=connbb_64;init_king_attacks();
  
  m_attacks_00=connbb_64;init_attacks_00();
  m_attacks_45=connbb_64;init_attacks_45();
  m_attacks_90=connbb_64;init_attacks_90();
  m_attacks_135=connbb_64;init_attacks_135();
  m_attacks_180=connbb_64;init_attacks_180();
  m_attacks_225=connbb_64;init_attacks_225();
  m_attacks_270=connbb_64;init_attacks_270();
  m_attacks_315=connbb_64;init_attacks_315();

  m_file=connbb_64;init_file();
  m_rank=connbb_64;init_rank();
  
  m_line_segment=connbb_64x64;
  m_closed_line_segment=connbb_64x64;
  init_linesegments();
  m_ray=connbb_64x64;init_ray();
}

const BitBoard& Connections::w_pawn_attacks(const Square& s)const
{
  return mw_pawn_attacks[s.id()];
}

const BitBoard& Connections::b_pawn_attacks(const Square& s)const
{
  return mb_pawn_attacks[s.id()];
}

const BitBoard& Connections::knight_attacks(const Square& s)const
{
  return mknight_attacks[s.id()];
}

const BitBoard& Connections::bishop_attacks(const Square& s)const
{
  return mbishop_attacks[s.id()];
}

const BitBoard& Connections::rook_attacks(const Square& s)const
{
  return mrook_attacks[s.id()];
}

const BitBoard& Connections::queen_attacks(const Square& s)const
{
  return mqueen_attacks[s.id()];
}

const BitBoard& Connections::king_attacks(const Square& s)const
{
  return mking_attacks[s.id()];
}

const BitBoard& Connections::attacks_00(const Square& s)const
{
  return m_attacks_00[s.id()];
}

const BitBoard& Connections::attacks_45(const Square& s)const
{
  return m_attacks_45[s.id()];
}

const BitBoard& Connections::attacks_90(const Square& s)const
{
  return m_attacks_90[s.id()];
}

const BitBoard& Connections::attacks_135(const Square& s)const
{
  return m_attacks_135[s.id()];
}

const BitBoard& Connections::attacks_180(const Square& s)const
{
  return m_attacks_180[s.id()];
}

const BitBoard& Connections::attacks_225(const Square& s)const
{
  return m_attacks_225[s.id()];
}

const BitBoard& Connections::attacks_270(const Square& s)const
{
  return m_attacks_270[s.id()];
}

const BitBoard& Connections::attacks_315(const Square& s)const
{
  return m_attacks_315[s.id()];
}

const BitBoard& Connections::file(const Square& s)const
{
  return m_file[s.id()];
}

const BitBoard& Connections::rank(const Square& s)const
{
  return m_rank[s.id()];
}

const BitBoard& Connections::ray(const Square& s1,const Square& s2)const
{
  return m_ray[s1.id()][s2.id()];
}

const BitBoard& Connections::line_segment(const Square& s1,const Square& s2)const
{
  return m_line_segment[s1.id()][s2.id()];
}

const BitBoard& Connections::closed_line_segment(const Square& s1,const Square& s2)const
{
  return m_closed_line_segment[s1.id()][s2.id()];
}

