/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#ifndef MAGICMOVES_H
#define MAGICMOVES_H

#include"square.h"
#include"rookmask.h"
#include"bishopmask.h"
#include"magicnumber.h"
#include"../common/bitboard.h"
#include<array>
#include<vector>
#include<atomic>
#include<cassert>
using std::atomic;
using std::array;
using std::vector;



class MagicMoves
{
 private:
  array<vector<BitBoard>,64> magic_bishop_moves;
  array<vector<unsigned char>,64> bishop_index;
  array<vector<BitBoard>,64> bishop_attacks;

  array<vector<BitBoard>,64> magic_rook_moves;
  array<vector<unsigned char>,64> rook_index;
  array<vector<BitBoard>,64> rook_attacks;

  bool selftest_ok()const;

 public:
  MagicMoves();
  ~MagicMoves();
  template<char> const BitBoard& get_magic_moves(const Square&,const BitBoard&)const;
  template<char> const BitBoard& get_magic_moves(const BitBoard&,const BitBoard&)const;
  int size()const;
};



template<char C>
const BitBoard& MagicMoves::get_magic_moves(const Square& s,const BitBoard& occupancy)const
{
  RookMask rm;
  BishopMask bm;
  MagicNumber mn;
  unsigned long long magic_number;
  int shift_value;
  unsigned long long index;
  unsigned long long occupancy_number;
  
  occupancy_number=C=='R'?BitBoard::to_ull(occupancy&rm.get_mask(s)):BitBoard::to_ull(occupancy&bm.get_mask(s));
  magic_number=mn.get_magic_number<C>(s).first;
  shift_value=mn.get_magic_number<C>(s).second;
  index=(occupancy_number*magic_number)>>shift_value;
  return C=='R'?rook_attacks[s.id()][rook_index[s.id()][index]]:bishop_attacks[s.id()][bishop_index[s.id()][index]];
}

template<char C>
const BitBoard& MagicMoves::get_magic_moves(const BitBoard& bb_s,const BitBoard& occupancy)const
{
  Square s=bb_s.get_unique_square();
  return get_magic_moves<C>(s,occupancy);
}






#endif

