/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<iostream>
#include<stdexcept>
#include"piece.h"
#include"numerals.h"
using std::logic_error;
using std::cout;
using std::endl;


Piece::Piece(): p{0}
{
}

Piece::Piece(const int& i): p{i}
{
}

Piece::Piece(const Piece& x): p{x.p}
{
}

Piece& Piece::operator=(const Piece& x)
{
  p=x.p;
  return *this;
}

Piece::operator bool()const
{
  return static_cast<bool>(p);
}

bool Piece::operator==(const Piece& x)const
{
  return (p==x.p);
}

bool Piece::operator!=(const Piece& x)const
{
  return (p!=x.p);
}

bool Piece::black_piece()const
{
  return p&0x08;
}

bool Piece::white_piece()const
{
  return !black_piece();
}

bool Piece::no_piece()const
{
  return (p==no_piece_id);
}


bool Piece::pawn()const
{
  return ((p&7)==white_pawn_id);
}

bool Piece::knight()const
{
  return ((p&7)==white_knight_id);
}

bool Piece::bishop()const
{
  return ((p&7)==white_bishop_id);
}

bool Piece::rook()const
{
  return ((p&7)==white_rook_id);
}

bool Piece::queen()const
{
  return ((p&7)==white_queen_id);
}

bool Piece::king()const
{
  return ((p&7)==white_king_id);
}

bool Piece::white_pawn()const
{
  return (p==white_pawn_id);
}

bool Piece::white_knight()const
{
  return (p==white_knight_id);
}

bool Piece::white_bishop()const
{
  return (p==white_bishop_id);
}

bool Piece::white_rook()const
{
  return (p==white_rook_id);
}

bool Piece::white_queen()const
{
  return (p==white_queen_id);
}

bool Piece::white_king()const
{
  return (p==white_king_id);
}

bool Piece::black_pawn()const
{
  return (p==black_pawn_id);
}

bool Piece::black_knight()const
{
  return (p==black_knight_id);
}

bool Piece::black_bishop()const
{
  return (p==black_bishop_id);
}

bool Piece::black_rook()const
{
  return (p==black_rook_id);
}

bool Piece::black_queen()const
{
  return (p==black_queen_id);
}

bool Piece::black_king()const
{
  return (p==black_king_id);
}

void Piece::set_white_pawn()
{
  p=white_pawn_id;
}

void Piece::set_white_knight()
{
  p=white_knight_id;
}

void Piece::set_white_bishop()
{
  p=white_bishop_id;
}

void Piece::set_white_rook()
{
  p=white_rook_id;
}

void Piece::set_white_queen()
{
  p=white_queen_id;
}

void Piece::set_white_king()
{
  p=white_king_id;
}

void Piece::set_no_piece()
{
  p=no_piece_id;
}

void Piece::set_black_pawn()
{
  p=black_pawn_id;
}

void Piece::set_black_knight()
{
  p=black_knight_id;
}

void Piece::set_black_bishop()
{
  p=black_bishop_id;
}

void Piece::set_black_rook()
{
  p=black_rook_id;
}

void Piece::set_black_queen()
{
  p=black_queen_id;
}

void Piece::set_black_king()
{
  p=black_king_id;
}

void Piece::set_id(int x)
{
  p=x;
}

int Piece::value()const
{
  switch(p)
    {
    case 0:return 0;
    case 1:return pawn_value;
    case 2:return knight_value;
    case 3:return bishop_value;
    case 4:return rook_value;
    case 5:return queen_value;
    case 6:return king_value;
    case 9:return -pawn_value;
    case 10:return -knight_value;
    case 11:return -bishop_value;
    case 12:return -rook_value;
    case 13:return -queen_value;
    case 14:return -king_value;
    default: 
      cout << "Piece: " << int(p) << endl;
      throw logic_error("Piece::value out of range");
    }
}

int Piece::abs_value()const
{
  switch(p&7)
    {
    case no_piece_id:return 0;
    case white_pawn_id:return pawn_value;
    case white_knight_id:return knight_value;
    case white_bishop_id:return bishop_value;
    case white_rook_id:return rook_value;
    case white_queen_id:return queen_value;
    case white_king_id:return king_value;
    default:
      cout << "Piece: " << int(p) << endl;
      throw logic_error("Piece::abs_value out of range:");
    }
}

string Piece::name()const
{
  switch(p)
    {
    case no_piece_id:return "no piece";
    case white_pawn_id:return "white pawn";
    case white_knight_id:return "white knight";
    case white_bishop_id:return "white bishop";
    case white_rook_id:return "white rook";
    case white_queen_id:return "white queen";
    case white_king_id:return "white king";
    case black_pawn_id:return "black pawn";
    case black_knight_id:return "black knight";
    case black_bishop_id:return "black bishop";
    case black_rook_id:return "black rook";
    case black_queen_id:return "black queen";
    case black_king_id:return "black king";
    default: throw logic_error("Piece::unknown piece");
    }
}


const int& Piece::id()const
{
  return p;
}
