/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<string>
#include<stdexcept>
#include<sstream>
#include"../common/bitboard.h"
#include"square.h"
using std::logic_error;
using std::ostringstream;

const array<BitBoard,65> Square::bb=[]()->array<BitBoard,65>
{
  array<BitBoard,65> lbb;
  for(int i=0;i<64;i++){
    lbb[i]=BitBoard(1ull << i);
  }
  return lbb;  
}();

Square::Square() : msquare(64)
{
}

Square::Square(const Square& x) : msquare(x.msquare)
{
}

Square::Square(int x) : msquare(x)
{
}

Square::Square(string x)
{
  char column,row;
  if(x.size()!=2){throw logic_error("Square::Square(string)  size() error");}
  column=x[0];
  row=x[1];
  column-='a';
  row-='1';
  msquare=8*row+column;
  if(msquare<0 || msquare > 63){throw logic_error("Square::Square(string)  out of range");}
}

Square& Square::operator=(const Square& x)
{
  msquare=x.msquare;
  return *this;
}

const int& Square::id()const
{
  return msquare;
}

string Square::name()const
{
  string r;
  if(msquare<0 || msquare>64){
    ostringstream E;
    E  << "Square::name()const msquare=" << int(msquare);
    throw logic_error(E.str());
  }
  else if(msquare==64){
    r="no_square";
  }
  else{
    r=file();r+=rank();
  }
  return r;
}

char Square::file()const
{
  char column;
  column=msquare%8;
  column+='a';
  return column;
}

char Square::rank()const
{
  char row;
  row=msquare/8;
  row+='1';
  return row;
}

int Square::file_number()const
{
  return msquare%8;
}

int Square::rank_number()const
{
  return msquare/8;
}

int Square::color()const
{
  return ( bitboard() & white_squares ) ? white : black;
}

const BitBoard& Square::bitboard()const
{
  return bb[msquare];
}

bool Square::operator==(Square x)const
{
  return (msquare==x.msquare);
}

bool Square::operator!=(Square x)const
{
  return (msquare!=x.msquare);
}

bool Square::operator<(Square x)const
{
  return (msquare<x.msquare);
}

bool Square::operator>(Square x)const
{
  return (msquare>x.msquare);
}

bool Square::operator<=(Square x)const
{
  return (msquare<=x.msquare);
}

bool Square::operator>=(Square x)const
{
  return (msquare>=x.msquare);
}

Square& Square::operator+=(int x)
{
  msquare+=x;
  return *this;
}

Square& Square::operator-=(int x)
{
  msquare-=x;
  return *this;
}


Square& Square::operator++()
{
  msquare++;
  return *this;
}

Square Square::operator++(int)
{
  Square r{*this};
  msquare++;
  return r;
}

Square& Square::operator--()
{
  msquare--;
  return *this;
}

Square Square::operator--(int)
{
  Square r{*this};
  msquare--;
  return r;
}

//------------------------------------------------------------------------
//
Square operator+(Square s,int i)
{
  return s+=i;
}

Square operator+(int i,Square s)
{
  return s+=i;
}

Square operator-(Square s,int i)
{
  return s-=i;
}

Square operator-(int i,Square s)
{
  return s-=i;
}

int operator-(Square x,Square y)
{
  return x.id()-y.id();
}

Square flip(const Square& x)
{
  return Square(x.id()^56);
}

Square flop(const Square& x)
{
  int r=x.id();
  r=(r&56) + 7 - (r&7);
  return Square(r);
}
