/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"egt4pieces.h"
#include"../common/numerals.h"
#include<fstream>
#include<sstream>





Egt4Pieces::Egt4Pieces(piece_t p1,piece_t p2)
{
  m_egt_name="KBNK";
  m_p1_id=white_knight_id;
  m_p2_id=white_bishop_id;
  if(p1==white_queen_id && p2==black_rook_id){
    m_p1_id=p1;
    m_p2_id=p2;
    m_egt_name="KQKR";
  }
  m_patta=new Attacks;
}

Egt4Pieces::~Egt4Pieces()
{
  delete m_patta;
  if(m_pp && m_memory_owner){
    delete[] m_pp;
  }
}

bool Egt4Pieces::busy()const
{
  return (m_depth>=0 && m_depth<=100);
}

bool Egt4Pieces::ready()const
{
  return m_depth>100;
}

void Egt4Pieces::clear()
{
  m_depth=-1;
  if(m_pp && m_memory_owner){
    delete[] m_pp;
    m_pp=nullptr;
  }
}

void Egt4Pieces::set_memory(unsigned char* pc)
{
  m_depth=-1;
  if(m_pp && m_memory_owner){
    delete[] m_pp;
    m_pp=nullptr;
  }
  m_memory_owner=false;
  m_pp=pc;
}

bool Egt4Pieces::operator==(const Egt4Pieces& x)const
{
  if(m_p1_id!=x.m_p1_id || m_p2_id!=x.m_p2_id || m_size!=x.m_size){return false;}
  for(unsigned i=0;i<m_size;i++){
    if(m_pp[i]!=x.m_pp[i]){return false;}
  }
  return true;
}

bool Egt4Pieces::operator!=(const Egt4Pieces& x)const
{
  return !operator==(x);
}

unsigned Egt4Pieces::operator[](unsigned x)const
{
  return static_cast<unsigned>(m_pp[x]);
}

int Egt4Pieces::stm()const
{
  return m_stm;
}

unsigned Egt4Pieces::size()const
{
  return m_size;
}

void Egt4Pieces::fen(string s)
{
  m_p1.clear();
  m_p2.clear();
  m_wk.clear();
  m_bk.clear();

  string s1,s2;
  istringstream istr(s);
  istr >> s1 >> s2;

  Square square=a8;
  for(auto c : s1){
    switch(c){
    case 'N':m_p1_id=white_knight_id;m_p1|=square.bitboard();square++;break;
    case 'B':m_p2_id=white_bishop_id;m_p2|=square.bitboard();square++;break;
    case 'R':m_p2_id=white_rook_id;m_p2|=square.bitboard();square++;break;
    case 'Q':m_p1_id=white_queen_id;m_p1|=square.bitboard();square++;break;
    case 'K':m_wk=square.bitboard();square++;break;
    case 'n':m_p1_id=black_knight_id;m_p1|=square.bitboard();square++;break;
    case 'b':m_p2_id=black_bishop_id;m_p2|=square.bitboard();square++;break;
    case 'r':m_p2_id=black_rook_id;m_p2|=square.bitboard();square++;break;
    case 'q':m_p1_id=black_queen_id;m_p1|=square.bitboard();square++;break;
    case 'k':m_bk=square.bitboard();square++;break;
    case '1':square++;break;
    case '2':square+=2;break;
    case '3':square+=3;break;
    case '4':square+=4;break;
    case '5':square+=5;break;
    case '6':square+=6;break;
    case '7':square+=7;break;
    case '8':square+=8;break;
    case '/':square-=16;break;
    default:
      ostringstream E;
      E << "Egt4Pieces::fen(" << s << ") \nwrong piece";
      throw std::logic_error(E.str());
    }      
  }  
  m_stm=s2=="w"?white:black;
  m_sign=1;

  if(m_p1_id==black_knight_id || m_p1_id==black_queen_id){
    swap(m_wk,m_bk);
    m_p1_id=static_cast<piece_t>(m_p1_id^8);
    m_p2_id=static_cast<piece_t>(m_p2_id^8);
    m_stm=m_stm==white?black:white;
    m_sign=-1;
  }
}

string Egt4Pieces::fen()const
{
  int empty{0};
  std::ostringstream oss;
  Square left_square{a8};
  for(int i=0;i<8;i++){
    for(Square s=left_square;s<left_square+8;s++){
      if(m_wk & s.bitboard()){
	if(empty>0){oss << empty;empty=0;}
	oss << "K";
      }else if(m_bk & s.bitboard()){
	if(empty>0){oss << empty;empty=0;}
	oss << "k";
      }else if(m_p1 & s.bitboard()){
	if(empty>0){oss << empty;empty=0;}
	oss << (m_p1_id==white_knight_id?"N":"Q");
      }else if(m_p2 & s.bitboard()){
	if(empty>0){oss << empty;empty=0;}
	oss << (m_p2_id==white_bishop_id?"B":"r");
      }else{
	empty++;
      }
    }
    if(empty>0){oss << empty;empty=0;}
    if(left_square>a1){
      oss << "/";    
    }else{
      oss << " " << (m_stm==white?"w":"b") << " - - 0 1";
    }
    left_square-=8;
  }
  return oss.str();
}

void Egt4Pieces::set_key(unsigned key)
{
  Square wk_square[10]={a1,b1,c1,d1,b2,c2,d2,c3,d3,d4};
  Square sK=Square(wk_square[key>>19 & 0x3f].id());
  Square sk=Square(key>>13  & 0x3f);
  Square s2=Square(key>>7 & 0x3f);
  Square s1=Square(key>>1  & 0x3f);
  m_stm=key & 0x1;
  m_wk=sK.bitboard();
  m_bk=sk.bitboard();
  m_p2=s2.bitboard();
  m_p1=s1.bitboard();
  if(m_p1==m_bk){m_p1.clear();}
  if(m_p2==m_bk){m_p2.clear();}
  if(m_p1==m_wk){m_p1.clear();}
  if(m_p2==m_wk){m_p2.clear();}
}

unsigned Egt4Pieces::get_key()const
{
  int wk_square_id[28]={0,1,2,3,64,64,64,64,64,4,5,6,64,64,64,64,64,64,7,8,64,64,64,64,64,64,64,9};

  BitBoard bb_wk{m_wk},bb_bk{m_bk},bb_p1{m_p1},bb_p2{m_p2};
  //---------------------------- use symmetry: ----------------------------
  //
  //------------ missing piece: -------------------------------------------
  //
  if(!bb_p2){bb_p2=bb_bk;}
  if(!bb_p1){bb_p1=bb_bk;}
  //
  //------------ white king in a1-d1-d4 triangle: -------------------------
  //
  
  if(bb_wk.get_unique_square().id()>=32){
    bb_wk.flip();
    bb_bk.flip();
    bb_p1.flip();
    bb_p2.flip();
  }
  if(bb_wk.get_unique_square().bitboard() & BitBoard(0xf0f0f0f0f0f0f0f0)){
    bb_wk.flop();
    bb_bk.flop();
    bb_p1.flop();
    bb_p2.flop();
  }
  if(bb_wk & a2_a4_c4){
    Square wk=bb_wk.get_unique_square();
    Square bk=bb_bk.get_unique_square();
    Square p1=bb_p1.get_unique_square();
    Square p2=bb_p2.get_unique_square();
    
    wk=Square(wk.file_number()*8 + wk.rank_number());
    bk=Square(bk.file_number()*8 + bk.rank_number());
    p1=Square(p1.file_number()*8 + p1.rank_number());
    p2=Square(p2.file_number()*8 + p2.rank_number());

    bb_wk=wk.bitboard();
    bb_bk=bk.bitboard();
    bb_p2=p2.bitboard();
    bb_p1=p1.bitboard();
  }

  unsigned key=wk_square_id[bb_wk.get_unique_square().id()]<<19;
  key|=bb_bk.get_unique_square().id()<<13;
  key|=bb_p2.get_unique_square().id()<<7;
  key|=bb_p1.get_unique_square().id()<<1;
  key|=m_stm;
  return key;
}

unsigned Egt4Pieces::get_mirror_key(unsigned key)const
{
  Square sk=Square(key>>13  & 0x3f);
  Square s2=Square(key>>7 & 0x3f);
  Square s1=Square(key>>1  & 0x3f);
  unsigned stm=key&1;
  
  sk=Square(sk.file_number()*8 + sk.rank_number());
  s2=Square(s2.file_number()*8 + s2.rank_number());
  s1=Square(s1.file_number()*8 + s1.rank_number());
  
  key&=0x3F << 19;
  key|=sk.id()<<13;
  key|=s2.id()<<7;
  key|=s1.id()<<1;
  key|=stm;
  
  return key;
}

string Egt4Pieces::get_key_str(unsigned key)const
{
  Square wk_square[10]={a1,b1,c1,d1,b2,c2,d2,c3,d3,d4};
  ostringstream oss;
  oss << "K: ";
  Square sK=Square(wk_square[key>>19&0x3f]);
  oss << sK.name() << "   ";
  oss << "k: ";
  Square sk=Square(key>>13&0x3f);
  oss << sk.name() << "   ";
  oss << (m_p1_id==white_knight_id?"N: ":"Q: ");
  Square s1=Square(key>>1&0x3f);
  oss << s1.name() << "   ";
  oss << (m_p2_id==white_bishop_id?"B: ":"r: ");
  Square s2=Square(key>>7&0x3f);
  oss << s2.name() << "   ";
  int color=key&1;
  oss << (color==white?"(white)":"(black)");
  return oss.str();
}

string Egt4Pieces::get_key_str()const
{
  unsigned key=get_key();
  return get_key_str(key);
}

string Egt4Pieces::mate_in(int x)const
{
  ostringstream oss;
  for(unsigned i=0;i<m_size;i++){
    if(m_pp[i]==x){
      oss << "mate in " << x << ":   " << fen() << endl;
    }
  }
  return oss.str();
}

int Egt4Pieces::diff(const Egt4Pieces& x)const
{
  unsigned d1{m_size},p{0};
  for(unsigned i=0;i<m_size;i++){
    if(m_pp[i]!=x.m_pp[i]){
      if(abs(m_pp[i])<d1){
	p=i;
	d1=abs(m_pp[i]);
      }
    }
  }
  return p;
}

int Egt4Pieces::eval()const
{
  unsigned x=get_key();
  return eval(x);
}

int Egt4Pieces::eval(string s)
{
  fen(s);
  unsigned x=get_key();
  return eval(x);
}

int Egt4Pieces::eval(unsigned x)const
{
  if(!m_pp || m_depth<0){return static_cast<int>(0xFF);}
  unsigned char c=m_pp[x];
  if(c==0xFD){return 0;}
  if(c==0xFE || c==0xFF){return static_cast<int>(c);}
  if(c&0x80){ // black wins
    c&=0x7f;
    return (-mate_value + static_cast<int>(c))*m_sign; 
  }else{      // white wins
    return (mate_value - static_cast<int>(c))*m_sign; 
  }
}

bool Egt4Pieces::create()
{
  if(m_depth<0){
    m_tim.start();
    m_depth=0;
    if(!m_pp && m_memory_owner){
      m_pp=new unsigned char[m_size];
    }
    set_initial_eval();
    m_tim.stop();
    //std::cerr << "time set_initial_eval(): " << m_tim.total_time().milliseconds() << endl;
    return false;
  }else if(m_depth>100){
    return true;
  }

  m_tim.start();
  create_next();
  m_tim.stop();
  //std::cerr << "time depth=" << m_depth << " " << m_tim.total_time().milliseconds() << endl;
  if(m_depth>=100){
    for(unsigned i=0;i<m_size;i++){
      if(m_pp[i]==0xFF){m_pp[i]=0xFD;}
    }
    return true;
  }else{
    return false;
  }
}

void Egt4Pieces::create_next()
{
  for(unsigned i=0;i<m_size;i++){
    if((m_pp[i]&0x7F)!=m_depth){continue;}

    //------------------------------- stm is losing: ----------------------------------------------------------
    //
    if(   ((i&1)==white && (m_pp[i]&0x80)) || (((i&1)==black) && !(m_pp[i]&0x80))   ){
      set_key(i);
      vector<Move> ml;
      generate_previous_moves(ml);
      for(auto m : ml){
	unmake_move(m);
	unsigned j=get_key();
	if(m_pp[j]==0xFF){
	  m_pp[j]=m_pp[i]+1;
	  if(m_wk & diagonals){
	    m_pp[get_mirror_key(j)]=m_pp[j];
	  }
	}
	make_move(m);
      }
    }

    //------------------------------- stm is winning: ---------------------------------------------------------
    //
    if(((i&1)==white && !(m_pp[i]&0x80)) || ((i&1)==black && (m_pp[i]&0x80))){
      set_key(i);
      vector<Move> ml;
      generate_previous_moves(ml);
      for(auto m : ml){
	unmake_move(m);
	unsigned j=get_key();
	if(m_pp[j]==0xFF){
	  bool finished=true;
	  vector<Move> ml2;
	  generate_moves(ml2);
	  int stm=m_stm;
	  unsigned char best_score=stm==white?0x80:0x00;
	  for(auto m2 : ml2){
	    make_move(m2);
	    unsigned t=get_key();
	    if(m_pp[t]==0xFF){
	      unmake_move(m2);finished=false;break;
	    }else if(m_pp[t]==0xFE){
	      unmake_move(m2);continue;
	    }else{
	      best_score=bestscore(stm,m_pp[t],best_score);
	    }
	    unmake_move(m2);
	  }
	  if(finished){
	    m_pp[j]=best_score==0xFD?0xFD:best_score+1;
	    if(m_wk & diagonals){
	      m_pp[get_mirror_key(j)]=m_pp[j];
	    }
	    //------------------ debug: ----------------------------
	    /*
	    if(j==m_debug_key){
	      string pmoves;
	      std::cerr << endl << "Side to move is winning" << endl;
	      std::cerr << "Debug:         " << fen() << "   m_pp[" << j << "]=" << (unsigned)m_pp[j] << endl;
	      for(auto m : ml2){pmoves+=m.algebraic() + (m.capture()?"("+m.capture().name()+")":"") + " ";}
	      std::cerr << "Moves         :    " << pmoves << endl;
	      std::cerr << "Next position: " << ifen <<  "   m_pp[" << i << "]=" << (unsigned)m_pp[i] << endl;
	      pmoves.clear();
	      for(auto m : ml){pmoves+=m.algebraic() + (m.capture()?"("+m.capture().name()+")":"") + " ";}
	      std::cerr << "Previous moves:    " << pmoves << endl;
	    }
	    */
	    //------------------------------------------------------
	  }
	}
	make_move(m);
      }
    }
	 
  }
  
  m_depth++;
}

void Egt4Pieces::set_initial_eval()
{
  for(unsigned i=0;i<m_size;i++){
    m_pp[i]=0xFF;
    set_initial_eval(i);
  }
}

void Egt4Pieces::set_initial_eval(unsigned key)
{
  if(!is_legal_key(key)){m_pp[key]=0xFE;return;}
  
  set_key(key);
  if(!is_legal()){m_pp[key]=0xFE;return;}

  if(m_p1_id==white_knight_id && m_p2_id==white_bishop_id){
    if(!m_p1){m_pp[key]=0xFD;return;}
    if(!m_p2){m_pp[key]=0xFD;return;}
  }

  // check?
  BitBoard pieces=m_wk | m_p1 | m_p2 | m_bk;
  BitBoard p1_moves,p2_moves;

  if(m_p1){
    p1_moves=m_p1_id==white_knight_id?m_patta->knight(m_p1):m_patta->queen(m_p1,pieces);
  }
  if(m_p2){
    p2_moves=m_p2_id==white_bishop_id?m_patta->bishop(m_p2,pieces):m_patta->rook(m_p2,pieces);
  }
  bool check{false};
  if(m_bk & p1_moves){check=true;}
  if(m_p2_id<black_pawn_id && (m_bk & p2_moves)){check=true;}
  if(m_p2_id>=black_pawn_id && (m_wk & p2_moves)){check=true;}
  //
  
  vector<Move> ml;
  generate_moves(ml);
  if(check){
    for(auto m : ml){
      make_move(m);
      if(is_legal()){return;}
      unmake_move(m);
    }
    m_pp[key]=m_stm==black?0x00:0x80;  // checkmate    
  }else{
    for(auto m : ml){
      make_move(m);
      if(is_legal()){return;}
      unmake_move(m);
    }
    m_pp[key]=0xFD;  // stalemate
  }
  return;
}

bool Egt4Pieces::is_legal_key(unsigned key)const
{
  if((key>>19 & 0x3f)>10){return false;}
  int wk_square[10]={0,1,2,3,9,10,11,18,19,27};
  int sK=wk_square[key>>19 & 0x3f];
  int sk=key>>13 & 0x3f;
  int s2=key>>7 & 0x3f;
  int s1=key>>1 & 0x3f;
  if(s1==sK || s2==sK || sk==sK || (s1==s2 && s1!=sk)){
    return false;
  }else{
    return true;
  }
}

bool Egt4Pieces::is_legal()const
{
  if(m_wk.popcount()!=1 || m_bk.popcount()!=1){return false;}
  BitBoard pieces=m_wk | m_bk | m_p1 | m_p2;
  BitBoard p1_moves,p2_moves;
  if(m_p1){p1_moves=m_p1_id==white_knight_id?m_patta->knight(m_p1):m_patta->queen(m_p1,pieces);}
  if(m_p2){p2_moves=m_p2_id==white_bishop_id?m_patta->bishop(m_p2,pieces):m_patta->rook(m_p2,pieces);}
  BitBoard bkmoves=m_patta->king(m_bk);

  if(m_wk & bkmoves){return false;}
  
  if(m_p2_id<black_pawn_id){
    if(m_stm==white){
      if(p1_moves & m_bk){return false;}
      if(p2_moves & m_bk){return false;}
    }
  }else{
    if(m_stm==white){
      if(p1_moves & m_bk){return false;}
    }else{
      if(p2_moves & m_wk){return false;}
    }
  }
  
  return true;
}

bool Egt4Pieces::is_consistent()
{
  bool ok{false};
  vector<Move> vm;
  for(unsigned i=0;i<size();i++){
    if(!is_legal_key(i)){continue;}
    set_key(i);
    if(!is_legal()){continue;}
    string ifen=fen();
    int ev=eval();
    
    // test for white winning: 
    if(ev>0 && ev<30000 && ev!=0xFE && ev!=0xFF){
      generate_moves(vm);
      if(m_stm==white){
	ok=false;
	for(auto m : vm){
	  make_move(m);
	  if(is_legal()){
	    if(eval()==ev+1){ok=true;}
	    if(eval()>ev+1){
	      std::cerr << "new eval > old eval + 1" << endl;
	      std::cerr << "orig fen: " << ifen <<  "  eval()=" << ev << "   key=" << i << endl;
	      std::cerr << "next fen: " << fen() << "  eval()=" << eval() << "   key=" << get_key() << endl;
	      for(auto m : vm){std::cerr << m.algebraic()<< (!m.capture().no_piece()?"("+m.capture().name()+")":"") << " ";}std::cerr << endl;
	      ok=false;
	      break;
	    }
	  }
	  unmake_move(m);
	}
      }else{
	ok=false;
	for(auto m : vm){
	  make_move(m);
	  if(is_legal()){
	    if(eval()==ev+1){ok=true;}
	    if(eval()<=ev){
	      std::cerr << "new eval < old eval" << endl;
	      std::cerr << "orig fen: " << ifen <<  "  eval()=" << ev << "   key=" << i << endl;
	      std::cerr << "next fen: " << fen() << "  eval()=" << eval() << "   key=" << get_key() << endl;
	      for(auto m : vm){std::cerr << m.algebraic()<< (!m.capture().no_piece()?"("+m.capture().name()+")":"") << " ";}std::cerr << endl;
	      ok=false;
	      break;
	    }
	  }
	  unmake_move(m);
	}
      }
      if(!ok){break;}
    }
    
    // test for black winning: 
    if(ev<0 && ev>-30000 && ev!=0xFE && ev!=0xFF){
      generate_moves(vm);
      if(m_stm==black){
	ok=false;
	for(auto m : vm){
	  make_move(m);
	  if(is_legal()){
	    if(eval()==ev-1){ok=true;}
	    if(eval()<ev-1){
	      std::cerr << "new eval > old eval + 1" << endl;
	      std::cerr << "orig fen: " << ifen <<  "  eval()=" << ev << "   key=" << i << endl;
	      std::cerr << "next fen: " << fen() << "  eval()=" << eval() << "   key=" << get_key() << endl;
	      for(auto m : vm){std::cerr << m.algebraic()<< (!m.capture().no_piece()?"("+m.capture().name()+")":"") << " ";}std::cerr << endl;
	      ok=false;
	      break;
	    }
	  }
	  unmake_move(m);
	}
      }else{
	ok=false;
	for(auto m : vm){
	  make_move(m);
	  if(is_legal()){
	    if(eval()==ev-1){ok=true;}
	    if(eval()>=ev){
	      std::cerr << "new eval < old eval" << endl;
	      std::cerr << "orig fen: " << ifen <<  "  eval()=" << ev << "   key=" << i << endl;
	      std::cerr << "next fen: " << fen() << "  eval()=" << eval() << "   key=" << get_key() << endl;
	      for(auto m : vm){std::cerr << m.algebraic()<< (!m.capture().no_piece()?"("+m.capture().name()+")":"") << " ";}std::cerr << endl;
	      ok=false;
	      break;
	    }
	  }
	  unmake_move(m);
	}
      }
      if(!ok){break;}
    }

    // test for draw: 
    if(ev==0){
      generate_moves(vm);
      ok=true;
      for(auto m : vm){
	make_move(m);
	if(is_legal()){
	  if(m_stm==black && eval()>ev){
	    std::cerr << "Test for draw:" << endl;
	    std::cerr << "new eval > old eval" << endl;
	    std::cerr << "orig fen: " << ifen <<  "  eval()=" << ev << "   key=" << i << endl;
	    std::cerr << "next fen: " << fen() << "  eval()=" << eval() << "   key=" << get_key() << endl;
	    for(auto m : vm){std::cerr << m.algebraic()<< (!m.capture().no_piece()?"("+m.capture().name()+")":"") << " ";}std::cerr << endl;
	    ok=false;
	    break;
	  }else if(m_stm==white && eval()<ev){
	    std::cerr << "Test for draw:" << endl;
	    std::cerr << "new eval < old eval" << endl;
	    std::cerr << "orig fen: " << ifen <<  "  eval()=" << ev << "   key=" << i << endl;
	    std::cerr << "next fen: " << fen() << "  eval()=" << eval() << "   key=" << get_key() << endl;
	    for(auto m : vm){std::cerr << m.algebraic()<< (!m.capture().no_piece()?"("+m.capture().name()+")":"") << " ";}std::cerr << endl;
	    ok=false;
	    break;
	  }
	}
	unmake_move(m);
      }
      if(!ok){
	//throw logic_error("Egt4Pieces::is_consistent():  Draw not ok");
      } 
    }
  }
    
  if(ok){return true;}else{return false;}
}

void Egt4Pieces::make_move(Move& m)
{

  m_make_move_counter++;
  BitBoard bb_move{m.from().bitboard() | m.to().bitboard()};
  switch(m.piece_id()){
  case white_knight_id:m_p1^=bb_move;break;
  case white_queen_id:m_p1^=bb_move;break;
  case white_bishop_id:m_p2^=bb_move;break;
  case black_rook_id:m_p2^=bb_move;break;
  case white_king_id:m_wk^=bb_move;break;
  case black_king_id:m_bk^=bb_move;break;
  }
  switch(m.capture_id()){
  case white_knight_id:m_p1.clear();break;
  case white_queen_id:m_p1.clear();break;
  case white_bishop_id:m_p2.clear();break;
  case black_rook_id:m_p2.clear();break;
  }
  m_stm=m_stm==white?black:white;
}

void Egt4Pieces::unmake_move(Move& m)
{
  BitBoard bb_move{m.from().bitboard() | m.to().bitboard()};
  switch(m.piece_id()){
  case white_knight_id:m_p1^=bb_move;break;
  case white_queen_id:m_p1^=bb_move;break;
  case white_bishop_id:m_p2^=bb_move;break;
  case black_rook_id:m_p2^=bb_move;break;
  case white_king_id:m_wk^=bb_move;break;
  case black_king_id:m_bk^=bb_move;break;
  }
  switch(m.capture_id()){
  case white_knight_id:m_p1=m.to().bitboard();break;
  case white_queen_id:m_p1=m.to().bitboard();break;
  case white_bishop_id:m_p2=m.to().bitboard();break;
  case black_rook_id:m_p2=m.to().bitboard();break;
  }
  m_stm=m_stm==white?black:white;
}
 
void Egt4Pieces::generate_moves(vector<Move>& ml)
{
  ml.clear();
  BitBoard wpieces=m_wk | m_p1;
  BitBoard bpieces=m_bk;
  if(m_p2_id<black_pawn_id){
    wpieces|= m_p2;
  }else{
    bpieces|=m_p2;
  }
  BitBoard pieces=wpieces | bpieces;
  BitBoard wkmoves=m_patta->king(m_wk);
  BitBoard bkmoves=m_patta->king(m_bk);
  BitBoard p1_moves,p2_moves;
  if(m_p1){
    p1_moves=m_p1_id==white_knight_id?m_patta->knight(m_p1):m_patta->queen(m_p1,pieces);
  }
  if(m_p2){
    p2_moves=m_p2_id==white_bishop_id?m_patta->bishop(m_p2,pieces):m_patta->rook(m_p2,pieces);
  }

  BitBoard legal_wkmoves=wkmoves & ~wpieces & ~m_bk & ~bkmoves;
  BitBoard legal_bkmoves=bkmoves & ~bpieces & ~m_wk & ~(wkmoves | p1_moves) & ~bpieces;
  
  if(m_stm==white){
    if(m_p1){
      Square p1_square=m_p1.get_unique_square();
      p1_moves&=~(wpieces | m_bk);
      while(p1_moves){
	Square to=p1_moves.get_reset_square().get_unique_square();
	Move egtmove=Move(p1_square,to,normal_move,WhiteKnight(),NoPiece());
	if(to.bitboard() & m_p2){egtmove.capture(m_p2_id);}
	ml.push_back(egtmove);
      }
    }
    if(m_p2_id==white_bishop_id && m_p2){
      Square p2_square=m_p2.get_unique_square();
      p2_moves&=~(wpieces | m_bk);
      while(p2_moves){
	Square to=p2_moves.get_reset_square().get_unique_square();
	Move egtmove=Move(p2_square,to,normal_move,WhiteBishop(),NoPiece());
	ml.push_back(egtmove);
      }
    }
    Square kingsquare=m_wk.get_unique_square();
    while(legal_wkmoves){
      Square to=legal_wkmoves.get_reset_square().get_unique_square();
      Move egtmove=Move(kingsquare,to,normal_move,WhiteKing(),NoPiece());
      if(to.bitboard() & m_p2){egtmove.capture(m_p2_id);}
      ml.push_back(egtmove);
    }
  }else{
    if(m_p2_id>=black_pawn_id && m_p2){
      Square p2_square=m_p2.get_unique_square();
      p2_moves&=~(bpieces | m_wk);
      while(p2_moves){
	Square to=p2_moves.get_reset_square().get_unique_square();
	Move egtmove=Move(p2_square,to,normal_move,BlackRook(),NoPiece());
	if(to.bitboard() & m_p1){egtmove.capture(m_p1_id);}
	ml.push_back(egtmove);
      }
    }
    Square kingsquare=m_bk.get_unique_square();
    while(legal_bkmoves){
      Square to=legal_bkmoves.get_reset_square().get_unique_square();
      Move egtmove=Move(kingsquare,to,normal_move,BlackKing(),NoPiece());
      if(to.bitboard() & m_p1){egtmove.capture(m_p1_id);}
      if(to.bitboard() & m_p2){egtmove.capture(m_p2_id);}
      ml.push_back(egtmove);
    }
  }
}


void Egt4Pieces::generate_previous_moves(vector<Move>& ml)
{
  ml.clear();
  BitBoard wpieces=m_wk | m_p1;
  BitBoard bpieces=m_bk;
  if(m_p2_id<black_pawn_id){
    wpieces|= m_p2;
  }else{
    bpieces|=m_p2;
  }
  BitBoard pieces=wpieces | bpieces;
  BitBoard wkmoves=m_wk?m_patta->king(m_wk):BitBoard();
  BitBoard bkmoves=m_bk?m_patta->king(m_bk):BitBoard();
  BitBoard p1_moves,p2_moves;
  if(m_p1){
    p1_moves=m_p1_id==white_knight_id?m_patta->knight(m_p1):m_patta->queen(m_p1,pieces);
  }
  if(m_p2){
    p2_moves=m_p2_id==white_bishop_id?m_patta->bishop(m_p2,pieces):m_patta->rook(m_p2,pieces);
  }

  //----------- never capture a piece: ----------
  p1_moves&=~pieces;
  p2_moves&=~pieces;
  wkmoves&=~pieces;
  bkmoves&=~pieces;

  //----------- don't give check: ---------------  
  BitBoard legal_wkmoves=wkmoves & ~bkmoves;
  BitBoard legal_bkmoves=bkmoves & ~wkmoves;
  BitBoard legal_p1_moves,legal_p2_moves;
  if(m_p2_id<black_pawn_id){
    legal_p1_moves=p1_moves & ~(m_patta->knight(m_bk));
    legal_p2_moves=p2_moves & ~(m_patta->bishop(m_bk,pieces));
  }else{
    legal_p1_moves=p1_moves & ~(m_patta->queen(m_bk,pieces));
    legal_p2_moves=p2_moves & ~(m_patta->rook(m_wk,pieces));    
  }

  if(m_stm==black){
    Square kingsquare=m_wk.get_unique_square();
    while(legal_wkmoves){
      Square s=legal_wkmoves.get_reset_square().get_unique_square();
      Move egtmove=Move(s,kingsquare,normal_move,WhiteKing(),NoPiece());
      ml.push_back(egtmove);
      if(!m_p2 && m_p2_id==black_rook_id){
	egtmove.capture(black_rook_id);
	ml.push_back(egtmove);
      }
    }
    if(m_p1){
      Square p1_square=m_p1.get_unique_square();
      while(legal_p1_moves){
	Square s=legal_p1_moves.get_reset_square().get_unique_square();
	Move egtmove=Move(s,p1_square,normal_move,m_p1_id,no_piece_id);
	ml.push_back(egtmove);
	if(!m_p2 && m_p2_id==black_rook_id){
	  egtmove.capture(black_rook_id);
	  ml.push_back(egtmove);
	}
      }
    }
    if(m_p2_id < black_pawn_id && m_p2){
      Square p2_square=m_p2.get_unique_square();
      while(legal_p2_moves){
	Square s=legal_p2_moves.get_reset_square().get_unique_square();
	Move egtmove=Move(s,p2_square,normal_move,m_p2_id,no_piece_id);
	ml.push_back(egtmove);
      }
    }
  }else{
    Square kingsquare=m_bk.get_unique_square();
    while(legal_bkmoves){
      Square s=legal_bkmoves.get_reset_square().get_unique_square();
      Move egtmove=Move(s,kingsquare,normal_move,BlackKing(),NoPiece());
      ml.push_back(egtmove);
      if(!m_p1){
	egtmove.capture(m_p1_id);
	ml.push_back(egtmove);
      }
      if(!m_p2 && m_p2_id<black_pawn_id){
	egtmove.capture(m_p2_id);
	ml.push_back(egtmove);
      }
    }
    if(m_p2_id>black_pawn_id && m_p2){
      Square p2_square=m_p2.get_unique_square();
      while(legal_p2_moves){
	Square s=legal_p2_moves.get_reset_square().get_unique_square();
	Move egtmove=Move(s,p2_square,normal_move,m_p2_id,no_piece_id);
	ml.push_back(egtmove);
	if(!m_p1){
	  egtmove.capture(m_p1_id);
	  ml.push_back(egtmove);
	}
      }
    }
  }
}

string Egt4Pieces::stats()const
{
  array<int,256> frequency;
  for(auto& i : frequency){i=0;}
  for(unsigned i=0;i<m_size;i++){
    frequency[static_cast<unsigned int>(m_pp[i])]++;
  }

  ostringstream oss;
  for(int i=0;i<256;i++){
    if(i%8==0){oss << endl;}
    oss << setw(7) << frequency[i] << "  ";
  }

  return oss.str();
}

void Egt4Pieces::save(string fn)const
{
  ofstream ofs(fn);
  ofs << "begin{"+m_egt_name+"}";
  for(unsigned i=0;i<m_size;i++){
    if(i%16==0){ofs << endl;}
    ofs << (int)m_pp[i] << " ";
  }
  ofs << endl << "end{"+m_egt_name+"}";
}

void Egt4Pieces::load(string fn)
{
  string s;
  int dtm;
  ifstream ifs(fn);
  ifs >> s;
  if(s.find("begin")==string::npos){throw runtime_error("Error: EGT not recognized");}
  if(s.find("KQKR")!=string::npos){
    m_p1_id=white_queen_id;
    m_p2_id=black_rook_id;
  }
  if(!m_pp && m_memory_owner){
    m_pp=new unsigned char[m_size];
  }
  for(unsigned i=0;i<m_size;i++){
    ifs >> dtm;
    m_pp[i]=dtm;
    if(!ifs.good()){
      ostringstream oss;
      oss << "Egt4Pieces::load(" << fn << ") failed    i=" << i << endl;
      throw runtime_error(oss.str());
    }
  }
  ifs >> s;
  if(s[0]!='e' && s[1]!='n' && s[2]!='d'){throw runtime_error("Error while reading EGT");}
}


unsigned char Egt4Pieces::bestscore(int stm,unsigned char sc,unsigned char bsc)const
{
  unsigned char r;
  int score,best_score;
  //-------------------------------------------------------------------------
  if(sc==0xFD){
    score=0;
  }else{
    score=(sc & 0x80)?-255+static_cast<int>(sc & 0x7F):255-static_cast<int>(sc);
  }
  if(bsc==0xFD){
    best_score=0;
  }else{
    best_score=(bsc & 0x80)?-255+static_cast<int>(bsc & 0x7F):255-static_cast<int>(bsc);
  }
  //-------------------------------------------------------------------------
  
  if(stm==white && score>best_score){
    best_score=score;
  }else if(stm==black && score<best_score){
    best_score=score;
  }
  
  //-------------------------------------------------------------------------
  if(best_score==0){
    r=0xFD;
  }else if(best_score>0){
    r=static_cast<unsigned char>(255-best_score);
  }else{
    r=(0x80 | static_cast<unsigned char>(255+best_score));    
  }
  //-------------------------------------------------------------------------

  return r;
}
