/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#ifndef ENGINE_H
#define ENGINE_H

#include<string>
#include<thread>
#include"board/cposition.h"
#include"book/book.h"
#include"communicator.h"
#include"search/principal.h"
#include"search/multipv.h"
#include"search/threadpool.h"
using std::string;
using std::thread;

class Analysis;

class Engine
{
 private:
  Communicator& m_com;
  ChessPosition cp;
  TranspositionTable m_tt;
  ThreadPool m_tp;
  Analysis* pana;
  Hasher mhh;
  Book book;

  int m_id;
  MultiPV m_mpv;
  int m_st_time;
  int m_nps;
  string m_command;
  string m_previous_position;

  //----------------- options: ----------------
  bool m_debug{false};
  bool m_own_book{false};
  bool m_chess960{false};
  bool m_ponder{true};
  int m_memory;
  int m_threads{1};
  //-------------------------------------------
  void send_info(Analysis&);
  void send_hashfull(Analysis&);
  ofstream& m_ofs;
  bool m_own_book_available{true};
  enum status_t {e_idle,e_thinking,e_pondering,e_infinite} m_status;
  
 public:
  Engine(Communicator&,ofstream&);
  ~Engine();
  void set_action(char);
  bool equal_options(string,string);
  void from_console();
  void to_console();
};




#endif
