/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#ifndef MATERIALINDEX_H
#define MATERIALINDEX_H

#include"../movegen/move.h"
#include<vector>
#include<string>
using std::vector;
using std::string;


constexpr array<uint64_t,15> inc_table{0,1ull<<32,1ull<<36,1ull<<40,1ull<<44,1ull<<48,0,0,0,1,1<<4,1<<8,1<<12,1<<16,0};


class MaterialIndex
{
  // Q:4bits, R:4bits, B:4bits, N:4bits, P:4bits, q:4bits, r:4bits, b:4bits, n:4bits, p:4bits

 private:
  uint64_t mi{0};
  
 public:
  bool operator<(const MaterialIndex&)const;
  bool operator==(const MaterialIndex&)const;
  bool operator!=(const MaterialIndex&)const;
  void set_whitequeen(int);
  void set_whiterook(int);
  void set_whitebishop(int);
  void set_whiteknight(int);
  void set_whitepawn(int);
  void set_blackqueen(int);
  void set_blackrook(int);
  void set_blackbishop(int);
  void set_blackknight(int);
  void set_blackpawn(int);
  void add_piece(int);
  void remove_piece(int);
  void set_index(const uint64_t&);

  int get_whitequeen()const;
  int get_whiterook()const;
  int get_whitebishop()const;
  int get_whiteknight()const;
  int get_whitepawn()const;
  int get_blackqueen()const;
  int get_blackrook()const;
  int get_blackbishop()const;
  int get_blackknight()const;
  int get_blackpawn()const;

  bool equal()const;
  bool white_wins()const;
  bool black_wins()const;

  void flip_colors();
  string str()const;
  int value()const;
  string value_report()const;
  const uint64_t& index()const;

  void make_move(const Move&);
  void unmake_move(const Move&);

};


#endif
