/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"xmovelist.h"



XMoveList::XMoveList()
{
  m_ml[0].set_vm(m_pvm);
}

void XMoveList::clear_all()
{
  m_current=0;
  m_ml[m_current].clear();
  m_pvm=m_vm;
  m_ml[m_current].set_vm(m_pvm);
}

void XMoveList::increment()
{
  m_pvm+=m_ml[m_current].size();
  m_current++;
  m_ml[m_current].set_vm(m_pvm);
}

void XMoveList::decrement()
{
  m_current--;
  m_pvm=m_ml[m_current].get_vm();
}

Move& XMoveList::operator[](int x)
{
  return m_ml[m_current][x];
}

void XMoveList::set_chessposition(ChessPosition& x)
{
  m_ml[m_current].set_chessposition(x);
}

void XMoveList::set_analysis(Analysis* x)
{
  for(auto& ml : m_ml){
    ml.set_analysis(x);
  }
}

void XMoveList::set_searchmoves(const vector<Move>& x)
{
  m_ml[m_current].set_searchmoves(x);
}

void XMoveList::set_hashtable_move(const Move& x)
{
  m_ml[m_current].set_hashtable_move(x);
}

void XMoveList::set_killers(const Move& x,const Move& y)
{
  m_ml[m_current].set_killers(x,y);
}

void XMoveList::set_killers_2(const Move& x,const Move& y)
{
  m_ml[m_current].set_killers_2(x,y);
}

void XMoveList::set_log(bool x)
{
  m_ml[m_current].set_log(x);
}

void XMoveList::set_xcodes()
{
  m_ml[m_current].set_xcodes();
}

void XMoveList::sort()
{
  m_ml[m_current].sort();
}

void XMoveList::add(const Move& m)
{
  m_ml[m_current].add(m);
}

void XMoveList::remove(const Move& m)
{
  m_ml[m_current].remove(m);
}

void XMoveList::clear()
{
  m_ml[m_current].clear();
}

const Move& XMoveList::first()
{
  return m_ml[m_current].first();
}

const Move& XMoveList::next()
{
  return m_ml[m_current].next();
}

Move& XMoveList::back()
{
  return m_ml[m_current].back();
}

void XMoveList::insert(vector<Move>::iterator x,vector<Move>::iterator y)
{
  m_ml[m_current].insert(x,y);
}

void XMoveList::pop_back()
{
  m_ml[m_current].pop_back();
}

void XMoveList::push_back(const Move& x)
{
  m_ml[m_current].push_back(x);
}

unsigned int XMoveList::size()const
{
  return m_ml[m_current].size();
}

string XMoveList::dump()const
{
  return m_ml[m_current].dump();
}
