/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"timer.h"



Timer::Timer()
{
  reset();
}

void Timer::start()
{
  m_stopped=false;
  m_t0=high_resolution_clock::now();
}

void Timer::stop()
{
  if(!m_stopped){
    m_stopped=true;
    auto t1=high_resolution_clock::now();
    auto d=t1-m_t0;
    m_total_time+=d;
    m_counter++;
  }
}

void Timer::reset()
{
  m_stopped=true;
  duration<double> d0{0};
  m_total_time=d0;
  m_counter=0;
}

Duration Timer::total_time()             // sum of all (start,stop) periods
{
  if(m_stopped){
    return Duration(m_total_time);
  }
  else{
    stop();
    start();
    return Duration(m_total_time);
  }
}

Duration Timer::average_time()const      // average of (start,stop) periods
{
  duration<double,nano> d0{};
  if(m_counter>0)
    {
      return Duration(m_total_time/m_counter);
    }
  else 
    return Duration(d0);
}

Duration Timer::running_time()const       // since last start
{
  auto t1=high_resolution_clock::now();
  auto d=t1-m_t0;
  return Duration(d);
}

