/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"engvar.h"



EngineVariables::EngineVariables()
{
  set_default_values();
}

EngineVariables::~EngineVariables()
{
}

//-------------- set features: -------------------------
//
void EngineVariables::set_default_values()
{
  m_clock.set_level(40,"1",0);
  m_white=false;
  m_forced_mode=false;
  m_ping=false;
  m_setboard=false;
  m_playother=false;
  m_san=false;
  m_usermove=false;
  m_time=true;
  m_draw=true;
  m_sigint=true;
  m_sigterm=true;
  m_reuse=true;
  m_analyze=true;
  m_myname="";
  m_variants="normal";
  m_colors=true;
  m_ics=false;
  m_name=false;
  m_pause=false;
  m_nps=false;
  m_debug=false;
  m_memory=false;
  m_smp=false;
  m_egt="";
  m_option.clear();
  m_done=-1;
}

void EngineVariables::set_ping(bool x)
{
  m_ping=x;
}

void EngineVariables::EngineVariables::set_setboard(bool x)
{
  m_setboard=x;
}

void EngineVariables::set_playother(bool x)
{
  m_playother=x;
}

void EngineVariables::set_san(bool x)
{
  m_san=x;
}

void EngineVariables::set_usermove(bool x)
{
  m_usermove=x;
}

void EngineVariables::set_time(bool x)
{
  m_time=x;
}

void EngineVariables::set_draw(bool x)
{
  m_draw=x;
}

void EngineVariables::set_sigint(bool x)
{
  m_sigint=x;
}

void EngineVariables::set_sigterm(bool x)
{
  m_sigterm=x;
}

void EngineVariables::set_reuse(bool x)
{
  m_reuse=x;
}

void EngineVariables::set_analyze(bool x)
{
  m_analyze=x;
}

void EngineVariables::set_myname(string x)
{
  m_myname=x;
}

void EngineVariables::set_author(string x)
{
  m_author=x;
}

void EngineVariables::set_variants(string x)
{
  m_variants=x;
}

void EngineVariables::set_colors(bool x)
{
  m_colors=x;
}

void EngineVariables::set_ics(bool x)
{
  m_ics=x;
}

void EngineVariables::set_name(bool x)
{
  m_name=x;
}

void EngineVariables::set_pause(bool x)
{
  m_pause=x;
}

void EngineVariables::set_nps(bool x)
{
  m_nps=x;
}

void EngineVariables::set_debug(bool x)
{
  m_debug=x;
}

void EngineVariables::set_memory(bool x)
{
  m_memory=x;
}

void EngineVariables::set_smp(bool x)
{
  m_smp=x;
}

void EngineVariables::set_egt(string x)
{
  m_egt=x;
}

void EngineVariables::add_option(Option x)
{
  m_option[x.name()]=x;
}

void EngineVariables::set_options(map<string,Option> x)
{
  m_option=x;
}

void EngineVariables::set_done(int x)
{
  m_done=x;
}

void EngineVariables::set_uci(bool x)
{
  m_uci=x;
}

void EngineVariables::set_forced_mode(bool x)
{
  m_forced_mode=x;
}


//--------------------------- get features: -----------------
//
bool EngineVariables::ping()const
{
  return m_ping;
}

bool EngineVariables::setboard()const
{
  return m_setboard;
}

bool EngineVariables::playother()const
{
  return m_playother;
}

bool EngineVariables::san()const
{
  return m_san;
}

bool EngineVariables::usermove()const
{
  return m_usermove;
}

bool EngineVariables::time()const
{
  return m_time;
}

bool EngineVariables::draw()const
{
  return m_draw;
}

bool EngineVariables::sigint()const
{
  return m_sigint;
}

bool EngineVariables::sigterm()const
{
  return m_sigterm;
}

bool EngineVariables::reuse()const
{
  return m_reuse;
}

bool EngineVariables::analyze()const
{
  return m_analyze;
}

string EngineVariables::myname()const
{
  return m_myname;
}

string EngineVariables::author()const
{
  return m_author;
}

string EngineVariables::variants()const
{
  return m_variants;
}

bool EngineVariables::colors()const
{
  return m_colors;
}

bool EngineVariables::ics()const
{
  return m_ics;
}

bool EngineVariables::name()const
{
  return m_name;
}

bool EngineVariables::pause()const
{
  return m_pause;
}

bool EngineVariables::nps()const
{
  return m_nps;
}

bool EngineVariables::debug()const
{
  return m_debug;
}

bool EngineVariables::memory()const
{
  return m_memory;
}

bool EngineVariables::smp()const
{
  return m_smp;
}

string EngineVariables::egt()const
{
  return m_egt;
}

map<string,Option> EngineVariables::get_options()const
{
  return m_option;
}

int EngineVariables::done()const
{
  return m_done;
}

bool EngineVariables::uci()const
{
  return m_uci;
}

bool EngineVariables::forced_mode()const
{
  return m_forced_mode;
}

void EngineVariables::set_time(Duration x)
{
  m_clock.set_time(x);
}

Duration EngineVariables::get_time()
{
  return m_clock.get_time();
}

Duration EngineVariables::get_inc()const
{
  return m_clock.get_inc();
}

int EngineVariables::get_movestogo()const
{
  return m_clock.get_movestogo();
}

bool EngineVariables::white()const
{
  return m_white?true:false;
}

void EngineVariables::set_white(bool x)
{
  m_white=x;
}

void EngineVariables::start_clock()
{
  m_clock.start_clock();
}

void EngineVariables::stop_clock()
{
  m_clock.stop_clock();
}

void EngineVariables::reset_clock()
{
  m_clock.reset();
}

void EngineVariables::move_and_stop_clock()
{
  m_clock.move_and_stop_clock();
}

void EngineVariables::set_level(Level x)
{
  m_clock.set_level(x);
}

void EngineVariables::set_level(int mps,string base,int inc)
{
  m_clock.set_level(mps,base,inc);
}

Level EngineVariables::level()
{
  return m_clock.level();
}

string EngineVariables::get_level()
{
  return m_clock.get_level();
}

Move EngineVariables::pondermove()const
{
  return m_pondermove;
}

void EngineVariables::set_pondermove(Move m)
{
  m_pondermove=m;
}

EngineStatus EngineVariables::get_status()const
{
  return m_status;
}

void EngineVariables::set_status(EngineStatus x)
{
  m_status=x;
}

bool EngineVariables::readyok()const
{
  return m_readyok;
}

void EngineVariables::set_readyok(bool x)
{
  m_readyok=x;
}
