/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#ifndef GAME_H
#define GAME_H

#include"../engine/board/cposition.h"
#include"../engine/movegen/move.h"
#include<fstream>
#include<sstream>
#include<vector>
#include<string>
#include<map>
#include<utility>
using std::vector;
using std::string;
using std::pair;
using std::istringstream;
using std::map;


enum class mse_t {unknown,comment,move,nag,subvariation,termination};
struct MoveSectionEntry
{
  string entry;
  int color;
  int movenumber;
  mse_t type;
};


class Game
{
 private:
  ChessPosition cp;
  string m_tag_pair_section;
  map<string,string> m_tag;
  vector<string> m_tag_name;  
  vector<vector<MoveSectionEntry>> m_mse;
  bool m_setup{false};
  string m_initial_fen;
  int m_initial_color;
  int m_initial_movenumber;
  string m_result;
  pair<unsigned,unsigned> m_current;

  //------------------- private functions: ------------
  void parse_pgn(string);
  string basic_str(bool);
  string basic_str_tags();
  string basic_str_movesection(pair<unsigned,unsigned>,bool);
  void set_color_and_movenumber(unsigned,int,int);
  bool is_first_move()const;
  bool is_last_move()const;
 
 public:
  Game();
  Game(string);
  Game& operator=(const string&);
  void clear();
  bool empty()const;
  string get_result()const;
  void set_result(string r);
  void set_tag(string,string);
  string get_tag(string);
  string str();
  string comment_str();
  const ChessPosition& chessposition()const;
  string get_title();
  string get_current()const;

  //----------- get positions: -------------------------
  string get_first_position();
  string get_last_position();
  string get_previous_position();
  string get_next_position();
  //----------------------------------------------------

  
  //----------- get / set moves: -----------------------
  bool find_subvariation(Move);
  void add_move(Move);
  Move get_move(); 
  void add_comment(string);
  string get_comment();
  void make_opening(unsigned int);
  //----------------------------------------------------


  
  void comment();
  string dump();
};


bool symbolcharacter(char);
vector<MoveSectionEntry> extract_moves(string);


#endif
