/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"interface.h"
#include"option.h"
#include"piecechoicedialog.h"
#include"../engine/hashtable/ttrecord.h"
#include"../engine/movegen/mlist.h"
#include<fstream>
#include<iostream>
#include<iomanip>
#include<sstream>
#include<string>
#include<map>
#include<tuple>
#include<ctime>
#include<stdexcept>
using std::logic_error;
using std::cout;
using std::endl;
using std::tuple;
using std::get;
using std::istringstream;
using std::ostringstream;
using std::ios_base;
using std::boolalpha;
using Map=std::map<string,Option>;

  
Interface::Interface(Communicator& x, string name_gui) : master(x,name_gui), m_Box(Gtk::ORIENTATION_VERTICAL),
							 m_gamelist(*this), m_fenlist(*this), com(x)
{
  //----------------- timer: -------------------------------
  sigc::slot<bool> my_slot = sigc::bind(sigc::mem_fun(*this,&Interface::on_timeout),0);
  sigc::connection conn = Glib::signal_timeout().connect(my_slot,50);
  //--------------------------------------------------------
  thread t1(std::ref(master));
  m_t1=std::move(t1);
  m_last_click_timer.start();
  
  m_white=Player::human;
  m_black=Player::machine1;

  {
    ifstream ifs("galjoen.files/png/wood_l.png");
    if(ifs.good()){m_prefix.clear();}
    else{
      ifs.close();
      ifs.open("/usr/local/games/galjoen.files/png/wood_l.png");
      if(ifs.good()){
	m_prefix="/usr/local/games/";
      }
      else{
	m_prefix="/usr/games/";
      }
    }
  }
  m_sound=m_prefix+"galjoen.files/wav/galjoen.wav";
  m_playsound=m_prefix+"galjoen.files/playsound";
  
  cp.newgame();
  m_name_whiteplayer=m_name_blackplayer="*";
  m_game.clear();
  m_game.set_tag("Event","Galjoen battles");
  m_game.set_tag("Site","The North Sea");
  time_t t=time(nullptr);
  tm* pdate=gmtime(&t);
  ostringstream datess;
  datess << 1900+pdate->tm_year << "." << pdate->tm_mon + 1 << "." << pdate->tm_mday;
  m_game.set_tag("Date",datess.str());
  m_game.set_tag("Round","01");m_round_number=0;
  m_game.set_tag("White","?");
  m_game.set_tag("Black","?");
  m_game.set_tag("Result","*");
  
  draw_board();
  m_board.set_parent_window(this);


  set_title("Galjoen");
  set_border_width(5);
  add(m_Box);

  //------------------------------ Menu --------------------------------
  //
  m_refActionGroup = Gtk::ActionGroup::create();
  
  //File menu:
  m_refActionGroup->add(Gtk::Action::create("FileMenu", "_File"));
  m_refActionGroup->add(Gtk::Action::create("NewGame", "New game"),
			sigc::mem_fun(*this, &Interface::on_menu_new_game));
  m_refActionGroup->add(Gtk::Action::create("LoadGame", "Load game"),
			sigc::mem_fun(*this, &Interface::on_menu_load_game));
  m_refActionGroup->add(Gtk::Action::create("SaveGame", "Save game"),
			sigc::mem_fun(*this, &Interface::on_menu_save_game));
  m_refActionGroup->add(Gtk::Action::create("LoadPosition", "Load position"),
			sigc::mem_fun(*this, &Interface::on_menu_load_position));
  m_refActionGroup->add(Gtk::Action::create("LoadNextPosition", "Load next position"),Gtk::AccelKey("n"),
			sigc::mem_fun(*this, &Interface::on_menu_load_next_position));
  m_refActionGroup->add(Gtk::Action::create("LoadPreviousPosition", "Load previous position"),Gtk::AccelKey("p"),
			sigc::mem_fun(*this, &Interface::on_menu_load_previous_position));
  m_refActionGroup->add(Gtk::Action::create("SavePosition", "Save position"),
			sigc::mem_fun(*this, &Interface::on_menu_save_position));
  m_refActionGroup->add(Gtk::Action::create("Quit", "Quit"),
			sigc::mem_fun(*this, &Interface::on_menu_quit));
  
  //View menu:
  m_refActionGroup->add(Gtk::Action::create("ViewMenu", "View"));
  m_refActionGroup->add(Gtk::Action::create("Flip", "Flip View"),
			sigc::mem_fun(*this, &Interface::on_menu_flip));
  m_refActionGroup->add(Gtk::ToggleAction::create("ShowOutput", "Show Engine Output"),
			sigc::mem_fun(*this, &Interface::on_menu_show_output));
  m_refActionGroup->add(Gtk::ToggleAction::create("ShowPgnView", "Show PGN"),
			sigc::mem_fun(*this, &Interface::on_menu_show_pgnviewer));
  m_refActionGroup->add(Gtk::ToggleAction::create("ShowCommentEditor", "Show Comment Editor"),
			sigc::mem_fun(*this, &Interface::on_menu_show_commenteditor));
  m_refActionGroup->add(Gtk::ToggleAction::create("ShowEpdCommands", "Show EPD Code"),
			sigc::mem_fun(*this, &Interface::on_menu_show_epd_commands));
  
  //Mode menu:
  m_refActionGroup->add(Gtk::Action::create("ModeMenu", "Mode"));
  Gtk::RadioAction::Group group_mode;

  m_setup=Gtk::RadioAction::create(group_mode,"Setup", "Setup");
  m_refActionGroup->add(m_setup,sigc::mem_fun(*this, &Interface::on_menu_setup));

  m_human_computer=Gtk::RadioAction::create(group_mode,"HumanComputer", "Human - computer");
  m_refActionGroup->add(m_human_computer,sigc::mem_fun(*this, &Interface::on_menu_human_computer));

  m_human_human=Gtk::RadioAction::create(group_mode,"HumanHuman", "Human - human");
  m_refActionGroup->add(m_human_human,sigc::mem_fun(*this, &Interface::on_menu_human_human));

  m_computer_computer=Gtk::RadioAction::create(group_mode,"ComputerComputer", "Match between engines ...");
  m_refActionGroup->add(m_computer_computer,sigc::mem_fun(*this, &Interface::on_menu_computer_computer));

  /*
  m_tournament=Gtk::RadioAction::create(group_mode,"Tournament", "Tournament ...");
  m_refActionGroup->add(m_tournament,sigc::mem_fun(*this, &Interface::on_menu_tournament));
  m_analyze_position=Gtk::RadioAction::create(group_mode,"AnalyzePosition", "Analyze position");
  m_refActionGroup->add(m_analyze_position,sigc::mem_fun(*this, &Interface::on_menu_analyze_position));
  */

  m_refActionGroup->add(Gtk::Action::create("StartThinking", "Start thinking"),
			sigc::mem_fun(*this, &Interface::on_menu_start_thinking));
  
  //Engine menu:
  m_refActionGroup->add(Gtk::Action::create("EngineMenu", "Engine"));
  m_refActionGroup->add(Gtk::Action::create("Engine1", "Load engine 1..."),
			sigc::mem_fun(*this, &Interface::on_menu_engine1));
  m_refActionGroup->add(Gtk::Action::create("Engine2", "Load engine 2..."),
			sigc::mem_fun(*this, &Interface::on_menu_engine2));
    
  //Option menu:
  m_refActionGroup->add(Gtk::Action::create("OptionMenu", "Options"));
  m_refActionGroup->add(Gtk::Action::create("TimeControl", "Time Control . . ."),
			sigc::mem_fun(*this, &Interface::on_menu_timecontrol));
  m_refActionGroup->add(Gtk::Action::create("Engine1_options", "Engine 1 . . ."),
			sigc::mem_fun(*this, &Interface::on_menu_engine1_options));
  m_refActionGroup->add(Gtk::Action::create("Engine2_options", "Engine 2 . . ."),
			sigc::mem_fun(*this, &Interface::on_menu_engine2_options));

  m_ponder=Gtk::ToggleAction::create("Ponder", "Ponder");
  m_refActionGroup->add(m_ponder,sigc::mem_fun(*this, &Interface::on_menu_ponder));
  m_refActionGroup->add(Gtk::ToggleAction::create("Debug", "Debug"),sigc::mem_fun(*this, &Interface::on_menu_debug));

  //Info menu:
  m_refActionGroup->add(Gtk::Action::create("InfoMenu", "Info"));
  m_refActionGroup->add(Gtk::Action::create("About", "About..."),
			sigc::mem_fun(*this, &Interface::on_menu_about));


  //-------------------------------------------------------------------------------------------------------------------
  m_refUIManager = Gtk::UIManager::create();
  m_refUIManager->insert_action_group(m_refActionGroup);  
  add_accel_group(m_refUIManager->get_accel_group());

  //Layout the actions in a menubar and toolbar:
  Glib::ustring ui_info = 
    "<ui>"
    "  <menubar name='MenuBar'>"
    "    <menu action='FileMenu'>"
    "        <menuitem action='NewGame'/>"
    "        <separator/>"
    "        <menuitem action='LoadGame'/>"
    "        <menuitem action='SaveGame'/>"
    "        <separator/>"
    "        <menuitem action='LoadPosition'/>"
    "        <menuitem action='LoadNextPosition'/>"
    "        <menuitem action='LoadPreviousPosition'/>"
    "        <menuitem action='SavePosition'/>"
    "        <separator/>"
    "        <menuitem action='Quit'/>"
    "    </menu>"
    "    <menu action='ViewMenu'>"
    "        <menuitem action='Flip'/>"
    "        <menuitem action='ShowOutput'/>"
    "        <menuitem action='ShowPgnView'/>"
    "        <menuitem action='ShowCommentEditor'/>"
    "        <menuitem action='ShowEpdCommands'/>"
    "    </menu>"
    "    <menu action='ModeMenu'>"
    "        <menuitem action='Setup'/>"
    "        <menuitem action='HumanHuman'/>"
    "        <menuitem action='HumanComputer'/>"
    "        <menuitem action='ComputerComputer'/>"
    //"        <menuitem action='Tournament'/>"
    //"        <menuitem action='AnalyzePosition'/>"
    "        <separator/>"
    "        <menuitem action='StartThinking'/>"
    "    </menu>"
    "    <menu action='EngineMenu'>"
    "        <menuitem action='Engine1'/>"
    "        <menuitem action='Engine2'/>"
    "    </menu>"
    "    <menu action='OptionMenu'>"
    "        <menuitem action='TimeControl'/>"
    "        <menuitem action='Engine1_options'/>"
    "        <menuitem action='Engine2_options'/>"
    "        <menuitem action='Ponder'/>"
    "        <menuitem action='Debug'/>"
    "    </menu>"
    "    <menu action='InfoMenu'>"
    "        <menuitem action='About'/>"
    "    </menu>"
    "  </menubar>"
    "</ui>";
  
  try
    {
      m_refUIManager->add_ui_from_string(ui_info);
    }
  catch(const Glib::Error& ex)
    {
      std::cerr << "building menus failed: " <<  ex.what();
    }

  //Get the menubar and toolbar widgets, and add them to a container widget:
  Gtk::Widget* pMenubar = m_refUIManager->get_widget("/MenuBar");
  if(pMenubar){
    m_Box.pack_start(*pMenubar, Gtk::PACK_SHRINK);  
  }
  m_human_computer->set_active();
  //
  //------------------------------------------------------------------

  {
    //------------------ clocks: -----------------------------
    m_Box.pack_start(m_clocks, Gtk::PACK_EXPAND_PADDING);
    m_clocks.show();
  }

  {
    //------------------ board: ----------------------------    
    m_board.signal_piece_moved.connect(sigc::mem_fun(*this,&Interface::on_piece_moved));
    m_Box.pack_start(m_board, Gtk::PACK_SHRINK);
  }

  {
    //----- navigation buttons: --------------------------------
    m_Box.pack_start(m_navigation_box, Gtk::PACK_SHRINK);
    m_navigation_box.pack_start(m_goto_first, Gtk::PACK_SHRINK);
    m_navigation_box.pack_start(m_go_backward, Gtk::PACK_SHRINK);
    m_navigation_box.pack_start(m_go_forward, Gtk::PACK_SHRINK);
    m_navigation_box.pack_start(m_goto_last, Gtk::PACK_SHRINK);
    m_navigation_box.set_layout(Gtk::BUTTONBOX_SPREAD);
    //m_navigation_box.set_layout(Gtk::BUTTONBOX_START);
    m_navigation_box.set_spacing(5);
    string filename=m_prefix+"galjoen.files/png/goto_first.png";
    m_image_goto_first.set(filename.c_str());
    filename=m_prefix+"galjoen.files/png/go_backward.png";
    m_image_go_backward.set(filename.c_str());
    filename=m_prefix+"galjoen.files/png/go_forward.png";
    m_image_go_forward.set(filename.c_str());
    filename=m_prefix+"galjoen.files/png/goto_last.png";
    m_image_goto_last.set(filename.c_str());
    m_goto_first.set_image(m_image_goto_first);
    m_go_backward.set_image(m_image_go_backward);
    m_go_forward.set_image(m_image_go_forward);
    m_goto_last.set_image(m_image_goto_last);
    m_goto_first.signal_clicked().connect(sigc::mem_fun(*this,&Interface::on_menu_back_to_start));
    m_go_backward.signal_clicked().connect(sigc::mem_fun(*this,&Interface::on_menu_backward));
    m_go_forward.signal_clicked().connect(sigc::mem_fun(*this,&Interface::on_menu_forward));
    m_goto_last.signal_clicked().connect(sigc::mem_fun(*this,&Interface::on_menu_forward_to_end));
  }

  //----- opening book used in matches between two engines: -----------------------
  m_book.signal_clicked().connect(sigc::mem_fun(*this,&Interface::on_opening_book));
  //---------------------------------------------------------------------------------

  show_all_children();

  m_clocks.set_level(0,"1:00",0);
  m_clocks.update_clock_images();
  string lev{"level 0 60 0"};
  com.to_engine(lev);
}

Interface::~Interface()
{
  m_t1.join();
}

bool Interface::on_piece_moved()
{
  if(m_last_click_timer.running_time().milliseconds()<500){return true;}
  pair<int,int> move=m_board.get_move();
  if(m_setup_position){
    Square sq1{static_cast<char>(move.first)};
    Square sq2{static_cast<char>(move.second)};
    Piece p=cp.get_piece_from(sq1);
    if(sq1==sq2){
      if(p!=NoPiece()){
	cp.remove_piece_from(sq1,p);
      }else{
	auto images=m_board.get_images();
	PieceChoiceDialog pcd(this,images);
	int choice=pcd.get_choice();
	m_last_click_timer.start();
	switch(choice)
	  {
	  case 1:p=WhitePawn();break;
	  case 2:p=WhiteKnight();break;
	  case 3:p=WhiteBishop();break;
	  case 4:p=WhiteRook();break;
	  case 5:p=WhiteQueen();break;
	  case 6:p=WhiteKing();break;
	  case 8:p=BlackPawn();break;
	  case 9:p=BlackKnight();break;
	  case 10:p=BlackBishop();break;
	  case 11:p=BlackRook();break;
	  case 12:p=BlackQueen();break;
	  case 13:p=BlackKing();break;
	  }
	cp.set_piece_on(sq1,p);
      }
    }else{
      Piece p2=cp.get_piece_from(sq2);
      if(p2!=NoPiece()){cp.remove_piece_from(sq2,p2);}
      cp.set_piece_on(sq2,p);
      cp.remove_piece_from(sq1,p);
    }
    if(p.white_piece() && cp.white_to_move()){cp.switch_color_to_move();}
    if(p.black_piece() && !cp.white_to_move()){cp.switch_color_to_move();}
    if(cp.white_to_move()){m_clocks.set_white_to_move();}else{m_clocks.set_black_to_move();}
    m_game.set_tag("SetUp","1");
    m_game.set_tag("FEN",cp.fen());
    string sfen="setboard " + cp.fen();
    com.to_engine(sfen);
  }
  else if((cp.white_to_move() && m_white==Player::human)
	  ||(!cp.white_to_move() && m_black==Player::human)){
    unsigned int promo{0};
    switch(move.second>>6)
      {
      case 4:promo=4;break;
      case 5:promo=5;break;
      case 6:promo=6;break;
      case 7:promo=7;break;
      default:promo=0;
      }
    Square sq1{static_cast<char>(move.first)};
    Square sq2{static_cast<char>(move.second & 0x3f)};
    MList ml(true);cp.generate_moves(ml);
    for(unsigned i=0;i<ml.size();i++){
      auto& m=ml[i];
      string im=sq1.name()+sq2.name();
      if(
	 (
	  m.from()==move.first && m.to()==(move.second & 0x3f) 
	  && ((m.code()==promo_queen && promo==4)
	      || (m.code()==promo_rook && promo==5)
	      || (m.code()==promo_bishop && promo==6)
	      || (m.code()==promo_knight && promo==7)
	      || m.code()==normal_move
	      || m.code()==castling
	      || m.code()==en_passant
	      )
	  )
	 ||
	 (im==m.algebraic()
	  )
	 )
	{
	  if(!m_force){
	    if(cp.white_to_move()){m_clocks.start_black_clock();}else{m_clocks.start_white_clock();}
	  }
	  try{
	    cp.make_move(m);
	  }
	  catch(...){
	    std::cerr << "make move failed" << endl;throw;
	  }
	  string command="usermove ";command+=m.algebraic();
	  com.to_engine(command);
	  m_game.add_comment(m_commenteditor.get_text());
	  m_game.add_move(m);
	  m_commenteditor.set_text(m_game.get_comment());
	  m_pgnview.set_text(m_game.comment_str());
	  if(m_eco.eco_code(cp.get_hash())!=""){
	    m_game.set_tag("ECO",m_eco.eco_code(cp.get_hash()));
	    m_game.set_tag("Opening",m_eco.eco_opening(cp.get_hash()));
	  }
	  break;
	}
    }
  }
  if(m_force){
    if(cp.white_to_move()){m_clocks.set_white_to_move();}else{m_clocks.set_black_to_move();}
  }
  draw_board(); 
  return true;
}


bool Interface::on_timeout(int x)
{
  m_clocks.update_clock_images();
  get_master_command();
  return true;
}

bool Interface::on_delete_event(GdkEventAny* E)
{
  string s{"quit"};
  com.to_engine(s);
  return true;
}

void Interface::get_master_command()
{
  string command,s1,s2,s3,s4;
  Move m;
  string master_command;
  do{
    master_command=com.from_engine();
    if(master_command.size()>0)
      {
	istringstream istr(master_command);
	istr >> command;
	if(command=="move")
	  {
	    istr >> s1;
	    m=cp.convert_to_move(s1);
	    m_game.add_move(m);
	    if(cp.white_to_move()){
	      m_game.add_comment(m_white_comment);
	      m_white_comment.clear();
	      m_clocks.start_black_clock();
	      if(m_name_whiteplayer==m_name_engine1){
		m_analysis_window_engine2.clear();
	      }else{
		m_analysis_window_engine1.clear();
	      }
	    }
	    else{
	      m_game.add_comment(m_black_comment);
	      m_black_comment.clear();
	      m_clocks.start_white_clock();
	      if(m_name_whiteplayer==m_name_engine1){
		m_analysis_window_engine1.clear();
	      }else{
		m_analysis_window_engine2.clear();
	      }
	    }
	    cp.make_move(m);
	    if(m_eco.eco_code(cp.get_hash())!=""){
	      m_game.set_tag("ECO",m_eco.eco_code(cp.get_hash()));
	      m_game.set_tag("Opening",m_eco.eco_opening(cp.get_hash()));
	    }
	    m_pgnview.set_text(m_game.comment_str());
	    draw_board();
	  }
	else if(command=="setboard")
	  {
	    istr >> s1 >> s2 >> s3 >> s4;
	    string sfen=s1+" "+s2+" "+s3+" "+s4;
	    cp.fen(sfen);
	    m_game.set_tag("SetUp","1");
	    m_game.set_tag("FEN",cp.fen());
	    if(cp.white_to_move()){
	      m_game.add_comment(m_white_comment);
	      m_white_comment.clear();
	      m_clocks.start_black_clock();
	    }
	    else{
	      m_game.add_comment(m_black_comment);
	      m_black_comment.clear();
	      m_clocks.start_white_clock();
	    }
	    m_analysis_window_engine1.clear();
	    m_analysis_window_engine2.clear();
	    draw_board();
	  }
	else if(command=="shout")
	  {
	    //removed
	  }
	else if(command=="level")
	  {
	    int mps;
	    float inc;
	    string base;
	    istr >> mps >> base >> inc;
	    m_clocks.set_level(mps,base,inc);
	    m_clocks.update_clock_images();
	  }
	else if(command=="wtime")
	  {
	    int t_milliseconds,t_centiseconds;
	    istr >> t_milliseconds;
	    t_centiseconds=t_milliseconds/10;
	    m_clocks.set_white_clock(t_centiseconds);
	  }
	else if(command=="btime")
	  {
	    int t_milliseconds,t_centiseconds;
	    istr >> t_milliseconds;
	    t_centiseconds=t_milliseconds/10;
	    m_clocks.set_black_clock(t_centiseconds);
	  }
	else if(command=="telluser")
	  {
	    string s1,s2;
	    istr >> s1;
	    istr >> s2;
	    telluser(s1,s2);
	  }
	else if(command=="engine1name")
	  {
	    string s1;
	    getline(istr,s1);
	    deque<char> temp;
	    for(auto c : s1){temp.push_back(c);}
	    s1.clear();
	    if(!temp.empty()){while(temp.front()==' '){temp.pop_front();}}
	    if(!temp.empty()){while(temp.back()==' '){temp.pop_back();}}
	    for(auto c : temp){s1.push_back(c);}
	    if(s1.empty()){s1="???";}
	    m_name_engine1=s1;
	  }
	else if(command=="engine2name")
	  {
	    string s1;
	    getline(istr,s1);
	    deque<char> temp;
	    for(auto c : s1){temp.push_back(c);}
	    s1.clear();
	    if(!temp.empty()){while(temp.front()==' '){temp.pop_front();}}
	    if(!temp.empty()){while(temp.back()==' '){temp.pop_back();}}
	    for(auto c : temp){s1.push_back(c);}
	    if(s1.empty()){s1="???";}
	    m_name_engine2=s1;
	  }
	else if(command=="whitename")
	  {
	    string s1;
	    getline(istr,s1);
	    deque<char> temp;
	    for(auto c : s1){temp.push_back(c);}
	    s1.clear();
	    if(!temp.empty()){while(temp.front()==' '){temp.pop_front();}}
	    if(!temp.empty()){while(temp.back()==' '){temp.pop_back();}}
	    for(auto c : temp){s1.push_back(c);}
	    if(s1.empty()){s1="???";}
	    m_name_whiteplayer=s1;
	    string title=m_name_whiteplayer + "  " + m_name_blackplayer;
	    set_title(title);
	    if(m_name_whiteplayer==m_name_engine1){
	      m_analysis_window_engine1.set_title(m_name_whiteplayer);
	    }else{
	      m_analysis_window_engine2.set_title(m_name_whiteplayer);
	    }
	  }
	else if(command=="blackname")
	  {
	    string s1;
	    getline(istr,s1);
	    deque<char> temp;
	    for(auto c : s1){temp.push_back(c);}
	    s1.clear();
	    if(!temp.empty()){while(temp.front()==' '){temp.pop_front();}}
	    if(!temp.empty()){while(temp.back()==' '){temp.pop_back();}}
	    for(auto c : temp){s1.push_back(c);}
	    if(s1.empty()){s1="???";}
	    m_name_blackplayer=s1;
	    string title=m_name_whiteplayer + "  " + m_name_blackplayer;
	    set_title(title);
	    if(m_name_blackplayer==m_name_engine1){
	      m_analysis_window_engine1.set_title(m_name_blackplayer);
	    }else{
	      m_analysis_window_engine2.set_title(m_name_blackplayer);
	    }
	  }
	else if(command=="new")
	  {
	    m_force=false;
	  
	    cp.newgame();
	    draw_board();
	    m_game.clear();
	    m_white_comment.clear();
	    m_black_comment.clear();
	    m_game.set_tag("Event","Galjoen battles");
	    m_game.set_tag("Site","the North Sea");
	    time_t t=time(nullptr);
	    tm* pdate=gmtime(&t);
	    {
	      ostringstream oss;
	      oss << 1900+pdate->tm_year << "." << pdate->tm_mon + 1 << "." << pdate->tm_mday;
	      m_game.set_tag("Date",oss.str());
	    }
	    {
	      ostringstream oss;
	      oss << ++m_round_number;
	      m_game.set_tag("Round",oss.str());
	    }
	    m_game.set_tag("White",m_name_whiteplayer);
	    m_game.set_tag("Black",m_name_blackplayer);
	    m_game.set_tag("Result","*");
	    m_game.set_result("*");
	    m_clocks.stop_clocks();
	    m_clocks.reset();
	  }
	else if(command=="quit")
	  {
	    hide();
	  }
	else if(command=="0-1" || command=="1-0" || command=="1/2-1/2")
	  {
	    string comment;
	    getline(istr,comment);
	    command+=comment;
	    if(!m_game.empty()){
	      m_game.set_tag("White",m_name_whiteplayer);
	      m_game.set_tag("Black",m_name_blackplayer);
	      m_game.set_tag("TimeControl",m_level);
	      m_game.set_result(command);
	      istringstream iss(command);
	      string res;iss >> res;
	      m_game.set_tag("Result",res);
	      m_pgnview.set_text(m_game.comment_str());
	      PgnFile pgnfile;
	      pgnfile.open_output("Galjoen.pgn");
	      pgnfile.add_game(m_game.comment_str());
	      pgnfile.close_output();
	    }
	  }
	else if(command=="white_info")
	  {
	    string s;
	    getline(istr,s);
	    if(m_name_whiteplayer==m_name_engine1){
	      m_analysis_window_engine1.add_text(s+"\n");
	    }else{
	      m_analysis_window_engine2.add_text(s+"\n");
	    }
	    istringstream iss(s);
	    string name,value_time,value_nodes,value_depth,value_cp,value_mate,value_hashfull;
	    iss >> name;
	    while(!iss.eof()){
	      if(name=="score"){
		iss >> name;
		if(name=="cp"){
		  iss >> value_cp;
		}
		else if(name=="mate"){
		  iss >> value_mate;
		}
	      }else if(name=="time"){
		iss >> value_time;
	      }else if(name=="nodes"){
		iss >> value_nodes;
	      }else if(name=="depth"){
		iss >> value_depth;
	      }else if(name=="hashfull"){
		iss >> value_hashfull;
	      }
	      iss >> name;
	    }
	    if(value_cp.size()>0){m_white_comment="cp "+value_cp;}
	    if(value_mate.size()>0){m_white_comment="mate "+value_mate;}
	    if(value_time.size()>0){m_white_comment+=" time "+value_time;}
	    if(value_nodes.size()>0){m_white_comment+=" nodes "+value_nodes;}
	    if(value_nodes.size()>0){m_white_comment+=" depth "+value_depth;}
	  }
	else if(command=="black_info")
	  {
	    string s;
	    getline(istr,s);
	    if(m_name_blackplayer==m_name_engine1){
	      m_analysis_window_engine1.add_text(s+"\n");
	    }else{
	      m_analysis_window_engine2.add_text(s+"\n");
	    }
	    
	    istringstream iss(s);
	    string name,value_time,value_nodes,value_depth,value_cp,value_mate,value_hashfull;
	    iss >> name;
	    while(!iss.eof()){
	      if(name=="score"){
		iss >> name;
		if(name=="cp"){
		  iss >> value_cp;
		}
		else if(name=="mate"){
		  iss >> value_mate;
		}
	      }else if(name=="time"){
		iss >> value_time;
	      }else if(name=="nodes"){
		iss >> value_nodes;
	      }else if(name=="depth"){
		iss >> value_depth;
	      }else if(name=="hashfull"){
		iss >> value_hashfull;
	      }
	      iss >> name;
	    }
	    if(value_cp.size()>0){m_black_comment="cp "+value_cp;}
	    if(value_mate.size()>0){m_black_comment="mate "+value_mate;}
	    if(value_time.size()>0){m_black_comment+=" time "+value_time;}
	    if(value_nodes.size()>0){m_black_comment+=" nodes "+value_nodes;}
	    if(value_nodes.size()>0){m_black_comment+=" depth "+value_depth;}
	  }
      }
  }
  while(master_command.size()>0);
}



void Interface::on_menu_new_game()
{
  cp.newgame();
  draw_board();
  com.to_engine("new");
  m_game.clear();
  m_white=Player::human;
  m_black=Player::machine1;
  m_white_comment.clear();
  m_black_comment.clear();
  m_human_computer->set_active();
}

void Interface::on_menu_load_position()
{
  m_human_human->set_active();
  m_fen_filename=file_chooser(OpenFen);
  if(m_fen_filename=="")return;
  ifstream ifs(m_fen_filename);
  string sfen;
  m_chosen_fen=m_fenlist.get_selection(m_fen_filename);
  for(int i=0;i<m_chosen_fen;i++){
    getline(ifs,sfen);
  }
  if(!is_epd(sfen)){return;}
  m_fen=sfen;
  m_game.clear();
  m_game.set_tag("Event","Galjoen battles");
  m_game.set_tag("Site","the North Sea");
  time_t t=clock();
  tm* pdate=gmtime(&t);
  ostringstream datess;
  datess << 1900+pdate->tm_year << "." << pdate->tm_mon + 1 << "." << pdate->tm_mday;
  m_game.set_tag("Date",datess.str());
  m_game.set_tag("Round","01");m_round_number=1;
  m_game.set_tag("White","?");
  m_game.set_tag("Black","?");
  m_game.set_tag("Result","*");
  m_game.set_tag("SetUp","1");
  cp.fen(sfen);
  m_game.set_tag("FEN",cp.fen());
  draw_board();
  sfen="setboard " + cp.fen();
  com.to_engine(sfen);
  ifs.close();
  if(cp.white_to_move()){m_clocks.set_white_to_move();}else{m_clocks.set_black_to_move();}
  m_epd_commands_window.set_text(m_fen);
}

void Interface::on_menu_load_next_position()
{
  if(m_computer_computer->get_active()){return;}
  m_human_human->set_active();
  ifstream ifs(m_fen_filename);
  m_chosen_fen++;
  string sfen;
  for(int i=0;i<m_chosen_fen;i++){
    getline(ifs,sfen);
    if(ifs.eof()){m_chosen_fen--;return;}
  }
  if(!is_epd(sfen)){return;}
  m_fen=sfen;
  m_game.clear();
  m_game.set_tag("Event","Galjoen battles");
  m_game.set_tag("Site","the North Sea");
  time_t t=clock();
  tm* pdate=gmtime(&t);
  ostringstream datess;
  datess << 1900+pdate->tm_year << "." << pdate->tm_mon + 1 << "." << pdate->tm_mday;
  m_game.set_tag("Date",datess.str());
  m_game.set_tag("Round","01");m_round_number=1;
  m_game.set_tag("White","?");
  m_game.set_tag("Black","?");
  m_game.set_tag("Result","*");
  m_game.set_tag("SetUp","1");
  cp.fen(sfen);
  m_game.set_tag("FEN",cp.fen());
  draw_board();
  sfen="setboard " + cp.fen();
  com.to_engine(sfen);
  ifs.close();
  if(cp.white_to_move()){m_clocks.set_white_to_move();}else{m_clocks.set_black_to_move();}
  m_epd_commands_window.set_text(m_fen);
}

void Interface::on_menu_load_previous_position()
{
  if(m_computer_computer->get_active()){return;}
  m_human_human->set_active();
  ifstream ifs(m_fen_filename);
  m_chosen_fen--;
  if(m_chosen_fen<=0){m_chosen_fen=1;}
  string sfen;
  for(int i=0;i<m_chosen_fen;i++){
    getline(ifs,sfen);
  }
  if(!is_epd(sfen)){return;}
  m_fen=sfen;
  m_game.clear();
  m_game.set_tag("Event","Galjoen battles");
  m_game.set_tag("Site","the North Sea");
  time_t t=clock();
  tm* pdate=gmtime(&t);
  ostringstream datess;
  datess << 1900+pdate->tm_year << "." << pdate->tm_mon + 1 << "." << pdate->tm_mday;
  m_game.set_tag("Date",datess.str());
  m_game.set_tag("Round","01");m_round_number=1;
  m_game.set_tag("White","?");
  m_game.set_tag("Black","?");
  m_game.set_tag("Result","*");
  m_game.set_tag("SetUp","1");
  cp.fen(sfen);
  m_game.set_tag("FEN",cp.fen());
  draw_board();
  sfen="setboard " + cp.fen();
  com.to_engine(sfen);
  ifs.close();
  if(cp.white_to_move()){m_clocks.set_white_to_move();}else{m_clocks.set_black_to_move();}
  m_epd_commands_window.set_text(m_fen);
}

void Interface::on_menu_load_game()
{
  m_human_human->set_active();
  string filename=file_chooser(OpenPgn);
  if(filename=="")return;
  PgnFile pgnfile;

  // load game:
  pgnfile.open_input(filename);
  string sgame=pgnfile.get_first_game();
  int chosen_game=m_gamelist.get_selection(filename);
  for(int i=1;i<chosen_game;i++){
    sgame=pgnfile.get_next_game();
  }
  pgnfile.close_input();

  try{
    m_game=sgame;
    string s=m_game.get_title();
    set_title(s);
    s=m_game.get_last_position();
    cp.fen(s);
    draw_board();
    s="setboard " + cp.fen();
    com.to_engine(s);
  }
  catch(...){
    std::cerr << "# invalid game: " << endl << sgame << endl;
  }
}

void Interface::on_menu_save_position()
{
  string filename=file_chooser(SaveFen);
  if(filename=="")return;
  ofstream ofs(filename,ios_base::out);
  ofs << cp.fen() << endl;
  ofs.close();
}

void Interface::on_menu_save_game()
{
  PgnFile pgnfile;
  string filename=file_chooser(SavePgn);
  pgnfile.open_output(filename);
  pgnfile.add_game(m_game.comment_str());
  pgnfile.close_output();
}

void Interface::on_menu_quit()
{
  string s{"quit"};
  com.to_engine(s);
}

void Interface::on_menu_backward()
{
  if(m_white!=Player::human && m_black!=Player::human){return;}
  m_force=true;
  m_human_human->set_active();
  m_game.add_comment(m_commenteditor.get_text());
  string s=m_game.get_previous_position();
  m_commenteditor.set_text(m_game.get_comment());
  m_pgnview.set_text(m_game.comment_str());
  if(s!=""){
    cp.fen(s);
    draw_board();
    s="setboard " + cp.fen();
    com.to_engine(s);
  }
}

void Interface::on_menu_forward()
{
  if(m_white!=Player::human && m_black!=Player::human){return;}
  m_force=true;
  m_human_human->set_active();
  m_game.add_comment(m_commenteditor.get_text());
  string s=m_game.get_next_position();
  m_commenteditor.set_text(m_game.get_comment());
  m_pgnview.set_text(m_game.comment_str());
  if(s!=""){
    cp.fen(s);
    draw_board();
    s="setboard " + cp.fen();
    com.to_engine(s);
  }
}

void Interface::on_menu_back_to_start()
{
  if(m_white!=Player::human && m_black!=Player::human){return;}
  m_force=true;
  m_human_human->set_active();
  m_game.add_comment(m_commenteditor.get_text());
  string s=m_game.get_first_position();
  m_commenteditor.set_text(m_game.get_comment());
  m_pgnview.set_text(m_game.comment_str());
  if(s!=""){
    cp.fen(s);
    draw_board();
    s="setboard " + cp.fen();
    com.to_engine(s);
  }
}

void Interface::on_menu_forward_to_end()
{
  if(m_white!=Player::human && m_black!=Player::human){return;}
  m_force=true;
  m_human_human->set_active();
  m_human_human->set_active();
  m_game.add_comment(m_commenteditor.get_text());
  string s=m_game.get_last_position();
  m_commenteditor.set_text(m_game.get_comment());
  m_pgnview.set_text(m_game.comment_str());
  cp.fen(s);
  draw_board();
  s="setboard " + cp.fen();
  com.to_engine(s);
}

void Interface::on_menu_setup()
{
  if(m_setup->get_active()){
    m_setup_position=true;
    cp.set_castlingrights(0);
  }else{
    m_setup_position=false;
    string s="setboard " + cp.fen();
    com.to_engine(s);
  }
}

void Interface::on_menu_human_human()
{
  if(m_human_human->get_active()){
    m_white=Player::human;
    m_black=Player::human;
    m_clocks.stop_clocks();
    m_force=true;
    com.to_engine("force");
    if(cp.white_to_move()){m_clocks.set_white_to_move();}else{m_clocks.set_black_to_move();}
  }
}

void Interface::on_menu_human_computer()
{
  if(m_human_computer->get_active()){
    if(cp.white_to_move()){m_white=Player::human;m_black=Player::machine1;}
    else{m_white=Player::machine1;m_black=Player::human;}
    m_force=false;
    com.to_engine("playother");
    if(cp.white_to_move()){m_clocks.set_white_to_move();}else{m_clocks.set_black_to_move();}
  }
}

void Interface::on_menu_analyze_position()
{
  /*
  if(m_analyze_position->get_active()){
    if(cp.white_to_move()){m_white=Player::human;m_black=Player::machine1;}
    else{m_white=Player::machine1;m_black=Player::human;}
    m_force=true;
    com.to_engine("analyzeposition");
  }
  */
}

void Interface::on_menu_computer_computer()
{
  if(m_computer_computer->get_active()){
    m_white=Player::machine1;
    m_black=Player::machine2;
    
    Gtk::Dialog dialog;
    dialog.set_transient_for(*this);
    dialog.set_title("Match");
    dialog.set_border_width(20);
    
    dialog.add_button("CANCEL", 0);
    dialog.add_button("OK", 1);
    
    Gtk::Box* pcontent=dialog.get_content_area();
    Gtk::Box  line_path(Gtk::ORIENTATION_HORIZONTAL);
    Gtk::Label label1("Number of games: ");
    Gtk::Entry ent1;
    ent1.set_width_chars(6);
    ent1.set_alignment(1);
    m_book.set_label("Select  PGN/EPD book");
    
    line_path.pack_start(label1);
    line_path.pack_start(ent1);
    pcontent->pack_start(line_path);
    pcontent->pack_start(m_book);
    pcontent->show_all_children();
    
    int result = dialog.run();
    
    switch(result)
      {
      case 1: // OK
	{
	  string nog=ent1.get_text();
	  string cmd="match ";
	  try{stoi(nog);}catch(...){nog="2";}
	  cmd+=nog;
	  m_force=false;
	  com.to_engine(cmd);
	  return;
	}
      case 0: // CANCEL
	{
	  m_human_human->set_active();com.to_engine("force");
	  return;
	}
      default:return;
      }
  }
}

void Interface::on_menu_tournament()
{
  std::cerr << "on menu tournament" << endl;
}

void Interface::on_menu_start_thinking()
{
  if(!m_human_computer->get_active()){return;}
  if(cp.white_to_move()){m_white=Player::machine1;m_black=Player::human;}
  else{m_white=Player::human;m_black=Player::machine1;}
  m_force=false;
  com.to_engine("go");
  if(cp.white_to_move()){m_clocks.start_white_clock();}else{m_clocks.start_black_clock();}
}

void Interface::on_opening_book()
{
  string name=file_chooser(OpenPgnEpd);
  string cmd="book ";
  cmd+=name;
  com.to_engine(cmd);
  return;
}

void Interface::on_menu_timecontrol()
{
  tuple<int,int,string,float> tmcontrol=time_control();
  int mps=get<1>(tmcontrol);
  string base=get<2>(tmcontrol);
  float inc=get<3>(tmcontrol);
  m_clocks.set_level(mps,base,inc);
  m_clocks.update_clock_images();

  // 
  ostringstream os1;
  os1 << mps << " " << base << " " << inc;
  m_level=os1.str();

  // convert string base to integer (seconds):
  int minutes{0},seconds{0};
  int x=count(base.begin(),base.end(),':');
  for(auto& c : base){if(c<'0'||c>'9'){c=' ';}}
  istringstream istr(base);
  istr >> minutes;
  if(x>0){istr >> seconds;}
  int ibase=(minutes*60+seconds);
  
  ostringstream os2;
  os2 << mps << " " << ibase << " " << inc;
  com.to_engine("level "+os2.str());

  if(get<0>(tmcontrol)>0){
    std::cerr << "not implemented" << endl;
    ostringstream os3;
    os3 << "nps " << get<0>(tmcontrol) << endl;
    com.to_engine(os3.str());
  }
}

void Interface::on_menu_engine1_options()
{
  auto mop=master.get_options_engine1();
  vector<Option> x;
  for(auto m : mop){x.push_back(m.second);}
  vector<Option> y{x};
  OptionsDialog optionsdialog("Set Options",*this,x);
  optionsdialog.run();
  for(size_t i=0;i<x.size();i++){
    if(x[i].name()==y[i].name() && x[i].value()==y[i].value()){continue;}
    ostringstream oss;
    oss << "setoption_engine1 name " << x[i].name() << " value " << x[i].value() << endl;
    com.to_engine(oss.str());
  }
}

void Interface::on_menu_engine2_options()
{
  auto mop=master.get_options_engine2();
  vector<Option> x;
  for(auto m : mop){x.push_back(m.second);}
  vector<Option> y{x};
  OptionsDialog optionsdialog("Set Options",*this,x);
  optionsdialog.run();
  for(size_t i=0;i<x.size();i++){
    if(x[i].name()==y[i].name() && x[i].value()==y[i].value()){continue;}
    ostringstream oss;
    oss << "setoption_engine2 name " << x[i].name() << " value " << x[i].value() << endl;
    com.to_engine(oss.str());
  }
}

void Interface::on_menu_ponder()
{
  if(m_ponder->get_active()){
    com.to_engine("ponder on");
  }else{
    com.to_engine("ponder off");
  }
}

void Interface::on_menu_debug()
{
  m_debug=!m_debug;
  ostringstream oss;
  oss << "debug ";
  oss << boolalpha << m_debug << endl;
  com.to_engine(oss.str());  
}

void Interface::on_menu_flip()
{
  m_board.flip();
  draw_board();
}

void Interface::on_menu_show_output()
{
  m_show_engine_output=!m_show_engine_output;
  if(m_show_engine_output){
    m_analysis_window_engine1.show();
    m_analysis_window_engine2.show();
  }
  else{
    m_analysis_window_engine1.hide();
    m_analysis_window_engine2.hide();
  }

}

void Interface::on_menu_show_pgnviewer()
{
  m_show_pgnview=!m_show_pgnview;
  if(m_show_pgnview){m_pgnview.show();}else{m_pgnview.hide();}
}

void Interface::on_menu_show_commenteditor()
{
  m_show_commenteditor=!m_show_commenteditor;
  if(m_show_commenteditor){m_commenteditor.show();}else{m_commenteditor.hide();}
}

void Interface::on_menu_show_epd_commands()
{
  m_show_epd_commands=!m_show_epd_commands;
  if(m_show_epd_commands){
    m_epd_commands_window.show();
  }else{
    m_epd_commands_window.hide();
  }
}

void Interface::on_menu_engine1()
{
  string x=file_chooser(OpenExe);
  if(x!=""){m_path_engine1=x;}else{return;}
  string cmd="load 1 ";cmd+=m_path_engine1;
  com.to_engine(cmd);
  m_ponder->set_active(false);
  return;
}

void Interface::on_menu_engine2()
{
  string x=file_chooser(OpenExe);
  if(x!=""){m_path_engine2=x;}else{return;}
  string cmd="load 2 ";cmd+=m_path_engine2;
  com.to_engine(cmd);
  m_ponder->set_active(false);
  return;
}

void Interface::on_menu_about()
{
  Gtk::AboutDialog aboutdialog;
  aboutdialog.set_transient_for(*this);
  aboutdialog.set_program_name("Galjoen");
  aboutdialog.set_version("0.39");
  aboutdialog.set_license_type(Gtk::License::LICENSE_GPL_3_0_ONLY);
  aboutdialog.set_website("http://www.goudengaljoen.be");
  std::vector<Glib::ustring> author{"Werner Taelemans"};
  aboutdialog.set_authors(author);
  string logo_file=m_prefix+"galjoen.files/png/galjoen_100x50.png";
  auto logo = Gdk::Pixbuf::create_from_file(logo_file);
  aboutdialog.set_logo(logo);
  aboutdialog.run();
}


string Interface::file_chooser(const char code)
{
  auto action=Gtk::FILE_CHOOSER_ACTION_OPEN;
  if(code==SaveFen || code==SavePgn){
    action=Gtk::FILE_CHOOSER_ACTION_SAVE;
  }

  Gtk::FileChooserDialog dialog("Choose File",action);
  dialog.set_transient_for(*this);

  //Add response buttons the the dialog:
  dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  dialog.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_OK);

  //Add filters, so that only certain file types can be selected:
  if(code==OpenFen || code==SaveFen){
    Glib::RefPtr<Gtk::FileFilter> filter_fen = Gtk::FileFilter::create();
    filter_fen->set_name("FEN files");
    filter_fen->add_pattern("*.fen");
    filter_fen->add_pattern("*.epd");
    dialog.add_filter(filter_fen);
  }
  else if(code==OpenPgnEpd){
    Glib::RefPtr<Gtk::FileFilter> filter_pgn = Gtk::FileFilter::create();
    filter_pgn->set_name("PGN/EPD files");
    filter_pgn->add_pattern("*.pgn");
    filter_pgn->add_pattern("*.epd");
    dialog.add_filter(filter_pgn);
  }
  else if(code==OpenPgn || code==SavePgn){
    Glib::RefPtr<Gtk::FileFilter> filter_pgn = Gtk::FileFilter::create();
    filter_pgn->set_name("PGN files");
    filter_pgn->add_pattern("*.pgn");
    dialog.add_filter(filter_pgn);
  }
  else if(code==OpenExe){
    Glib::RefPtr<Gtk::FileFilter> filter_pgn = Gtk::FileFilter::create();
    filter_pgn->set_name("engine");
    filter_pgn->add_mime_type("application/octet-stream");
    dialog.add_filter(filter_pgn);
  }
  Glib::RefPtr<Gtk::FileFilter> filter_text = Gtk::FileFilter::create();
  filter_text->set_name("Text files");
  filter_text->add_mime_type("text/plain");
  dialog.add_filter(filter_text);
  Glib::RefPtr<Gtk::FileFilter> filter_any = Gtk::FileFilter::create();
  filter_any->set_name("Any files");
  filter_any->add_pattern("*");
  dialog.add_filter(filter_any);

  //Show the dialog and wait for a user response:
  int result = dialog.run();

  //Handle the response:
  switch(result)
    {
    case(Gtk::RESPONSE_OK):
      {
	std::string filename = dialog.get_filename();
	return filename;
      }
    case(Gtk::RESPONSE_CANCEL):
      {
	return "";
      }
    default:
      {
	return "";
      }
    }
}


tuple<int,int,string,float> Interface::time_control()
{
  Gtk::Dialog dialog("Time control",*this,true);
  //dialog.set_transient_for(*this);
  dialog.set_border_width(20);

  //Add response buttons the the dialog:
  dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  dialog.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);

  //Prepare content area:
  Gtk::Box* pcontent=dialog.get_content_area();
  //Gtk::Box  line_nps(Gtk::ORIENTATION_HORIZONTAL);
  Gtk::Box  line_mps(Gtk::ORIENTATION_HORIZONTAL);
  Gtk::Box  line_base(Gtk::ORIENTATION_HORIZONTAL);
  Gtk::Box  line_inc(Gtk::ORIENTATION_HORIZONTAL);
  Gtk::Label label1("NPS: "),label2("Moves per session: "),
    label3("Base (mm:ss): "),label4("Increment (sec): ");

  //Gtk::CheckButton nps_button("Fixed NPS");

  string mps,base,inc;
  istringstream iss(m_level);
  iss >> mps >> base >> inc;
  //Gtk::Entry ent1;
  //ent1.set_width_chars(10);
  //ent1.set_alignment(1);
  Gtk::Entry ent2;
  ent2.set_width_chars(10);
  ent2.set_alignment(1);
  ent2.set_text(mps);
  Gtk::Entry ent3;
  ent3.set_width_chars(10);
  ent3.set_alignment(1);
  ent3.set_text(base);
  Gtk::Entry ent4;
  ent4.set_width_chars(10);
  ent4.set_alignment(1);
  ent4.set_text(inc);

  //line_nps.pack_start(nps_button);
  //line_nps.pack_start(label1);
  //line_nps.pack_start(ent1);
  line_mps.pack_start(label2);
  line_mps.pack_start(ent2);
  line_base.pack_start(label3);
  line_base.pack_start(ent3);
  line_inc.pack_start(label4);
  line_inc.pack_start(ent4);
  //pcontent->pack_start(line_nps);
  pcontent->pack_start(line_mps);
  pcontent->pack_start(line_base);
  pcontent->pack_start(line_inc);
  pcontent->show_all_children();

  
  //Show the dialog and wait for a user response:
  int result = dialog.run();

  //Handle the response:
  switch(result)
    {
    case(Gtk::RESPONSE_OK):
      {
	//string t1=ent1.get_text();
	string t2=ent2.get_text();
	string t3=ent3.get_text();
	string t4=ent4.get_text();
	std::vector<float> v{0,0,0,0};
	//v[0]=atoi(t1.c_str());
	v[1]=atoi(t2.c_str());
	//v[2]=atoi(t3.c_str());
	v[3]=atof(t4.c_str());
	//if(!nps_button.get_active()){v[0]=-1;}
	tuple<int,int,string,float> t{v[0],v[1],t3,v[3]};
	return t;
      }
    case(Gtk::RESPONSE_CANCEL):
      {
	tuple<int,int,string,float> v{0,atoi(mps.c_str()),base.c_str(),atof(inc.c_str())};
	return v;
      }
    default:
      {
	tuple<int,int,string,float> v{0,0,"0:00",0};
	return v;
      }
    }
}

void Interface::draw_board()
{
  vector<char> b(64,'?');
  Piece p;
  for(Square sq=a1;sq<=h8;sq++){
    p=cp.get_piece_from(sq);
    switch(p.id())
      {
      case no_piece_id:b[sq.id()]='?';break;
      case white_pawn_id:b[sq.id()]='P';break;
      case white_knight_id:b[sq.id()]='N';break;
      case white_bishop_id:b[sq.id()]='B';break;
      case white_rook_id:b[sq.id()]='R';break;
      case white_queen_id:b[sq.id()]='Q';break;
      case white_king_id:b[sq.id()]='K';break;
      case black_pawn_id:b[sq.id()]='p';break;
      case black_knight_id:b[sq.id()]='n';break;
      case black_bishop_id:b[sq.id()]='b';break;
      case black_rook_id:b[sq.id()]='r';break;
      case black_queen_id:b[sq.id()]='q';break;
      case black_king_id:b[sq.id()]='k';break;
      }
  }
  m_board.initialize(b);
}

void Interface::telluser(string s1,string s2)
{
  for(auto& c: s1){if(c=='_'){c=' ';}}
  Gtk::MessageDialog dialog(*this, s1.c_str());
  dialog.set_title("Message:");
  dialog.set_secondary_text(s2);
  dialog.run();
}
