/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"master.h"
#include<string>
#include<iostream>
#include<iomanip>
#include<fstream>
#include<sstream>
#include<stdexcept>
#include"game.h"
using std::istringstream;
using std::ostringstream;
using std::ofstream;
using std::logic_error;
using std::string;
using std::to_string;
using std::cout;
using std::endl;
using std::flush;
using std::boolalpha;
using std::ios_base;


Master::Master(Communicator& cc, string gui_name) : m_com_interface(cc)
{
  m_quit=false;
  m_guiname=gui_name;
  load_galjoen();  
  m_log=false;
  m_bookname="";
  m_number_of_rounds=1;
  m_current_round=0;
  m_status=MasterStatus::human_computer;
}

Master::~Master()
{
  m_lle1.kill();
  m_lle2.kill();
  if(m_log){m_ofs.close();}
}

//------------------------ process commands ------------------------------------
//
void Master::operator()()
{
  string s0,s1,s2;
  while(!m_quit){
    process_command_from_interface();
    process_answer_from_engine<1>();
    master_directions();
    process_answer_from_engine<2>();
    master_directions();
    std::this_thread::sleep_for(milliseconds(5));
  }
}

void Master::process_command_from_interface()
{
  string x=m_com_interface.from_interface();
  if(x.empty()){return;}
  m_ofs << "interface -> " << x << endl;
  string command;
  istringstream iss(x);
  iss >> command;

  if(command=="load"){
    int eng_id;iss >> eng_id;
    string name;iss >> name;
    load_engine(eng_id,name);
    newgame();
  }
  else if(command=="book"){
    iss >> m_bookname;
    m_book.open_input(m_bookname);
    if(!m_book.good()){m_bookname="";}
  }
  else if(command=="debug"){
    bool debug;
    iss >> boolalpha >> debug;
    set_log(debug);
  }
  else if(command=="accept_engine_time"){
    bool b;
    iss >> boolalpha >> b;
    m_accept_engine_time=b;
  }
  else if(command=="match"){
    m_status=MasterStatus::match;
    m_match_status=MatchStatus::computer_computer;
    m_game_status=GameStatus::idle;
    iss >> m_number_of_rounds;

    string s1{"whitename "};
    s1+=m_peng1->myname();
    m_com_interface.to_interface(s1);
    s1="blackname ";
    s1+=m_peng2->myname();
    m_com_interface.to_interface(s1);

    m_draws=m_engine1_wins=m_engine2_wins=0;
    m_com_interface.to_interface("new");
    m_current_round=0;
    setup_bookposition();
  }
  else if(command=="setoption_engine1"){
    string s1,s2,s;
    iss >> s;
    if(s=="name"){
      iss >> s;
      while(s!="value" && !iss.eof()){
	s1+=s+" ";
	iss >> s;
      }
      while(s1.back()==' '){s1.pop_back();}
      if(s=="value"){
	iss >> s;
	while(!iss.eof()){
	  s2+=s+" ";
	  iss >> s;
	}
	while(s2.back()==' '){s2.pop_back();}
      }
      Map x=m_peng1->get_options();
      if(!s1.empty()){
	x[s1].set_value(s2);
	m_peng1->set_options(x);
      }
    }
    return;
  }
  else if(command=="setoption_engine2"){
    string s1,s2,s;
    iss >> s;
    if(s=="name"){
      iss >> s;
      while(s!="value" && !iss.eof()){
	s1+=s+" ";
	iss >> s;
      }
      while(s1.back()==' '){s1.pop_back();}
      if(s=="value"){
	iss >> s;
	while(!iss.eof()){
	  s2+=s+" ";
	  iss >> s;
	}
	while(s2.back()==' '){s2.pop_back();}
      }
      Map x=m_peng2->get_options();
      if(!s1.empty()){
	x[s1].set_value(s2);
	m_peng2->set_options(x);
      }
    }
    return;
  }
  else if(command=="new"){
    newgame();
    m_status=MasterStatus::human_computer;
    m_match_status=MatchStatus::human_computer;
    m_game_status=GameStatus::human_to_move;
    m_com_interface.to_interface("new");
    m_peng1->set_level(m_level);
    m_peng2->set_level(m_level);
    m_peng1->newgame();
    m_peng2->newgame();
    
    if(m_peng1->get_options().find("UCI_Chess960")!=m_peng1->get_options().end()){
      if(m_peng1->get_options()["UCI_Chess960"].value()=="true"){
	m_cp.newgame960();
	m_com_interface.to_interface("setboard "+m_cp.fen());	
      }
    }
    return;
  }
  else if(command=="quit"){
    m_com_interface.to_interface("quit");
    m_quit=true;
    return;
  }
  else if(command=="force"){
    m_peng1->stop();
    m_peng2->stop();
    m_status=MasterStatus::human_human;
    m_match_status=MatchStatus::idle;
    m_game_status=GameStatus::idle;
    return;
  }
  else if(command=="playother"){
    m_status=MasterStatus::human_computer;
    m_game_status=GameStatus::human_to_move;
    m_unprocessed_move=NullMove();
    if(m_cp.white_to_move()){m_peng1->set_white(false);}else{m_peng1->set_white(true);}
    return;
  }
  else if(command=="go"){
    m_status=MasterStatus::human_computer;
    m_game_status=GameStatus::engine1_to_move;
    m_unprocessed_move=NullMove();
    if(m_cp.white_to_move()){m_peng1->set_white(true);}else{m_peng1->set_white(false);}
    m_peng1->position(m_cp);
    m_peng1->go();
    return;
  }
  else if(command=="analyzeposition"){
    m_status=MasterStatus::human_computer;
    m_unprocessed_move=NullMove();
    if(m_cp.white_to_move()){m_peng1->set_white(true);}else{m_peng1->set_white(false);}
    m_peng1->position(m_cp);
    m_peng1->go_infinite();
    return;
  }
  else if(command=="level"){
    int mps,base;
    float inc;
    iss >> mps >> base >> inc;
    m_level.set_mps(mps);
    Duration d;
    d.seconds(base);
    m_level.set_base(d);
    d.seconds(inc);
    m_level.set_inc(d);
    m_peng1->set_level(m_level);
    m_peng2->set_level(m_level);
    
    //-----------------------------------------------
    int wtime=m_peng1->get_wtime().milliseconds();
    string wt="wtime "+to_string(wtime);
    m_com_interface.to_interface(wt);
    int btime=m_peng1->get_btime().milliseconds();
    string bt="btime "+to_string(btime);
    m_com_interface.to_interface(bt);
    //-----------------------------------------------
    
    return;
  }
  else if(command=="usermove"){
    string s;iss >> s;
    Move m=alg2move(s);
    if(m_status==MasterStatus::match){}
    else if(m_status==MasterStatus::human_computer){
      m_game_status=GameStatus::human_to_move;
      m_unprocessed_move=m;
      if(m_cp.white_to_move()){m_peng1->set_white(false);}else{m_peng1->set_white(true);}
    }
    else if(m_status==MasterStatus::human_human){
      m_game_status=GameStatus::human_to_move;
      m_unprocessed_move=m;
      if(m_cp.white_to_move()){m_peng1->set_white(false);}else{m_peng1->set_white(true);}
    }
    return;
  }
  else if(command=="setboard"){
    m_peng1->stop();
    m_peng2->stop();
    string s1;
    getline(iss,s1);
    m_cp.fen(s1);
    return;
  }
  else if(command=="ponder"){
    string ponder;iss >> ponder;
    if(ponder=="on"){
      m_ponder=true;
      m_peng1->set_ponder(true);
      m_peng2->set_ponder(true);
    }else{
      m_ponder=false;
      m_peng1->set_ponder(false);
      m_peng2->set_ponder(false);
    }
    
    return;
  }
}

void Master::setup_bookposition()
{
  newgame();
  m_unprocessed_move=NullMove();
  if((m_current_round&0x01)==0){
    m_peng1->set_white(true);m_peng2->set_white(false);
  }else{
    m_peng1->set_white(false);m_peng2->set_white(true);
  }
  
  //-------------------------------------------------------
  //   play without book:
  if(m_bookname==""){
    if(m_peng1->white()){
      m_peng1->position(m_cp);
      m_peng1->go();
      m_game_status=GameStatus::engine1_to_move;
    }
    else{
      m_peng2->position(m_cp);
      m_peng2->go();
      m_game_status=GameStatus::engine2_to_move;
    }
    return;
  }
  //-------------------------------------------------------  

  //-------------------------------------------------------
  //   book is *.epd file:
  if(m_bookname.back()=='d' && m_bookname[m_bookname.size()-2]=='p'
     && m_bookname[m_bookname.size()-3]=='e' && m_bookname[m_bookname.size()-4]=='.'){
    ifstream ifs(m_bookname);
    string epdline;
    for(int i=0;i<=m_current_round/2;i++){
      getline(ifs,epdline);if(ifs.eof()){throw logic_error{"Master::setup_bookposition   reached end of file"};}
    }
    ifs.close();
    m_cp.fen(epdline);
    m_com_interface.to_interface("setboard "+m_cp.fen());
    if((m_current_round&0x01)==0){
      m_peng1->set_white(true);m_peng2->set_white(false);
    }else{
      m_peng1->set_white(false);m_peng2->set_white(true);
    }
    if((m_peng1->white() && m_cp.white_to_move()) || (!m_peng1->white() && !m_cp.white_to_move())){
      m_peng1->position(m_cp);
      m_peng1->go();
      m_game_status=GameStatus::engine1_to_move;
    }
    else{
      m_peng2->position(m_cp);
      m_peng2->go();
      m_game_status=GameStatus::engine2_to_move;
    }
    return;
  }
  //-------------------------------------------------------


  if((m_current_round&0x01)==0){
    m_current_opening=m_book.get_next_game();
    if(m_current_opening==""){m_current_opening=m_book.get_first_game();}
  }
  Game game(m_current_opening);
  string s=game.get_first_position();
  if(s==""){
    string s="telluser invalid pgn-book";
    m_com_interface.to_interface(s);
    return;
  }
  while(s!=""){
    string x;
    Move m;
    try{
      m=game.get_move();
    }catch(...){}
    if(m.code()==nullmove){break;}
    string y=m.algebraic();x="move ";x+=y;
    m_com_interface.to_interface(x);
    
    if(m_cp.white_to_move()){
      if(m_peng1->white()){m_peng1->move_and_stop_clock();}else{m_peng2->move_and_stop_clock();}
    }else{
      if(m_peng1->white()){m_peng2->move_and_stop_clock();}else{m_peng1->move_and_stop_clock();}
    }
    
    if(!m_cp.is_legal(m)){m_ofs << "Master::setup_bookposition   illegal move: " << m.dump() << endl;break;}
    m_cp.make_move(m);
    s=game.get_next_position();
  }
  if(m_cp.white_to_move()){
    if(m_peng1->white()){
      m_peng1->position(m_cp);
      m_peng1->go();
      m_game_status=GameStatus::engine1_to_move;
    }else{
      m_peng2->position(m_cp);
      m_peng2->go();
      m_game_status=GameStatus::engine2_to_move;
    }
  }
  else{
    if(m_peng1->white()){
      m_peng2->position(m_cp);
      m_peng2->go();
      m_game_status=GameStatus::engine2_to_move;
    }else{
      m_peng1->position(m_cp);
      m_peng1->go();
      m_game_status=GameStatus::engine1_to_move;
    }
  }
  m_unprocessed_move=NullMove();
}

Map Master::get_options_engine1()
{
  return m_peng1->get_options();
}

Map Master::get_options_engine2()
{
  return m_peng2->get_options();
}

void Master::set_log(bool x)
{
  if(x && !m_log){
    string filename="master.log";
    m_ofs.open(filename,ios_base::ate | ios_base::out);
  }
  else if(!x && m_log){
    m_ofs.close();
  }
  m_log=x;
  m_peng1->set_log(x);
  m_peng2->set_log(x);
}

void Master::newgame()
{
  m_peng1->newgame();
  m_peng2->newgame();
  m_cp.newgame();
}

void Master::load_galjoen()
{
  string prefix,engine_name;
  ifstream ifs;
  if(m_guiname.find(".exe")!=string::npos){
    engine_name="galjoen_eng.exe";
  }else{
    engine_name="galjoen_eng";
  }
  ifs.open(engine_name);
  if(ifs.good()){
    prefix.clear();
  }else{
    ifs.clear();
    ifs.open("/usr/local/games/"+engine_name);
    if(ifs.good()){
      prefix="/usr/local/games/";
    }else{
      throw logic_error("Error: missing engine");
    }
  }
  string name_engine{prefix+engine_name};
  m_lle1.load(1,name_engine);
  m_lle2.load(2,name_engine);
  if(m_lle1.uci()){
    m_uci_eng1.connect(1,m_lle1);
    m_peng1=&m_uci_eng1;
    m_uci_eng2.connect(2,m_lle2);
    m_peng2=&m_uci_eng2;
  }else{
    m_cecp_eng1.connect(1,m_lle1);
    m_peng1=&m_cecp_eng1;
    m_cecp_eng2.connect(2,m_lle2);
    m_peng2=&m_cecp_eng2;
  }
  m_peng1->set_opponent(m_peng2);
  m_peng2->set_opponent(m_peng1);
  m_peng1->set_logstream(&m_ofs);
  m_peng2->set_logstream(&m_ofs);
  Level level;
  level.set_mps(40);
  Duration d;
  d.minutes(1);
  level.set_base(d);
  d.seconds(0);
  level.set_inc(d);
  m_peng1->set_level(level);
  m_peng2->set_level(level);
  m_peng1->set_ponder(false);
  m_peng2->set_ponder(false);
  string s1{"engine1name "};
  s1+=m_peng1->myname();
  m_com_interface.to_interface(s1);
  string s2{"engine2name "};
  s2+=m_peng2->myname();
  m_com_interface.to_interface(s2);
  newgame();
}

void Master::load_engine(int eng_id,string name)
{
  bool uci{false};
  {
    LowLevelEngine lle;
    lle.load(eng_id,name);
    if(!lle.good()){return;}
    uci=lle.uci();
    lle.kill();
  }

  string s=eng_id==1?"engine1name ":"engine2name ";
  if(eng_id==1){
    m_lle1.kill();
    m_lle1.load(eng_id,name);
    if(uci){
      m_uci_eng1.connect(eng_id,m_lle1);
      m_peng1=&m_uci_eng1;
    }else{
      m_cecp_eng1.connect(eng_id,m_lle1);
      m_peng1=&m_cecp_eng1;
    }
    s+=m_peng1->myname();
    m_com_interface.to_interface(s);
    m_peng1->set_log(m_log);
    m_peng1->set_ponder(m_ponder);
  }else{
    m_lle2.kill();
    m_lle2.load(eng_id,name);
    if(uci){
      m_uci_eng2.connect(eng_id,m_lle2);
      m_peng2=&m_uci_eng2;
    }else{
      m_cecp_eng2.connect(eng_id,m_lle2);
      m_peng2=&m_cecp_eng2;
    }
    s+=m_peng2->myname();
    m_com_interface.to_interface(s);
    m_peng2->set_log(m_log);
    m_peng2->set_ponder(m_ponder);
  }
  m_peng1->set_opponent(m_peng2);
  m_peng2->set_opponent(m_peng1);
  m_peng1->set_logstream(&m_ofs);
  m_peng2->set_logstream(&m_ofs);
  m_peng1->set_level(m_level);
  m_peng2->set_level(m_level);
}

void Master::reload_engine(int eng_id)
{
  if(eng_id==1){
    m_lle1.reload();
  }else{
    m_lle2.reload();
  }
  newgame();
}

Move Master::alg2move(string s)
{
  Move m=m_cp.convert_to_move(s);
  return m;
}

template<int I>
void Master::process_answer_from_engine()
{
  auto& peng=I==1?m_peng1:m_peng2;  
  string x=peng->get_answer();
  if(x.empty()){return;}
  m_ofs << "engine" << I << " -> " << x << endl;
  istringstream iss(x);
  string command;iss >> command;
  if(command=="bestmove"){
    if(m_game_status==GameStatus::idle){return;}
    string s;iss >> s;
    if(s=="0000"){return;}
    m_unprocessed_move=alg2move(s);
    if(!m_cp.is_legal(m_unprocessed_move)){
      s=peng->white()?"0-1":"1-0";
      s+=" {illegal move: "+ m_unprocessed_move.algebraic() +"}";
      end_of_game_processing(s);
      m_unprocessed_move=NullMove();
    }
  }
  else if(command=="resign"){
    string s;iss >> s;
    s=peng->white()?"0-1 {white resigns}":"1-0 {black resigns}";
    end_of_game_processing(s);
    m_unprocessed_move=NullMove();    
  }
  else if(command=="info"){
    string s;
    getline(iss,s);
    send_info<I>(s);
  }
}


template<int I>
void Master::send_info(string x)
{
  auto& ev=I==1?m_peng1:m_peng2;  
  string info;
  if(ev->white()){info="white_info ";}else{info="black_info ";}
  m_com_interface.to_interface(info + x);    
}

void Master::master_directions()
{
  switch(m_status){
  case MasterStatus::human_computer:
    //
    if(!m_lle1.good()){
      string s="telluser engine1 crashed";
      m_com_interface.to_interface(s);
      reload_engine(1);
      m_unprocessed_move.clear();
    }
    //
    if(m_game_status==GameStatus::human_to_move && m_unprocessed_move.code()!=nullmove){
      m_cp.make_move(m_unprocessed_move);
      m_unprocessed_move=NullMove();
      m_peng1->position(m_cp);
      m_ofs << "engine1 <- position " << m_cp.fen() << endl;
      m_peng1->go();
      m_game_status=GameStatus::engine1_to_move;
      if(m_cp.result()!="*"){end_of_game_processing();break;}
    }
    else if(m_game_status==GameStatus::engine1_to_move && m_unprocessed_move.code()!=nullmove){
      string answer="move "+m_unprocessed_move.algebraic();
      m_com_interface.to_interface(answer);
      
      //----------------------------------------
      if(m_peng1->white()){
	int wtime=m_peng1->get_wtime().milliseconds();
	answer="wtime "+to_string(wtime);
      }else{
	int btime=m_peng1->get_btime().milliseconds();
	answer="btime "+to_string(btime);
      }
      m_com_interface.to_interface(answer);
      std::this_thread::sleep_for(std::chrono::milliseconds(100));
      m_com_interface.to_interface("shout");
      //----------------------------------------      
      
      m_cp.make_move(m_unprocessed_move);
      m_unprocessed_move=NullMove();
      if(m_cp.result()!="*"){end_of_game_processing();break;}
      m_game_status=GameStatus::human_to_move;
    }
    break;
  case MasterStatus::match:
    if(m_game_status==GameStatus::engine1_to_move && m_unprocessed_move.code()!=nullmove){
      string answer="move "+m_unprocessed_move.algebraic();
      m_com_interface.to_interface(answer);
      m_ofs << "interface <- " << answer << endl;
      int wtime=m_peng1->get_wtime().milliseconds();
      answer="wtime "+to_string(wtime);
      m_com_interface.to_interface(answer);
      int btime=m_peng1->get_btime().milliseconds();
      answer="btime "+to_string(btime);
      m_com_interface.to_interface(answer);
      try{
      m_cp.make_move(m_unprocessed_move);
      }
      catch(...){
	std::cerr << "Master:  MasterStatus::match   engine1 to move" << endl;
	throw;
      }
      if(m_cp.result()!="*"){end_of_game_processing();break;}
      m_unprocessed_move=NullMove();
      m_peng2->position(m_cp);
      m_ofs << "engine2 <- position " << m_cp.fen() << endl;
      m_peng2->go();
      m_ofs << "engine2 <- go" << endl;
      m_game_status=GameStatus::engine2_to_move;
    }
    else if(m_game_status==GameStatus::engine2_to_move && m_unprocessed_move.code()!=nullmove){
      string answer="move "+m_unprocessed_move.algebraic();
      m_com_interface.to_interface(answer);
      m_ofs << "interface <- " << answer << endl;
      int wtime=m_peng2->get_wtime().milliseconds();
      answer="wtime "+to_string(wtime);
      m_com_interface.to_interface(answer);
      int btime=m_peng2->get_btime().milliseconds();
      answer="btime "+to_string(btime);
      m_com_interface.to_interface(answer);
      try{
      m_cp.make_move(m_unprocessed_move);
      }
      catch(...){
	std::cerr << "Master:  MasterStatus::match   engine2 to move" << endl;
	throw;
      }
      if(m_cp.result()!="*"){end_of_game_processing();break;}
      m_unprocessed_move=NullMove();
      m_peng1->position(m_cp);
      m_ofs << "engine1 <- position " << m_cp.fen() << endl;
      m_peng1->go();
      m_ofs << "engine1 <- go" << endl;
      m_game_status=GameStatus::engine1_to_move;
    }
    else if(m_game_status==GameStatus::engine1_to_move && m_peng1->get_time().seconds()<0 && !m_accept_engine_time){
      m_peng1->stop();
      if(m_peng1->white()){end_of_game_processing("0-1 {time}");}else{end_of_game_processing("1-0 {time}");}
    }
    else if(m_game_status==GameStatus::engine2_to_move && m_peng2->get_time().seconds()<0 && !m_accept_engine_time){
      m_peng2->stop();
      if(m_peng2->white()){end_of_game_processing("0-1 {time}");}else{end_of_game_processing("1-0 {time}");}
    }
    else {
      m_unprocessed_move=NullMove();
    }
    break;
  case MasterStatus::human_human:
    if(m_game_status==GameStatus::human_to_move && m_unprocessed_move.code()!=nullmove){
      m_cp.make_move(m_unprocessed_move);
      m_unprocessed_move=NullMove();
      m_game_status=GameStatus::human_to_move;
      if(m_cp.result()!="*"){end_of_game_processing();}
    }
    break;
  case MasterStatus::swiss:break;
  case MasterStatus::round_robin:break;
  case MasterStatus::keyzer:break;
  case MasterStatus::gauntlet:break;
  }
}

void Master::end_of_game_processing(string x)
{
  m_game_status=GameStatus::idle;
  string r=m_cp.result();
  if(!x.empty()){r=x;}
  m_com_interface.to_interface(r);
  m_peng1->stop();
  m_peng2->stop();

  istringstream iss(r);
  string result;iss >> result;
  if(m_status==MasterStatus::human_computer){
    string s="telluser result: ";s+=result;
    m_com_interface.to_interface(s);
    return;
  }
  
  if(m_status==MasterStatus::match){
    if(result=="1/2-1/2"){
      m_draws++;
    }
    else if(result=="1-0"){
      if(m_peng1->white()){m_engine1_wins++;}
      else{m_engine2_wins++;}
    }
    else if(result=="0-1"){
      if(m_peng1->white()){m_engine2_wins++;}
      else{m_engine1_wins++;}
    }
    
    m_current_round++;
    if(m_current_round<m_number_of_rounds){
      m_ofs << "master -> newgame " << m_current_round+1 << endl;
      m_com_interface.to_interface("new");
      setup_bookposition();
      m_match_status=MatchStatus::computer_computer;

      if(m_peng1->white()){
	string s1{"whitename "};
	s1+=m_peng1->myname();
	m_com_interface.to_interface(s1);
	s1="blackname ";
	s1+=m_peng2->myname();
	m_com_interface.to_interface(s1);
      }
      else{
	string s1{"blackname "};
	s1+=m_peng1->myname();
	m_com_interface.to_interface(s1);
	s1="whitename ";
	s1+=m_peng2->myname();
	m_com_interface.to_interface(s1);
      }
    }
    else{
      m_match_status=MatchStatus::idle;
      string s1=m_peng1->myname();
      string s2=m_peng2->myname();
      s1+=" - "+s2;
      for(auto& c : s1){if(c==' '){c='_';}}
      ostringstream oss2;
      oss2 << m_engine1_wins << "-" << m_engine2_wins << "-" << m_draws;
      string s3="telluser ";s3+=s1;s3+=" ";s3+=oss2.str();
      m_com_interface.to_interface(s3);
    }     
  }
}

void Master::dump()
{
  string r=m_cp.result();
  m_com_interface.to_interface(r);

  m_ofs << endl << "------------------------ master: dump -------------------------" << endl << endl;
  m_ofs << "movecounter=" << m_cp.get_make_move_counter() << endl;
  m_ofs << "history: ";
  for(auto m : m_cp.history()){m_ofs << m.last_move.algebraic() << " ";}m_ofs << endl;
  m_ofs << "result: " << r << endl << endl;

  m_ofs << "------------- ChessPosition dump: -----------------------" << endl;
  m_ofs << m_cp.dump() << endl;


  //------------- quit: ------------------------------------
  m_lle1.kill();
  m_lle2.kill();
  m_com_interface.to_interface("quit");
  m_quit=true;
}
