/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#ifndef GTKMM_OPTIONSDIALOG_H
#define GTKMM_OPTIONSDIALOG_H

#include<gtkmm/grid.h>
#include<gtkmm/box.h>
#include<gtkmm/dialog.h>
#include<gtkmm/checkbutton.h>
#include<gtkmm/spinbutton.h>
#include<gtkmm/button.h>
#include<gtkmm/entry.h>
#include<gtkmm/comboboxtext.h>
#include"option.h"
#include<iostream>
#include<string>
using std::to_string;



class OptionCheckButton : public Gtk::CheckButton
{
 private:
  Option* opt;
  
 public:
  void set_opt(Option* x){opt=x;}
  void on_toggled(){
    opt->set_changed(true);
    opt->set_value(get_active()?"true":"false");
  }
};


class OptionButton : public Gtk::Button
{
 private:
  Option* opt;
  
 public:
  void set_opt(Option* x){opt=x;}
  void on_clicked(){opt->set_changed(true);}
};

class OptionSpinButton : public Gtk::SpinButton
{
 private:
  Option* opt;
  
 public:
  void set_opt(Option* x){opt=x;}
  void on_value_changed(){
    opt->set_changed(true);
    opt->set_value(to_string(get_value_as_int()));
  }
};

class OptionComboBoxText : public Gtk::ComboBoxText
{
 private:
  Option* opt;
  
 public:
  void set_opt(Option* x){opt=x;}
  void on_changed(){
    opt->set_changed(true);
    opt->set_value(get_active_text());
  }
};

class OptionEntry : public Gtk::Entry
{
 private:
  Option* opt;
  
 public:
  void set_opt(Option* x){opt=x;}
  void on_changed(){
    opt->set_changed(true);
    opt->set_value(get_text());
  }
};



class OptionsDialog : public Gtk::Dialog
{
 public:
  OptionsDialog(const Glib::ustring&,Gtk::Window&,vector<Option>&);
  virtual ~OptionsDialog();

 private:
  static const size_t max_options{100};

  // Signal handlers:
  
  // Widgets:
  Gtk::Box* p_content;
  Gtk::Grid m_grid;
  OptionCheckButton m_checkbutton[max_options];
  OptionSpinButton m_spinbutton[max_options];
  Gtk::Label m_label[max_options];
  OptionEntry m_entry[max_options];
  OptionButton m_button[max_options];
  OptionComboBoxText m_combobox[max_options];
  vector<Option>& m_option;
};



#endif

