/*

Copyright (C) 2018 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"pgnfile.h"
#include<iostream>
#include<sstream>
#include<string>
#include<stdexcept>
#include<utility>
using std::cout;
using std::endl;
using std::string;
using std::istringstream;
using std::ifstream;
using std::ofstream;
using std::ios_base;
using std::logic_error;



void PgnFile::open_input(string filename)
{
  if(ifs.is_open()){ifs.close();}
  ifs.open(filename,ios_base::in);
  if(ifs.good()){m_good=true;}
}

void PgnFile::open_output(string filename)
{
  if(ofs.is_open()){ofs.close();}
  ofs.open(filename,ios_base::app);
}

void PgnFile::close_input()
{
  if(ifs.is_open()){
    ifs.close();
  }
  m_good=false;
}

void PgnFile::close_output()
{
  if(ofs.is_open()){
    ofs.close();
  }
}

string PgnFile::get_first_game()
{
  string r;
  bool comment{false};
  int blankline_counter{0};
  string sline;

  ifs.seekg(0);
  while(sline.empty() && !ifs.eof()){
    getline(ifs,sline);
    remove_carriage_return(sline);
  }
  while(blankline_counter<2 && !ifs.eof()){
    r+=sline+"\n";
    for(auto c : sline){
      if(c=='{'){comment=true;}
      if(c=='}'){comment=false;}
    }
    if(sline.empty() && !comment){blankline_counter++;}
    if(blankline_counter>=2){break;}
    getline(ifs,sline);
    remove_carriage_return(sline);
  }
  
  //---------- remove BOM: -------------------------------------------------------------  
  if(r[0]==(char)0xef && r[1]==(char)0xbb && r[2]==(char)0xbf){r=r.substr(3,r.size()-3);}
  //------------------------------------------------------------------------------------  
  return r;
}

string PgnFile::get_next_game()
{
  string r;
  bool comment{false};
  int blankline_counter{0};
  string sline;
  while(sline.empty() && !ifs.eof()){
    getline(ifs,sline);
    remove_carriage_return(sline);
  }
  while(blankline_counter<2 && !ifs.eof()){
    r+=sline+"\n";
    for(auto c : sline){
      if(c=='{'){comment=true;}
      if(c=='}'){comment=false;}
    }
    if(sline.empty() && !comment){blankline_counter++;}
    if(blankline_counter>=2){break;}
    getline(ifs,sline);
    remove_carriage_return(sline);
  }
  return r;
}

void PgnFile::add_game(string game)
{
  ofs << game;
}

vector<pair<string,string>> PgnFile::get_first_tags()
{
  vector<pair<string,string>> r;
  pair<string,string> tag;
  if(ifs.eof()){return r;}
  ifs.seekg(0,ios_base::beg);
  string sline;
  while(sline.find("[")==string::npos){
    getline(ifs,sline);
    remove_carriage_return(sline);
    //---------- remove BOM: --------------------------------------------------------------------------------------  
    if(sline[0]==(char)0xef && sline[1]==(char)0xbb && sline[2]==(char)0xbf){sline=sline.substr(3,sline.size()-3);}
    //-------------------------------------------------------------------------------------------------------------  
  }
  
  while(sline.find("[")!=string::npos){
    while(sline.find("[")!=string::npos){sline.replace(sline.find("["),1,"");}
    while(sline.find("]")!=string::npos){sline.replace(sline.find("]"),1,"");}
    while(sline.find(R"(\")")!=string::npos){sline.replace(sline.find(R"(\")"),2,R"(')");}
    while(sline.find("\"")!=string::npos){sline.replace(sline.find("\""),1,"");}
    {
      istringstream istr(sline);
      istr >> tag.first;
      getline(istr,tag.second);
      while(tag.second[0]==' '){tag.second.replace(0,1,"");}
      while(tag.second[tag.second.size()-1]==' '){tag.second.replace(tag.second.size()-1,1,"");}
      if(tag.first=="Event" || tag.first=="Site" || tag.first=="Date" || tag.first=="Round"
	 || tag.first=="White" || tag.first=="Black" || tag.first=="Result"){
	r.push_back(tag);
      }
    }
    getline(ifs,sline);
    remove_carriage_return(sline);
  }
  
  return r;
}

vector<pair<string,string>> PgnFile::get_next_tags()
{
  vector<pair<string,string>> r;
  pair<string,string> tag;
  
  string sline;
  while(!ifs.eof()){
    getline(ifs,sline);
    remove_carriage_return(sline);
    if(sline.find("[")!=string::npos && sline.find("Event")!=string::npos){break;}
  }
  if(ifs.eof()){return r;}
  
  while(sline.find("[")!=string::npos){
    while(sline.find("[")!=string::npos){sline.replace(sline.find("["),1,"");}
    while(sline.find("]")!=string::npos){sline.replace(sline.find("]"),1,"");}
    while(sline.find(R"(\")")!=string::npos){sline.replace(sline.find(R"(\")"),2,R"(')");}
    while(sline.find("\"")!=string::npos){sline.replace(sline.find("\""),1,"");}
    {
      istringstream istr(sline);
      istr >> tag.first;
      getline(istr,tag.second);
      if(tag.second.size()>0){
	while(tag.second[0]==' '){tag.second.replace(0,1,"");}
	while(tag.second[tag.second.size()-1]==' '){tag.second.replace(tag.second.size()-1,1,"");}
      }
      if(tag.first=="Event" || tag.first=="Site" || tag.first=="Date" || tag.first=="Round"
	 || tag.first=="White" || tag.first=="Black" || tag.first=="Result"){
	r.push_back(tag);
      }
    }
    getline(ifs,sline);
    remove_carriage_return(sline);
  }
  
  return r;
}

bool PgnFile::good()const
{
  return m_good;
}

void PgnFile::remove_carriage_return(string& x)const
{
  while(x[x.size()-1]=='\r'){
    x.pop_back();
  }
}
